#include "conjugate_gradient.h"
#include "math.h"

int conjugate_gradient(SparseRowMat &A, DenseVect &x, DenseVect &b, double &tol, int &max_iter)
{
	if (x.n != A.n_c)
		x.resize(A.n_c);

	double resid;
	DenseVect p, z, q;
	double alpha, beta, rho, rho_1;

	A.dotVect(x, q);
	DenseVect r = b;
	r -= q;

	double normb = sqrt(b*b);
	if (normb == 0.0) 
		normb = 1;

	/*if ((resid = sqrt(r*r) / normb) <= tol) 
	{
		tol = resid;
		max_iter = 0;
		return 0;
	}*/

	for (int i = 1; i <= max_iter; i++) 
	{
		rho = r*r;

		if ((resid = sqrt(rho) / normb) <= tol) 
		{
			tol = resid;
			max_iter = i;
			return 0;     
		}

		if (i == 1)
			p = r;
		else 
		{
			beta = rho / rho_1;
			//p = r + p*beta;
			q = p*beta;
			p = r;
			p += q;
		}

		A.dotVect(p, q);
		alpha = rho / (p*q);

		x += p*alpha;
		r -= q*alpha;

		rho_1 = rho;
	}

	tol = resid;
	return 1;
}
