#pragma once

#include <vector>
#include "assert.h"

using namespace std;

struct DenseVect
{
	int n;
	vector<double> d;

	DenseVect()
	{
		resize(0);
	}
	DenseVect(int size)
	{
		resize(size);
	}

	void resize(int size)
	{
		n = size;
		d.resize(size);
		clear();
	}

	void clear()
	{
		for (int i = 0; i < n; i++)
			d[i] = 0;
	}

	double get(int i)
	{
		assert(i >= 0 && i < n);
		return d[i];
	}
	
	void set(int i, double v)
	{
		assert(i >= 0 && i < n);
		d[i] = v;
	}

	template <class T>
	double dot(T &v)
	{
		double sum = 0;
		for (int i = 0; i < n; i++)
			sum += d[i]*v.get(i);
		return sum;
	}

	template <class T>
	double operator*(T &v)
	{
		return dot(v);
	}

	DenseVect operator*(double v)
	{
		DenseVect r(n);
		for (int i = 0; i < n; i++)
			r.d[i] = d[i] * v;
		return r;
	}
	void operator*=(double v)
	{
		for (int i = 0; i < n; i++)
			d[i] *= v;
	}

	template <class T>
	DenseVect operator+(T &v)
	{
		DenseVect r(n);
		for (int i = 0; i < n; i++)
			r.d[i] = d[i] + v.get(i);
		return r;
	}
	template <class T>
	void operator+=(T &v)
	{
		for (int i = 0; i < n; i++)
			d[i] += v.get(i);
	}

	template <class T>
	DenseVect operator-(T &v)
	{
		DenseVect r(n);
		for (int i = 0; i < n; i++)
			r.d[i] = d[i] - v.get(i);
		return r;
	}
	template <class T>
	void operator-=(T &v)
	{
		for (int i = 0; i < n; i++)
			d[i] -= v.get(i);
	}
};
