#pragma once

//#define QUADRIC_BOUNDARY_DERIV_FIGURE
//#define SAVE_OBJ_AND_EXIT

#include <vector>
#include <string>
#include <list>
#include "vect.h"
#include "image.h"
#include "array2d.h"

using namespace std;

struct SDL_Surface;

enum Method
{
	points_linear,
	wachspress,
	lines_linear_old,
	lines_linear,
	lines_quad,
	mean_value,
	harmonic,
	maximum_entropy,
	from_file,

	method_num
};

enum WeightFunction
{
	length_difference,
	reciprocal_distance,
};

struct Curve
{
	bool closed;
	list<vect2f> pts;
};

struct Global
{
	Global();

	vector<vect2d> poly[2]; /// vertices of the control, deformed polys
	vector<vect2d> norms[2]; /// normals of the control, deformed polys
	vector<double> height_vals, deriv_vals; /// heights associated with the control poly vertices (for 3d surface)

	Image *img;

	SDL_Surface* screen;

	float time;
	float frame_time;
	float frame_rate() {return 1000.0 / frame_time;}

	int scr_width, scr_height;
	bool fullscreen;
    bool running; // set false to quit the game

	unsigned int sel_poly;
	unsigned int sel_vert;
	unsigned int sel_type; /// 0 == vertex, 1 == normal

	bool draw_grid;
	bool draw_image;
	bool draw_surface;
	bool draw_contours;
	bool use_basis_values; // draw a function using height_vals, deriv_vals
	float height_scale;

	Method method;
	WeightFunction weight_function;
	int order;
	int deriv;
	int alpha;
	int N;
	vector<vect2d> deformation;
	vector<double> barycoords;
	vector<double> barycoords2;
	vect3d surf(int x, int y);
	vector<vector<vect2f>> contours;
	vector<vector<vect2i>> contours_edges;
	vector<vector<Curve>> curves;

	vector<vect2i> tri_edges;
	vector<vect3i> tri_tris;
	vector<vect2d> tri_pt_pos;
	vector<double> tri_pt_barys;
	vector<double> tri_pt_barys2;
	vector<bool> tri_pt_border;

	void allocBarycoords()
	{
		barycoords.resize((N+1)*(N+1)*poly[0].size());
		barycoords2.resize((N+1)*(N+1)*norms[0].size());
		
		memset(&barycoords[0], 0, barycoords.size()*sizeof(double));
		memset(&barycoords2[0], 0, barycoords2.size()*sizeof(double));
	}
};

extern Global g;
