#include "matrixd.h"

#include <stdio.h>
//#include <ipp.h>
#include <assert.h>
#include <stdarg.h>
#include <math.h>
#include <vector>

using namespace std;

#define ROTATE(a,i,j,k,l) g=a(i,j);h=a(k,l);a(i,j)=g-s*(h+g*tau);a(k,l)=h+s*(g-h*tau);

// input matrix = u, eigen vals = d, eigen vecs = v
bool jacobi ( Matrixd &u, vector<double> &d, Matrixd &v )
{
	int n = u.cols;

	int j, iq, ip, i, k;
	double tresh, theta, tau, t, sm, s, h, g, c;
	vector<double> b(n);
	vector<double> z(n);
	Matrixd a(n,n);

	for ( i = 0; i < n; i++ )
	{
		for ( j = 0; j < n; j++ )
		{
			a (i,j) = u (i,j);
		}
	}

	for ( ip = 0; ip < n; ip++ ) 
	{
		for ( iq = 0; iq < n; iq++ )
		{
			v (ip,iq) = 0.0f;
		}
		v (ip,ip) = 1.0f;
	}

	for ( ip = 0; ip < n; ip++ )
	{
		b [ ip ] = a (ip,ip);
		d [ ip ] = b [ ip ];
		z [ ip ] = 0.0f;
	}

	// was to 50 before
	for ( i = 1; i <= n*n; i++ )
	{
		sm = 0.0f;
		for ( ip = 0; ip < n - 1; ip++ )
		{
			for ( iq = ip + 1; iq < n; iq++ )
			{
				sm += fabs ( a (ip,iq) );
			}
		}

		if ( sm == 0.0f )
		{
			// sort the stupid things and transpose

			// transpose
			for ( i = 0; i < n; i++ )
			{
				for ( j = 0; j < n; j++ )
				{
					a (i,j) = v (j,i);
				}
			}

			// stupid sort n^2... however n should always be small
			// bubble sort
			// largest first
			for ( i = 0; i < n; i++ )
			{
				for ( j = 0; j < n - i - 1; j++ )
				{
					if ( fabs ( d [ j ] ) < fabs ( d [ j + 1 ] ) )
					{
						sm = d [ j ];
						d [ j ] = d [ j + 1 ];
						d [ j + 1 ] = sm;

						for ( k = 0; k < n; k++ )
						{
							sm = a (j,k);
							a (j,k) = a (j+1,k);
							a (j+1,k) = sm;
						}
					}
				}
			}

			for ( i = 0; i < n; i++ )
			{
				for ( j = 0; j < n; j++ )
				{
					v (i,j) = a (i,j);
				}
			}

			return true;
		}

		if ( i < 4 )
		{
			tresh = 0.2f * sm / ( n * n );
		}
		else
		{
			tresh = 0.0f;
		}

		for ( ip = 0; ip < n - 1; ip++ )
		{
			for ( iq = ip + 1; iq < n; iq++ ) 
			{
				g = 100.0f * (double)fabs ( a (ip,iq) );
				if ( i > 4 && ( fabs ( d [ ip ] ) + g ) == fabs ( d [ ip ] )
					&& ( fabs ( d [ iq ] ) + g ) == fabs ( d [ iq ] ) )
				{
					a (ip,iq) = 0.0f;
				}
				else
				{
					if ( fabs ( a (ip,iq) ) > tresh )
					{
						h = d [ iq ] - d [ ip ];
						if ( ( fabs ( h ) + g ) == fabs ( h ) )
						{
							t = ( a (ip,iq) ) / h;
						}
						else
						{
							theta = 0.5f * h / ( a (ip,iq) );
							t = 1.0f / (double)( fabs ( theta ) + sqrt ( 1.0f + theta * theta ) );
							if ( theta < 0.0f ) 
							{
								t = -1.0f * t;
							}
						}

						c = 1.0f / (double)sqrt ( 1 + t * t );
						s = t * c;
						tau = s / ( 1.0f + c );
						h = t * a (ip,iq);
						z [ ip ] -= h;
						z [ iq ] += h;
						d [ ip ] -= h;
						d [ iq ] += h;
						a (ip,iq) = 0.0f;
						for ( j = 0; j <= ip - 1; j++ )
						{
							ROTATE ( a, j, ip, j, iq )
						}
						for ( j = ip + 1; j <= iq - 1; j++ )
						{
							ROTATE ( a, ip, j, j, iq )
						}
						for ( j = iq + 1; j < n; j++ )
						{
							ROTATE ( a, ip, j, iq, j )
						}
						for ( j = 0; j < n; j++ )
						{
							ROTATE ( v, j, ip, j, iq )
						}
					}
				}
			}
		}

		for ( ip = 0; ip < n; ip++ )
		{
			b [ ip ] += z [ ip ];
			d [ ip ] = b [ ip ];
			z [ ip ] = 0.0f;
		}
	}

	return false;
}

void matInverse ( Matrixd &mat, Matrixd &rvalue, double tolerance = 1e-9 )
{
	int n = mat.cols;

	// there is an implicit assumption that mat is symmetric and real
	// U and V in the SVD will then be the same matrix whose rows are the eigenvectors of mat
	// W will just be the eigenvalues of mat
	vector<double> w(n);
	Matrixd u(n);
	int i, j, k;

	jacobi( mat, w, u );

/*	if ( fabs ( w [ 0 ] ) < tolerance )
	{
		for ( i = 0; i < n; i++ )
		{
			for ( j = 0; j < n; j++ )
			{
				rvalue (i,j) = 0;
			}
		}
//		printf ( "error: largest eigenvalue is 0!\n" );
		return;
	}
	else */
	{
		for ( i = 1; i < n; i++ )
		{
//			if ( w [ i ] / w [ 0 ] < 0.02f ) // / w [ 0 ] < TOLERANCE )
			if ( fabs ( w [ i ] / w [ 0 ] ) < tolerance )
			{
					w [ i ] = 0;
			}
			else
			{
				w [ i ] = 1.0f / w [ i ];
			}
		}
		w [ 0 ] = 1.0f / w [ 0 ];
	}

	for ( i = 0; i < n; i++ )
	{
		for ( j = 0; j < n; j++ )
		{
			rvalue (i,j) = 0;
			for ( k = 0; k < n; k++ )
			{
				rvalue (i,j) += w [ k ] * u (k,i) * u (k,j);
			}
		}
	}
}


void set_matrixd(Matrixd &ret, const int r, const int c, ...)
{
	ret.resize(r,c);
	int num_args = r*c;
	va_list ap;
	va_start(ap, c);
	for (int i = 0; i < num_args; i++)
		ret.data[i] = va_arg(ap, double);
	va_end(ap);
}

Matrixd new_matrixd(const int r, const int c, ...)
{
	Matrixd ret(r,c);
	int num_args = r*c;
	va_list ap;
	va_start(ap, c);
	for (int i = 0; i < num_args; i++)
		ret.data[i] = va_arg(ap, double);
	va_end(ap);
	return ret;
}


Matrixd::Matrixd()
{
	data = 0;
	rows = cols = 0;
	data = 0;
}

Matrixd::Matrixd(const int r, const int c)
{
	data = 0;
	resize(r, c);
}

Matrixd::Matrixd(const Matrixd &m)
{
	if (m.data)
	{
		rows = m.rows;
		cols = m.cols;
		int n = rows*cols;

		data = new double [n];
		for (int i = 0; i < n; i++)
			data[i] = m.data[i];
	}
	else
	{
		data = 0;
		rows = cols = 0;
	}
}


void Matrixd::operator=(const Matrixd &m)
{
	delete[] data;

	if (m.data)
	{
		rows = m.rows;
		cols = m.cols;
		int n = rows*cols;

		data = new double [n];
		for (int i = 0; i < n; i++)
			data[i] = m.data[i];
	}
	else
	{
		data = 0;
		rows = cols = 0;
	}
}

Matrixd::~Matrixd()
{
	free();
}

void Matrixd::free()
{
	delete[] data;
	data = 0;
	rows = cols = 0;
}

void Matrixd::resize(const int r, const int c)
{
	rows = r;
	cols = (c == 0 ? r : c);
	delete[] data;
	data = new double [rows*cols];
}

void Matrixd::print() const
{
	for (int r = 0; r < rows; r++)
	{
		for (int c = 0; c < cols; c++)
		{
			printf("% 7.3f ", get(r, c));
		}
		printf("\n");
	}
}

Matrixd Matrixd::inverse() const
{
	assert(rows == cols);
	Matrixd ret(rows, cols);
	/*double *buf = new double [rows*rows + rows];
	int status = ippmInvert_m_64f(data, sizeof(double)*cols, sizeof(double), buf, ret.data, sizeof(double)*cols, sizeof(double), rows);
	delete[] buf;
	assert(status == ippStsNoErr);
	return ret;*/

	matInverse((Matrixd)*this, ret);
	return ret;
}

Matrixd Matrixd::pseudo_inverse() const
{
	////const double epsilon = 1e-9;

	//assert(rows == cols);
	//Matrixd vals, vecs, vals_diag(rows, cols);
	//eigen_decomp_sym(vals, vecs);

	//vals_diag.set_zero();
	//for (int i = 0; i < rows; i++)
	//{
	//	//if (fabs(vals[i] / vals[0]) >= epsilon)
	//		vals_diag(i,i) = 1.0 / vals[i];
	//}

	//return vecs*vals_diag*vecs.t();
	
	assert(rows == cols);
	Matrixd ret(rows, cols);
	matInverse((Matrixd)*this, ret);
	return ret;
}

void Matrixd::eigen_decomp_sym(Matrixd &vals, Matrixd &vecs) const
{
	assert(rows == cols);
	vals.resize(rows, 1);
	vecs.resize(rows, cols);
	/*double *buf = new double [rows*rows];
	int status = ippmEigenValuesVectorsSym_m_64f(data, sizeof(double)*cols, sizeof(double), buf, vecs.data, sizeof(double)*cols, sizeof(double), vals.data, rows);
	delete[] buf;
	return status == ippStsNoErr;*/
	vector<double> vvals(cols);
	jacobi((Matrixd)*this, vvals, vecs);
	for (int i = 0; i < cols; i++)
		vals[i] = vvals[i];
}

Matrixd Matrixd::transpose() const
{
	Matrixd ret(cols, rows);
	for (int r = 0; r < rows; r++)
		for (int c = 0; c < cols; c++)
			ret(c,r) = get(r,c);
	return ret;
}

double Matrixd::det() const
{
	assert(rows == cols);
	/*double ret;
	double *buf = new double [rows*rows + rows];
	int status = ippmDet_m_64f(data, sizeof(double)*cols, sizeof(double), rows, buf, &ret);
	delete[] buf;
	assert(status == ippStsNoErr);
	return ret;*/

	// only true if A is symmetric
	Matrixd vecs;
	vecs.resize(rows, cols);
	vector<double> vvals(cols);
	jacobi((Matrixd)*this, vvals, vecs);
	double ret = 1;
	for (int i = 0; i < cols; i++)
		ret *= vvals[i];
	return ret;
}

Matrixd Matrixd::operator*(const Matrixd &m) const
{
	Matrixd ret(rows, m.cols);
	//int status = ippmMul_mm_64f(data, sizeof(double)*cols, sizeof(double), cols, rows, 
	//	m.data, sizeof(double)*m.cols, sizeof(double), m.cols, m.rows, 
	//	ret.data, sizeof(double)*ret.cols, sizeof(double));
	//assert(status == ippStsNoErr);
	
	for (int i = 0; i < rows; i++)
	{
		for (int j = 0; j < m.cols; j++)
		{
			double s = 0;
			for (int k = 0; k < cols; k++)
				s += get(i,k)*m.get(k, j);
			ret(i,j) = s;
		}
	}
	return ret;
}

Matrixd Matrixd::operator+(const Matrixd &m) const
{
	assert(m.rows == rows && m.cols == cols);
	Matrixd ret(rows, cols);
	/*int status = ippmAdd_mm_64f(data, sizeof(double)*cols, sizeof(double),
		m.data, sizeof(double)*m.cols, sizeof(double), 
		ret.data, sizeof(double)*ret.cols, sizeof(double), cols, rows);
	assert(status == ippStsNoErr);*/

	int n = rows*cols;
	for (int i = 0; i < n; i++)
		ret.data[i] = data[i] + m.data[i];

	return ret;
}


Matrixd Matrixd::operator-(const Matrixd &m) const
{
	assert(m.rows == rows && m.cols == cols);
	Matrixd ret(rows, cols);
	/*int status = ippmSub_mm_64f(data, sizeof(double)*cols, sizeof(double),
		m.data, sizeof(double)*m.cols, sizeof(double), 
		ret.data, sizeof(double)*ret.cols, sizeof(double), cols, rows);
	assert(status == ippStsNoErr);*/
	
	/*for (int i = 0; i < rows; i++)
	{
		for (int j = 0; j < m.cols; j++)
		{
			ret(i,j) = get(i,j) - m.get(i,j);
		}
	}*/
	
	int n = rows*cols;
	for (int i = 0; i < n; i++)
		ret.data[i] = data[i] - m.data[i];
	return ret;
}

Matrixd Matrixd::operator*(const double &v) const
{
	int n = rows*cols;
	Matrixd ret(rows, cols);
	for (int i = 0; i < n; i++)
		ret.data[i] = data[i] * v;
	return ret;
}

Matrixd Matrixd::operator+(const double &v) const
{
	int n = rows*cols;
	Matrixd ret(rows, cols);
	for (int i = 0; i < n; i++)
		ret.data[i] = data[i] + v;
	return ret;
}

void Matrixd::operator+=(const Matrixd &v)
{
	assert(rows == v.rows && cols == v.cols);
	int n = rows*cols;
	for (int i = 0; i < n; i++)
		data[i] += v.data[i];
}

Matrixd Matrixd::operator-(const double &v) const
{
	int n = rows*cols;
	Matrixd ret(rows, cols);
	for (int i = 0; i < n; i++)
		ret.data[i] = data[i] - v;
	return ret;
}

void Matrixd::set_zero()
{
	for (int r = 0; r < rows; r++)
		for (int c = 0; c < cols; c++)
			set(r, c, 0);
}

void Matrixd::set_identity()
{
	for (int r = 0; r < rows; r++)
		for (int c = 0; c < cols; c++)
			set(r, c, r == c ? 1 : 0);
}

void Matrixd::set_row(const int i, const double *val)
{
	for (int k = 0; k < cols; k++)
		set(i, k, val[k]);
}

void Matrixd::set_col(const int i, const double *val)
{
	for (int k = 0; k < rows; k++)
		set(k, i, val[k]);
}

void Matrixd::set_row(const int i, const double *val, const int limit)
{
	for (int k = 0; k < limit; k++)
		set(i, k, val[k]);
}

void Matrixd::set_col(const int i, const double *val, const int limit)
{
	for (int k = 0; k < limit; k++)
		set(k, i, val[k]);
}
