#pragma once

#include <vector>

using namespace std;

#include "sparse_vect.h"

struct SparseRowMat
{
	int n_r, n_c;
	vector<SparseVect> rows;

	SparseRowMat()
	{
		resize(0,0);
	}
	SparseRowMat(int r, int c)
	{
		resize(r,c);
	}
	
	void resize(int r, int c)
	{
		n_r = r;
		n_c = c;
		rows.resize(n_r);
		for (int i = 0; i < n_r; i++)
			rows[i].resize(n_c);
	}

	void clear()
	{
		for (int i = 0; i < n_r; i++)
			rows[i].clear();
	}

	double get(int r, int c)
	{
		return rows[r].get(c);
	}

	void set(int r, int c, double v)
	{
		rows[r].set(c, v);
	}

	// r = M.v
	template <class V, class R>
	void dotVect(V &v, R &r)
	{
		r.resize(v.n);
		for (int i = 0; i < v.n; i++)
			r.set(i, rows[i].dot(v));
	}

	// r = M.v
	template <class V, class R>
	void dotMat(V &v, R &r)
	{
		r.resize(n_r,v.n_c);
		for (int i = 0; i < r.n_r; i++)
		{
			for (int j = 0; j < r.n_c; j++)
			{
				double sum = 0;
				for (SparseVect::DataIt it = rows[i].d.begin(); it != rows[i].d.end(); ++it)
				{
					sum += it->second*v.get(it->first,j);
				}
				r.set(i,j,sum);
			}
		}
	}

	template <class R>
	void transpose(R &r)
	{
		r.resize(n_c, n_r);
		
		for (int i = 0; i < n_r; i++)
			for (SparseVect::DataIt it = rows[i].d.begin(); it != rows[i].d.end(); ++it)
				r.set(it->first, i, it->second);
	}
};
