#pragma once

#include <map>
#include "assert.h"

using namespace std;

#include "dense_vect.h"

struct SparseVect
{
	int n;
	typedef map<int, double> Data;
	typedef Data::iterator DataIt;
	Data d;

	SparseVect()
	{
		resize(0);
	}
	SparseVect(int size)
	{
		resize(size);
	}

	void resize(int size)
	{
		n = size;
		clear();
	}

	void clear()
	{
		d.clear();
	}

	double get(int i)
	{
		assert(i >= 0 && i < n);
		DataIt it = d.find(i);
		if (it == d.end())
			return 0;
		else
			return it->second;
	}
	
	void set(int i, double v)
	{
		assert(i >= 0 && i < n);
		if (v == 0)
		{
			DataIt it = d.find(i);
			if (it != d.end())
				d.erase(it);
		}
		else
		{
			d[i] = v;
		}
	}

	template<class T>
	double dot(T &v)
	{
		double sum = 0;
		if ((void*)&v == this)
		{
			for (DataIt i = d.begin(); i != d.end(); ++i)
				sum += i->second*i->second;
		}
		else
		{
			for (DataIt i = d.begin(); i != d.end(); ++i)
				sum += i->second*v.get(i->first);
		}
		return sum;
	}

	DenseVect operator*(double v)
	{
		DenseVect r(n);
		for (DataIt i = d.begin(); i != d.end(); ++i)
			r.set(i->first, i->second * v);
		return r;
	}
	void operator*=(double v)
	{
		for (DataIt i = d.begin(); i != d.end(); ++i)
			i->second *= v;
	}
};
