
#define _USE_MATH_DEFINES
#include <map>
#include <float.h>

#include "global.h"
#include "matrixd.h"
#include "bary.h"


void calcBaryQuad_Alpha1(double *bary, double *bary2, vect2d &x, vector<vect2d> &pv, vector<vect2i> &p, vector<vect2d> &r)
{
	double epsilon = 1e-3;

	int n = pv.size();
	if (n < 3)
		return;

	// calculate distances from line and projection onto line
	vector<double> dists(n);
	vector<double> projs(n);
	for (int i = 0; i < n; i++)
	{
		vect2d vv = pv[p[i][1]] - pv[p[i][0]];
		vect2d xx = x - pv[p[i][0]];
		projs[i] = xx*vv/vv.length2();
		dists[i] = (xx - vv*projs[i]).length();
	}

	// fix degenerate cases (force to interpolate the line)
	for (int i = 0; i < n; i++)
	{
		if (dists[i] < epsilon && projs[i] >= 0 && projs[i] <= 1)
		{
			bary[p[i][0]] += 1 - projs[i];
			bary[p[i][1]] += projs[i];
			return;
		}
	}

	// calculate constant
	Matrixd M(6,6), W(6,6);
	M.set_zero();
	for (int i = 0; i < n; i++)
	{
		// calculate some common terms
		vect2d &p1 = pv[p[i][0]];
		vect2d &p2 = pv[p[i][1]];
		double ax = p1[0];
		double ay = p1[1];
		double bx = p2[0];
		double by = p2[1];

		double a = (p1 - x)*(p1 - x);
		double b = (p1 - p2)*(p1 - x)*(-2);
		double c = (p1 - p2)*(p1 - p2);

		double sqrt_c = sqrt(c);

		double r5 = -b*b + 4*a*c;
		double r1 = sqrt(r5);
		double r4 = 1.0 / (r1*r5);
		double r2 = atan(b/r1);
		double r3 = atan((b+2*c)/r1);
		double r6 = a+b+c;

		// calc matrix of PtP
		W(0,0) = (-2*(r2 - r3))/r1;
		W(0,1) = (-2*(-(b*bx) + ax*(b + 2*c))*r2 + 2*(-(b*bx) + ax*(b + 2*c))*r3 + (ax - bx)*r1*(Log(a) - Log(r6)))/(2.*c*r1);
		W(0,2) = (-2*(-(b*by) + ay*(b + 2*c))*r2 + 2*(-(b*by) + ay*(b + 2*c))*r3 + (ay - by)*r1*(Log(a) - Log(r6)))/(2.*c*r1);
		W(0,3) = (2*Power(ax - bx,2)*c - (2*(Power(bx,2)*(Power(b,2) - 2*a*c) - 2*ax*bx*(Power(b,2) - 2*a*c + b*c) + Power(ax,2)*(Power(b,2) + 2*b*c + 2*c*(-a + c)))*r2)/r1 + (2*(Power(bx,2)*(Power(b,2) - 2*a*c) - 2*ax*bx*(Power(b,2) - 2*a*c + b*c) + Power(ax,2)*(Power(b,2) + 2*b*c + 2*c*(-a + c)))*r3)/r1 + (ax - bx)*(-(b*bx) + ax*(b + 2*c))*Log(a) - (ax - bx)*(-(b*bx) + ax*(b + 2*c))*Log(r6))/(2.*Power(c,2));
		W(0,4) = (2*(ax - bx)*(ay - by)*c - (2*(bx*(by*(Power(b,2) - 2*a*c) - ay*(Power(b,2) - 2*a*c + b*c)) + ax*(-(by*(Power(b,2) - 2*a*c + b*c)) + ay*(Power(b,2) + 2*b*c + 2*c*(-a + c))))*r2)/r1 + (2*(bx*(by*(Power(b,2) - 2*a*c) - ay*(Power(b,2) - 2*a*c + b*c)) + ax*(-(by*(Power(b,2) - 2*a*c + b*c)) + ay*(Power(b,2) + 2*b*c + 2*c*(-a + c))))*r3)/r1 - (bx*(-(b*by) + ay*(b + c)) + ax*(by*(b + c) - ay*(b + 2*c)))*Log(a) + (bx*(-(b*by) + ay*(b + c)) + ax*(by*(b + c) - ay*(b + 2*c)))*Log(r6))/(2.*Power(c,2));
		W(0,5) = (2*Power(ay - by,2)*c - (2*(Power(by,2)*(Power(b,2) - 2*a*c) - 2*ay*by*(Power(b,2) - 2*a*c + b*c) + Power(ay,2)*(Power(b,2) + 2*b*c + 2*c*(-a + c)))*r2)/r1 + (2*(Power(by,2)*(Power(b,2) - 2*a*c) - 2*ay*by*(Power(b,2) - 2*a*c + b*c) + Power(ay,2)*(Power(b,2) + 2*b*c + 2*c*(-a + c)))*r3)/r1 + (ay - by)*(-(b*by) + ay*(b + 2*c))*Log(a) - (ay - by)*(-(b*by) + ay*(b + 2*c))*Log(r6))/(2.*Power(c,2));
		W(1,1) = (2*Power(ax - bx,2)*c - (2*(Power(bx,2)*(Power(b,2) - 2*a*c) - 2*ax*bx*(Power(b,2) - 2*a*c + b*c) + Power(ax,2)*(Power(b,2) + 2*b*c + 2*c*(-a + c)))*r2)/r1 + (2*(Power(bx,2)*(Power(b,2) - 2*a*c) - 2*ax*bx*(Power(b,2) - 2*a*c + b*c) + Power(ax,2)*(Power(b,2) + 2*b*c + 2*c*(-a + c)))*r3)/r1 + (ax - bx)*(-(b*bx) + ax*(b + 2*c))*Log(a) - (ax - bx)*(-(b*bx) + ax*(b + 2*c))*Log(r6))/(2.*Power(c,2));
		W(1,2) = (2*(ax - bx)*(ay - by)*c - (2*(bx*(by*(Power(b,2) - 2*a*c) - ay*(Power(b,2) - 2*a*c + b*c)) + ax*(-(by*(Power(b,2) - 2*a*c + b*c)) + ay*(Power(b,2) + 2*b*c + 2*c*(-a + c))))*r2)/r1 + (2*(bx*(by*(Power(b,2) - 2*a*c) - ay*(Power(b,2) - 2*a*c + b*c)) + ax*(-(by*(Power(b,2) - 2*a*c + b*c)) + ay*(Power(b,2) + 2*b*c + 2*c*(-a + c))))*r3)/r1 - (bx*(-(b*by) + ay*(b + c)) + ax*(by*(b + c) - ay*(b + 2*c)))*Log(a) + (bx*(-(b*by) + ay*(b + c)) + ax*(by*(b + c) - ay*(b + 2*c)))*Log(r6))/(2.*Power(c,2));
		W(1,3) = (-(Power(ax - bx,3)*Power(c,2)) + 2*Power(ax - bx,2)*c*(-(b*bx) + ax*(b + 3*c)) - (2*(-(b*bx) + ax*(b + 2*c))*(Power(bx,2)*(Power(b,2) - 3*a*c) - ax*bx*(2*Power(b,2) - 6*a*c + b*c) + Power(ax,2)*(Power(b,2) + b*c + c*(-3*a + c)))*r2)/r1 + (2*(-(b*bx) + ax*(b + 2*c))*(Power(bx,2)*(Power(b,2) - 3*a*c) - ax*bx*(2*Power(b,2) - 6*a*c + b*c) + Power(ax,2)*(Power(b,2) + b*c + c*(-3*a + c)))*r3)/r1 + (ax - bx)*(Power(bx,2)*(Power(b,2) - a*c) + ax*bx*(-2*Power(b,2) + 2*a*c - 3*b*c) + Power(ax,2)*(Power(b,2) + 3*b*c + c*(-a + 3*c)))*Log(a) - (ax - bx)*(Power(bx,2)*(Power(b,2) - a*c) + ax*bx*(-2*Power(b,2) + 2*a*c - 3*b*c) + Power(ax,2)*(Power(b,2) + 3*b*c + c*(-a + 3*c)))*Log(r6))/(2.*Power(c,3));
		W(1,4) = (-(Power(ax - bx,2)*(ay - by)*Power(c,2)) + 2*(ax - bx)*c*(-(bx*(-(b*by) + ay*(b + c))) + ax*(-(by*(b + 2*c)) + ay*(b + 3*c))) + (2*(Power(bx,2)*(b*by*(Power(b,2) - 3*a*c) - ay*(Power(b,3) - 3*a*b*c + Power(b,2)*c - 2*a*Power(c,2))) + 2*ax*bx*(-(by*(Power(b,3) - 3*a*b*c + Power(b,2)*c - 2*a*Power(c,2))) + ay*(Power(b,3) + 2*Power(b,2)*c - 4*a*Power(c,2) + b*c*(-3*a + c))) + Power(ax,2)*(-(ay*(b + 2*c)*(Power(b,2) + b*c + c*(-3*a + c))) + by*(Power(b,3) + 2*Power(b,2)*c - 4*a*Power(c,2) + b*c*(-3*a + c))))*r2)/r1 + (2*(Power(bx,2)*(-(b*by*(Power(b,2) - 3*a*c)) + ay*(Power(b,3) - 3*a*b*c + Power(b,2)*c - 2*a*Power(c,2))) - 2*ax*bx*(-(by*(Power(b,3) - 3*a*b*c + Power(b,2)*c - 2*a*Power(c,2))) + ay*(Power(b,3) + 2*Power(b,2)*c - 4*a*Power(c,2) + b*c*(-3*a + c))) + Power(ax,2)*(ay*(b + 2*c)*(Power(b,2) + b*c + c*(-3*a + c)) - by*(Power(b,3) + 2*Power(b,2)*c - 4*a*Power(c,2) + b*c*(-3*a + c))))*r3)/r1 - (-(Power(bx,2)*(-(Power(b,2)*by) + a*by*c + ay*(Power(b,2) - a*c + b*c))) + 2*ax*bx*(-(by*(Power(b,2) - a*c + b*c)) + ay*(Power(b,2) - a*c + 2*b*c + Power(c,2))) + Power(ax,2)*(by*(Power(b,2) - a*c + 2*b*c + Power(c,2)) - ay*(Power(b,2) - a*c + 3*b*c + 3*Power(c,2))))*Log(a) + (-(Power(bx,2)*(-(Power(b,2)*by) + a*by*c + ay*(Power(b,2) - a*c + b*c))) + 2*ax*bx*(-(by*(Power(b,2) - a*c + b*c)) + ay*(Power(b,2) - a*c + 2*b*c + Power(c,2))) + Power(ax,2)*(by*(Power(b,2) - a*c + 2*b*c + Power(c,2)) - ay*(Power(b,2) - a*c + 3*b*c + 3*Power(c,2))))*Log(r6))/(2.*Power(c,3));
		W(1,5) = ((-ax + bx)*Power(ay - by,2)*Power(c,2) + 2*(ay - by)*c*(bx*(b*by - ay*(b + 2*c)) + ax*(-(by*(b + c)) + ay*(b + 3*c))) + (2*(bx*(b*Power(by,2)*(Power(b,2) - 3*a*c) - 2*ay*by*(Power(b,3) - 3*a*b*c + Power(b,2)*c - 2*a*Power(c,2)) + Power(ay,2)*(Power(b,3) + 2*Power(b,2)*c - 4*a*Power(c,2) + b*c*(-3*a + c))) - ax*(Power(by,2)*(Power(b,3) - 3*a*b*c + Power(b,2)*c - 2*a*Power(c,2)) + Power(ay,2)*(b + 2*c)*(Power(b,2) + b*c + c*(-3*a + c)) - 2*ay*by*(Power(b,3) + 2*Power(b,2)*c - 4*a*Power(c,2) + b*c*(-3*a + c))))*r2)/r1 + (2*(-(bx*(b*Power(by,2)*(Power(b,2) - 3*a*c) - 2*ay*by*(Power(b,3) - 3*a*b*c + Power(b,2)*c - 2*a*Power(c,2)) + Power(ay,2)*(Power(b,3) + 2*Power(b,2)*c - 4*a*Power(c,2) + b*c*(-3*a + c)))) + ax*(Power(by,2)*(Power(b,3) - 3*a*b*c + Power(b,2)*c - 2*a*Power(c,2)) + Power(ay,2)*(b + 2*c)*(Power(b,2) + b*c + c*(-3*a + c)) - 2*ay*by*(Power(b,3) + 2*Power(b,2)*c - 4*a*Power(c,2) + b*c*(-3*a + c))))*r3)/r1 - (bx*(Power(by,2)*(Power(b,2) - a*c) - 2*ay*by*(Power(b,2) - a*c + b*c) + Power(ay,2)*(Power(b,2) - a*c + 2*b*c + Power(c,2))) - ax*(Power(by,2)*(Power(b,2) - a*c + b*c) - 2*ay*by*(Power(b,2) - a*c + 2*b*c + Power(c,2)) + Power(ay,2)*(Power(b,2) - a*c + 3*b*c + 3*Power(c,2))))*Log(a) + (bx*(Power(by,2)*(Power(b,2) - a*c) - 2*ay*by*(Power(b,2) - a*c + b*c) + Power(ay,2)*(Power(b,2) - a*c + 2*b*c + Power(c,2))) - ax*(Power(by,2)*(Power(b,2) - a*c + b*c) - 2*ay*by*(Power(b,2) - a*c + 2*b*c + Power(c,2)) + Power(ay,2)*(Power(b,2) - a*c + 3*b*c + 3*Power(c,2))))*Log(r6))/(2.*Power(c,3));
		W(2,2) = (2*Power(ay - by,2)*c - (2*(Power(by,2)*(Power(b,2) - 2*a*c) - 2*ay*by*(Power(b,2) - 2*a*c + b*c) + Power(ay,2)*(Power(b,2) + 2*b*c + 2*c*(-a + c)))*r2)/r1 + (2*(Power(by,2)*(Power(b,2) - 2*a*c) - 2*ay*by*(Power(b,2) - 2*a*c + b*c) + Power(ay,2)*(Power(b,2) + 2*b*c + 2*c*(-a + c)))*r3)/r1 + (ay - by)*(-(b*by) + ay*(b + 2*c))*Log(a) - (ay - by)*(-(b*by) + ay*(b + 2*c))*Log(r6))/(2.*Power(c,2));
		W(2,3) = (-(Power(ax - bx,2)*(ay - by)*Power(c,2)) + 2*(ax - bx)*c*(-(bx*(-(b*by) + ay*(b + c))) + ax*(-(by*(b + 2*c)) + ay*(b + 3*c))) + (2*(Power(bx,2)*(b*by*(Power(b,2) - 3*a*c) - ay*(Power(b,3) - 3*a*b*c + Power(b,2)*c - 2*a*Power(c,2))) + 2*ax*bx*(-(by*(Power(b,3) - 3*a*b*c + Power(b,2)*c - 2*a*Power(c,2))) + ay*(Power(b,3) + 2*Power(b,2)*c - 4*a*Power(c,2) + b*c*(-3*a + c))) + Power(ax,2)*(-(ay*(b + 2*c)*(Power(b,2) + b*c + c*(-3*a + c))) + by*(Power(b,3) + 2*Power(b,2)*c - 4*a*Power(c,2) + b*c*(-3*a + c))))*r2)/r1 + (2*(Power(bx,2)*(-(b*by*(Power(b,2) - 3*a*c)) + ay*(Power(b,3) - 3*a*b*c + Power(b,2)*c - 2*a*Power(c,2))) - 2*ax*bx*(-(by*(Power(b,3) - 3*a*b*c + Power(b,2)*c - 2*a*Power(c,2))) + ay*(Power(b,3) + 2*Power(b,2)*c - 4*a*Power(c,2) + b*c*(-3*a + c))) + Power(ax,2)*(ay*(b + 2*c)*(Power(b,2) + b*c + c*(-3*a + c)) - by*(Power(b,3) + 2*Power(b,2)*c - 4*a*Power(c,2) + b*c*(-3*a + c))))*r3)/r1 - (-(Power(bx,2)*(-(Power(b,2)*by) + a*by*c + ay*(Power(b,2) - a*c + b*c))) + 2*ax*bx*(-(by*(Power(b,2) - a*c + b*c)) + ay*(Power(b,2) - a*c + 2*b*c + Power(c,2))) + Power(ax,2)*(by*(Power(b,2) - a*c + 2*b*c + Power(c,2)) - ay*(Power(b,2) - a*c + 3*b*c + 3*Power(c,2))))*Log(a) + (-(Power(bx,2)*(-(Power(b,2)*by) + a*by*c + ay*(Power(b,2) - a*c + b*c))) + 2*ax*bx*(-(by*(Power(b,2) - a*c + b*c)) + ay*(Power(b,2) - a*c + 2*b*c + Power(c,2))) + Power(ax,2)*(by*(Power(b,2) - a*c + 2*b*c + Power(c,2)) - ay*(Power(b,2) - a*c + 3*b*c + 3*Power(c,2))))*Log(r6))/(2.*Power(c,3));
		W(2,4) = ((-ax + bx)*Power(ay - by,2)*Power(c,2) + 2*(ay - by)*c*(bx*(b*by - ay*(b + 2*c)) + ax*(-(by*(b + c)) + ay*(b + 3*c))) + (2*(bx*(b*Power(by,2)*(Power(b,2) - 3*a*c) - 2*ay*by*(Power(b,3) - 3*a*b*c + Power(b,2)*c - 2*a*Power(c,2)) + Power(ay,2)*(Power(b,3) + 2*Power(b,2)*c - 4*a*Power(c,2) + b*c*(-3*a + c))) - ax*(Power(by,2)*(Power(b,3) - 3*a*b*c + Power(b,2)*c - 2*a*Power(c,2)) + Power(ay,2)*(b + 2*c)*(Power(b,2) + b*c + c*(-3*a + c)) - 2*ay*by*(Power(b,3) + 2*Power(b,2)*c - 4*a*Power(c,2) + b*c*(-3*a + c))))*r2)/r1 + (2*(-(bx*(b*Power(by,2)*(Power(b,2) - 3*a*c) - 2*ay*by*(Power(b,3) - 3*a*b*c + Power(b,2)*c - 2*a*Power(c,2)) + Power(ay,2)*(Power(b,3) + 2*Power(b,2)*c - 4*a*Power(c,2) + b*c*(-3*a + c)))) + ax*(Power(by,2)*(Power(b,3) - 3*a*b*c + Power(b,2)*c - 2*a*Power(c,2)) + Power(ay,2)*(b + 2*c)*(Power(b,2) + b*c + c*(-3*a + c)) - 2*ay*by*(Power(b,3) + 2*Power(b,2)*c - 4*a*Power(c,2) + b*c*(-3*a + c))))*r3)/r1 - (bx*(Power(by,2)*(Power(b,2) - a*c) - 2*ay*by*(Power(b,2) - a*c + b*c) + Power(ay,2)*(Power(b,2) - a*c + 2*b*c + Power(c,2))) - ax*(Power(by,2)*(Power(b,2) - a*c + b*c) - 2*ay*by*(Power(b,2) - a*c + 2*b*c + Power(c,2)) + Power(ay,2)*(Power(b,2) - a*c + 3*b*c + 3*Power(c,2))))*Log(a) + (bx*(Power(by,2)*(Power(b,2) - a*c) - 2*ay*by*(Power(b,2) - a*c + b*c) + Power(ay,2)*(Power(b,2) - a*c + 2*b*c + Power(c,2))) - ax*(Power(by,2)*(Power(b,2) - a*c + b*c) - 2*ay*by*(Power(b,2) - a*c + 2*b*c + Power(c,2)) + Power(ay,2)*(Power(b,2) - a*c + 3*b*c + 3*Power(c,2))))*Log(r6))/(2.*Power(c,3));
		W(2,5) = (-(Power(ay - by,3)*Power(c,2)) + 2*Power(ay - by,2)*c*(-(b*by) + ay*(b + 3*c)) - (2*(-(b*by) + ay*(b + 2*c))*(Power(by,2)*(Power(b,2) - 3*a*c) - ay*by*(2*Power(b,2) - 6*a*c + b*c) + Power(ay,2)*(Power(b,2) + b*c + c*(-3*a + c)))*r2)/r1 + (2*(-(b*by) + ay*(b + 2*c))*(Power(by,2)*(Power(b,2) - 3*a*c) - ay*by*(2*Power(b,2) - 6*a*c + b*c) + Power(ay,2)*(Power(b,2) + b*c + c*(-3*a + c)))*r3)/r1 + (ay - by)*(Power(by,2)*(Power(b,2) - a*c) + ay*by*(-2*Power(b,2) + 2*a*c - 3*b*c) + Power(ay,2)*(Power(b,2) + 3*b*c + c*(-a + 3*c)))*Log(a) - (ay - by)*(Power(by,2)*(Power(b,2) - a*c) + ay*by*(-2*Power(b,2) + 2*a*c - 3*b*c) + Power(ay,2)*(Power(b,2) + 3*b*c + c*(-a + 3*c)))*Log(r6))/(2.*Power(c,3));
		W(3,3) = (2*Power(ax - bx,4)*Power(c,3) - 3*Power(ax - bx,3)*Power(c,2)*(-(b*bx) + ax*(b + 4*c)) + 6*Power(ax - bx,2)*c*(Power(bx,2)*(Power(b,2) - a*c) - 2*ax*bx*(Power(b,2) - a*c + 2*b*c) + Power(ax,2)*(Power(b,2) + 4*b*c + c*(-a + 6*c))) - (6*(Power(bx,4)*(Power(b,4) - 4*a*Power(b,2)*c + 2*Power(a,2)*Power(c,2)) - 4*ax*Power(bx,3)*(Power(b,4) - 4*a*Power(b,2)*c + Power(b,3)*c + 2*Power(a,2)*Power(c,2) - 3*a*b*Power(c,2)) + 6*Power(ax,2)*Power(bx,2)*(Power(b,4) + 2*Power(b,3)*c - 6*a*b*Power(c,2) + 2*a*(a - c)*Power(c,2) + Power(b,2)*c*(-4*a + c)) - 4*Power(ax,3)*bx*(Power(b,4) + 3*Power(b,3)*c + 2*a*(a - 3*c)*Power(c,2) + b*Power(c,2)*(-9*a + c) + Power(b,2)*c*(-4*a + 3*c)) + Power(ax,4)*(Power(b,4) + 4*Power(b,3)*c + 4*b*Power(c,2)*(-3*a + c) + 2*Power(c,2)*(Power(a,2) - 6*a*c + Power(c,2)) + Power(b,2)*(-4*a*c + 6*Power(c,2))))*r2)/r1 + (6*(Power(bx,4)*(Power(b,4) - 4*a*Power(b,2)*c + 2*Power(a,2)*Power(c,2)) - 4*ax*Power(bx,3)*(Power(b,4) - 4*a*Power(b,2)*c + Power(b,3)*c + 2*Power(a,2)*Power(c,2) - 3*a*b*Power(c,2)) + 6*Power(ax,2)*Power(bx,2)*(Power(b,4) + 2*Power(b,3)*c - 6*a*b*Power(c,2) + 2*a*(a - c)*Power(c,2) + Power(b,2)*c*(-4*a + c)) - 4*Power(ax,3)*bx*(Power(b,4) + 3*Power(b,3)*c + 2*a*(a - 3*c)*Power(c,2) + b*Power(c,2)*(-9*a + c) + Power(b,2)*c*(-4*a + 3*c)) + Power(ax,4)*(Power(b,4) + 4*Power(b,3)*c + 4*b*Power(c,2)*(-3*a + c) + 2*Power(c,2)*(Power(a,2) - 6*a*c + Power(c,2)) + Power(b,2)*(-4*a*c + 6*Power(c,2))))*r3)/r1 + 3*(ax - bx)*(-(b*bx) + ax*(b + 2*c))*(Power(bx,2)*(Power(b,2) - 2*a*c) - 2*ax*bx*(Power(b,2) - 2*a*c + b*c) + Power(ax,2)*(Power(b,2) + 2*b*c + 2*c*(-a + c)))*Log(a) - 3*(ax - bx)*(-(b*bx) + ax*(b + 2*c))*(Power(bx,2)*(Power(b,2) - 2*a*c) - 2*ax*bx*(Power(b,2) - 2*a*c + b*c) + Power(ax,2)*(Power(b,2) + 2*b*c + 2*c*(-a + c)))*Log(r6))/(6.*Power(c,4));
		W(3,4) = (2*Power(ax - bx,3)*(ay - by)*Power(c,3) + 3*Power(ax - bx,2)*Power(c,2)*(bx*(-(b*by) + ay*(b + c)) + ax*(by*(b + 3*c) - ay*(b + 4*c))) + 6*(ax - bx)*c*(Power(bx,2)*(-(Power(b,2)*by) + a*by*c + ay*(Power(b,2) - a*c + b*c)) + ax*bx*(by*(2*Power(b,2) - 2*a*c + 3*b*c) - ay*(2*Power(b,2) - 2*a*c + 5*b*c + 3*Power(c,2))) + Power(ax,2)*(-(by*(Power(b,2) - a*c + 3*b*c + 3*Power(c,2))) + ay*(Power(b,2) - a*c + 4*b*c + 6*Power(c,2)))) - (6*(Power(bx,3)*(by*(Power(b,4) - 4*a*Power(b,2)*c + 2*Power(a,2)*Power(c,2)) - ay*(Power(b,4) - 4*a*Power(b,2)*c + Power(b,3)*c + 2*Power(a,2)*Power(c,2) - 3*a*b*Power(c,2))) + 3*ax*Power(bx,2)*(-(by*(Power(b,4) - 4*a*Power(b,2)*c + Power(b,3)*c + 2*Power(a,2)*Power(c,2) - 3*a*b*Power(c,2))) + ay*(Power(b,4) + 2*Power(b,3)*c - 6*a*b*Power(c,2) + 2*a*(a - c)*Power(c,2) + Power(b,2)*c*(-4*a + c))) - 3*Power(ax,2)*bx*(-(by*(Power(b,4) + 2*Power(b,3)*c - 6*a*b*Power(c,2) + 2*a*(a - c)*Power(c,2) + Power(b,2)*c*(-4*a + c))) + ay*(Power(b,4) + 3*Power(b,3)*c + 2*a*(a - 3*c)*Power(c,2) + b*Power(c,2)*(-9*a + c) + Power(b,2)*c*(-4*a + 3*c))) + Power(ax,3)*(-(by*(Power(b,4) + 3*Power(b,3)*c + 2*a*(a - 3*c)*Power(c,2) + b*Power(c,2)*(-9*a + c) + Power(b,2)*c*(-4*a + 3*c))) + ay*(Power(b,4) + 4*Power(b,3)*c + 4*b*Power(c,2)*(-3*a + c) + 2*Power(c,2)*(Power(a,2) - 6*a*c + Power(c,2)) + Power(b,2)*(-4*a*c + 6*Power(c,2)))))*r2)/r1 + (6*(Power(bx,3)*(by*(Power(b,4) - 4*a*Power(b,2)*c + 2*Power(a,2)*Power(c,2)) - ay*(Power(b,4) - 4*a*Power(b,2)*c + Power(b,3)*c + 2*Power(a,2)*Power(c,2) - 3*a*b*Power(c,2))) + 3*ax*Power(bx,2)*(-(by*(Power(b,4) - 4*a*Power(b,2)*c + Power(b,3)*c + 2*Power(a,2)*Power(c,2) - 3*a*b*Power(c,2))) + ay*(Power(b,4) + 2*Power(b,3)*c - 6*a*b*Power(c,2) + 2*a*(a - c)*Power(c,2) + Power(b,2)*c*(-4*a + c))) - 3*Power(ax,2)*bx*(-(by*(Power(b,4) + 2*Power(b,3)*c - 6*a*b*Power(c,2) + 2*a*(a - c)*Power(c,2) + Power(b,2)*c*(-4*a + c))) + ay*(Power(b,4) + 3*Power(b,3)*c + 2*a*(a - 3*c)*Power(c,2) + b*Power(c,2)*(-9*a + c) + Power(b,2)*c*(-4*a + 3*c))) + Power(ax,3)*(-(by*(Power(b,4) + 3*Power(b,3)*c + 2*a*(a - 3*c)*Power(c,2) + b*Power(c,2)*(-9*a + c) + Power(b,2)*c*(-4*a + 3*c))) + ay*(Power(b,4) + 4*Power(b,3)*c + 4*b*Power(c,2)*(-3*a + c) + 2*Power(c,2)*(Power(a,2) - 6*a*c + Power(c,2)) + Power(b,2)*(-4*a*c + 6*Power(c,2)))))*r3)/r1 - 3*(Power(bx,3)*(-(b*by*(Power(b,2) - 2*a*c)) + ay*(Power(b,3) - 2*a*b*c + Power(b,2)*c - a*Power(c,2))) - 3*ax*Power(bx,2)*(ay*(b + c)*(Power(b,2) - 2*a*c + b*c) - by*(Power(b,3) - 2*a*b*c + Power(b,2)*c - a*Power(c,2))) + 3*Power(ax,2)*bx*(-(by*(b + c)*(Power(b,2) - 2*a*c + b*c)) + ay*(Power(b,3) + 3*Power(b,2)*c + Power(c,2)*(-3*a + c) + b*c*(-2*a + 3*c))) + Power(ax,3)*(-(ay*(b + 2*c)*(Power(b,2) + 2*b*c + 2*c*(-a + c))) + by*(Power(b,3) + 3*Power(b,2)*c + Power(c,2)*(-3*a + c) + b*c*(-2*a + 3*c))))*Log(a) + 3*(Power(bx,3)*(-(b*by*(Power(b,2) - 2*a*c)) + ay*(Power(b,3) - 2*a*b*c + Power(b,2)*c - a*Power(c,2))) - 3*ax*Power(bx,2)*(ay*(b + c)*(Power(b,2) - 2*a*c + b*c) - by*(Power(b,3) - 2*a*b*c + Power(b,2)*c - a*Power(c,2))) + 3*Power(ax,2)*bx*(-(by*(b + c)*(Power(b,2) - 2*a*c + b*c)) + ay*(Power(b,3) + 3*Power(b,2)*c + Power(c,2)*(-3*a + c) + b*c*(-2*a + 3*c))) + Power(ax,3)*(-(ay*(b + 2*c)*(Power(b,2) + 2*b*c + 2*c*(-a + c))) + by*(Power(b,3) + 3*Power(b,2)*c + Power(c,2)*(-3*a + c) + b*c*(-2*a + 3*c))))*Log(r6))/(6.*Power(c,4));
		W(3,5) = (2*Power(ax - bx,2)*Power(ay - by,2)*Power(c,3) + 3*(-ax + bx)*(ay - by)*Power(c,2)*(bx*(b*by - ay*(b + 2*c)) + ax*(-(by*(b + 2*c)) + ay*(b + 4*c))) + 6*c*(Power(bx,2)*(Power(by,2)*(Power(b,2) - a*c) - 2*ay*by*(Power(b,2) - a*c + b*c) + Power(ay,2)*(Power(b,2) - a*c + 2*b*c + Power(c,2))) - 2*ax*bx*(Power(by,2)*(Power(b,2) - a*c + b*c) - 2*ay*by*(Power(b,2) - a*c + 2*b*c + Power(c,2)) + Power(ay,2)*(Power(b,2) - a*c + 3*b*c + 3*Power(c,2))) + Power(ax,2)*(Power(by,2)*(Power(b,2) - a*c + 2*b*c + Power(c,2)) - 2*ay*by*(Power(b,2) - a*c + 3*b*c + 3*Power(c,2)) + Power(ay,2)*(Power(b,2) - a*c + 4*b*c + 6*Power(c,2)))) - (6*(Power(bx,2)*(Power(by,2)*(Power(b,4) - 4*a*Power(b,2)*c + 2*Power(a,2)*Power(c,2)) - 2*ay*by*(Power(b,4) - 4*a*Power(b,2)*c + Power(b,3)*c + 2*Power(a,2)*Power(c,2) - 3*a*b*Power(c,2)) + Power(ay,2)*(Power(b,4) + 2*Power(b,3)*c - 6*a*b*Power(c,2) + 2*a*(a - c)*Power(c,2) + Power(b,2)*c*(-4*a + c))) - 2*ax*bx*(Power(by,2)*(Power(b,4) - 4*a*Power(b,2)*c + Power(b,3)*c + 2*Power(a,2)*Power(c,2) - 3*a*b*Power(c,2)) - 2*ay*by*(Power(b,4) + 2*Power(b,3)*c - 6*a*b*Power(c,2) + 2*a*(a - c)*Power(c,2) + Power(b,2)*c*(-4*a + c)) + Power(ay,2)*(Power(b,4) + 3*Power(b,3)*c + 2*a*(a - 3*c)*Power(c,2) + b*Power(c,2)*(-9*a + c) + Power(b,2)*c*(-4*a + 3*c))) + Power(ax,2)*(Power(by,2)*(Power(b,4) + 2*Power(b,3)*c - 6*a*b*Power(c,2) + 2*a*(a - c)*Power(c,2) + Power(b,2)*c*(-4*a + c)) - 2*ay*by*(Power(b,4) + 3*Power(b,3)*c + 2*a*(a - 3*c)*Power(c,2) + b*Power(c,2)*(-9*a + c) + Power(b,2)*c*(-4*a + 3*c)) + Power(ay,2)*(Power(b,4) + 4*Power(b,3)*c + 4*b*Power(c,2)*(-3*a + c) + 2*Power(c,2)*(Power(a,2) - 6*a*c + Power(c,2)) + Power(b,2)*(-4*a*c + 6*Power(c,2)))))*r2)/r1 + (6*(Power(bx,2)*(Power(by,2)*(Power(b,4) - 4*a*Power(b,2)*c + 2*Power(a,2)*Power(c,2)) - 2*ay*by*(Power(b,4) - 4*a*Power(b,2)*c + Power(b,3)*c + 2*Power(a,2)*Power(c,2) - 3*a*b*Power(c,2)) + Power(ay,2)*(Power(b,4) + 2*Power(b,3)*c - 6*a*b*Power(c,2) + 2*a*(a - c)*Power(c,2) + Power(b,2)*c*(-4*a + c))) - 2*ax*bx*(Power(by,2)*(Power(b,4) - 4*a*Power(b,2)*c + Power(b,3)*c + 2*Power(a,2)*Power(c,2) - 3*a*b*Power(c,2)) - 2*ay*by*(Power(b,4) + 2*Power(b,3)*c - 6*a*b*Power(c,2) + 2*a*(a - c)*Power(c,2) + Power(b,2)*c*(-4*a + c)) + Power(ay,2)*(Power(b,4) + 3*Power(b,3)*c + 2*a*(a - 3*c)*Power(c,2) + b*Power(c,2)*(-9*a + c) + Power(b,2)*c*(-4*a + 3*c))) + Power(ax,2)*(Power(by,2)*(Power(b,4) + 2*Power(b,3)*c - 6*a*b*Power(c,2) + 2*a*(a - c)*Power(c,2) + Power(b,2)*c*(-4*a + c)) - 2*ay*by*(Power(b,4) + 3*Power(b,3)*c + 2*a*(a - 3*c)*Power(c,2) + b*Power(c,2)*(-9*a + c) + Power(b,2)*c*(-4*a + 3*c)) + Power(ay,2)*(Power(b,4) + 4*Power(b,3)*c + 4*b*Power(c,2)*(-3*a + c) + 2*Power(c,2)*(Power(a,2) - 6*a*c + Power(c,2)) + Power(b,2)*(-4*a*c + 6*Power(c,2)))))*r3)/r1 + 3*(-(bx*(-(b*by) + ay*(b + c))) + ax*(-(by*(b + c)) + ay*(b + 2*c)))*(bx*(by*(Power(b,2) - 2*a*c) - ay*(Power(b,2) - 2*a*c + b*c)) + ax*(-(by*(Power(b,2) - 2*a*c + b*c)) + ay*(Power(b,2) + 2*b*c + 2*c*(-a + c))))*Log(a) - 3*(-(bx*(-(b*by) + ay*(b + c))) + ax*(-(by*(b + c)) + ay*(b + 2*c)))*(bx*(by*(Power(b,2) - 2*a*c) - ay*(Power(b,2) - 2*a*c + b*c)) + ax*(-(by*(Power(b,2) - 2*a*c + b*c)) + ay*(Power(b,2) + 2*b*c + 2*c*(-a + c))))*Log(r6))/(6.*Power(c,4));
		W(4,4) = (2*Power(ax - bx,2)*Power(ay - by,2)*Power(c,3) + 3*(-ax + bx)*(ay - by)*Power(c,2)*(bx*(b*by - ay*(b + 2*c)) + ax*(-(by*(b + 2*c)) + ay*(b + 4*c))) + 6*c*(Power(bx,2)*(Power(by,2)*(Power(b,2) - a*c) - 2*ay*by*(Power(b,2) - a*c + b*c) + Power(ay,2)*(Power(b,2) - a*c + 2*b*c + Power(c,2))) - 2*ax*bx*(Power(by,2)*(Power(b,2) - a*c + b*c) - 2*ay*by*(Power(b,2) - a*c + 2*b*c + Power(c,2)) + Power(ay,2)*(Power(b,2) - a*c + 3*b*c + 3*Power(c,2))) + Power(ax,2)*(Power(by,2)*(Power(b,2) - a*c + 2*b*c + Power(c,2)) - 2*ay*by*(Power(b,2) - a*c + 3*b*c + 3*Power(c,2)) + Power(ay,2)*(Power(b,2) - a*c + 4*b*c + 6*Power(c,2)))) - (6*(Power(bx,2)*(Power(by,2)*(Power(b,4) - 4*a*Power(b,2)*c + 2*Power(a,2)*Power(c,2)) - 2*ay*by*(Power(b,4) - 4*a*Power(b,2)*c + Power(b,3)*c + 2*Power(a,2)*Power(c,2) - 3*a*b*Power(c,2)) + Power(ay,2)*(Power(b,4) + 2*Power(b,3)*c - 6*a*b*Power(c,2) + 2*a*(a - c)*Power(c,2) + Power(b,2)*c*(-4*a + c))) - 2*ax*bx*(Power(by,2)*(Power(b,4) - 4*a*Power(b,2)*c + Power(b,3)*c + 2*Power(a,2)*Power(c,2) - 3*a*b*Power(c,2)) - 2*ay*by*(Power(b,4) + 2*Power(b,3)*c - 6*a*b*Power(c,2) + 2*a*(a - c)*Power(c,2) + Power(b,2)*c*(-4*a + c)) + Power(ay,2)*(Power(b,4) + 3*Power(b,3)*c + 2*a*(a - 3*c)*Power(c,2) + b*Power(c,2)*(-9*a + c) + Power(b,2)*c*(-4*a + 3*c))) + Power(ax,2)*(Power(by,2)*(Power(b,4) + 2*Power(b,3)*c - 6*a*b*Power(c,2) + 2*a*(a - c)*Power(c,2) + Power(b,2)*c*(-4*a + c)) - 2*ay*by*(Power(b,4) + 3*Power(b,3)*c + 2*a*(a - 3*c)*Power(c,2) + b*Power(c,2)*(-9*a + c) + Power(b,2)*c*(-4*a + 3*c)) + Power(ay,2)*(Power(b,4) + 4*Power(b,3)*c + 4*b*Power(c,2)*(-3*a + c) + 2*Power(c,2)*(Power(a,2) - 6*a*c + Power(c,2)) + Power(b,2)*(-4*a*c + 6*Power(c,2)))))*r2)/r1 + (6*(Power(bx,2)*(Power(by,2)*(Power(b,4) - 4*a*Power(b,2)*c + 2*Power(a,2)*Power(c,2)) - 2*ay*by*(Power(b,4) - 4*a*Power(b,2)*c + Power(b,3)*c + 2*Power(a,2)*Power(c,2) - 3*a*b*Power(c,2)) + Power(ay,2)*(Power(b,4) + 2*Power(b,3)*c - 6*a*b*Power(c,2) + 2*a*(a - c)*Power(c,2) + Power(b,2)*c*(-4*a + c))) - 2*ax*bx*(Power(by,2)*(Power(b,4) - 4*a*Power(b,2)*c + Power(b,3)*c + 2*Power(a,2)*Power(c,2) - 3*a*b*Power(c,2)) - 2*ay*by*(Power(b,4) + 2*Power(b,3)*c - 6*a*b*Power(c,2) + 2*a*(a - c)*Power(c,2) + Power(b,2)*c*(-4*a + c)) + Power(ay,2)*(Power(b,4) + 3*Power(b,3)*c + 2*a*(a - 3*c)*Power(c,2) + b*Power(c,2)*(-9*a + c) + Power(b,2)*c*(-4*a + 3*c))) + Power(ax,2)*(Power(by,2)*(Power(b,4) + 2*Power(b,3)*c - 6*a*b*Power(c,2) + 2*a*(a - c)*Power(c,2) + Power(b,2)*c*(-4*a + c)) - 2*ay*by*(Power(b,4) + 3*Power(b,3)*c + 2*a*(a - 3*c)*Power(c,2) + b*Power(c,2)*(-9*a + c) + Power(b,2)*c*(-4*a + 3*c)) + Power(ay,2)*(Power(b,4) + 4*Power(b,3)*c + 4*b*Power(c,2)*(-3*a + c) + 2*Power(c,2)*(Power(a,2) - 6*a*c + Power(c,2)) + Power(b,2)*(-4*a*c + 6*Power(c,2)))))*r3)/r1 + 3*(-(bx*(-(b*by) + ay*(b + c))) + ax*(-(by*(b + c)) + ay*(b + 2*c)))*(bx*(by*(Power(b,2) - 2*a*c) - ay*(Power(b,2) - 2*a*c + b*c)) + ax*(-(by*(Power(b,2) - 2*a*c + b*c)) + ay*(Power(b,2) + 2*b*c + 2*c*(-a + c))))*Log(a) - 3*(-(bx*(-(b*by) + ay*(b + c))) + ax*(-(by*(b + c)) + ay*(b + 2*c)))*(bx*(by*(Power(b,2) - 2*a*c) - ay*(Power(b,2) - 2*a*c + b*c)) + ax*(-(by*(Power(b,2) - 2*a*c + b*c)) + ay*(Power(b,2) + 2*b*c + 2*c*(-a + c))))*Log(r6))/(6.*Power(c,4));
		W(4,5) = (2*(ax - bx)*Power(ay - by,3)*Power(c,3) - 3*Power(ay - by,2)*Power(c,2)*(bx*(b*by - ay*(b + 3*c)) + ax*(-(by*(b + c)) + ay*(b + 4*c))) + 6*(ay - by)*c*(bx*(Power(by,2)*(-Power(b,2) + a*c) + ay*by*(2*Power(b,2) - 2*a*c + 3*b*c) - Power(ay,2)*(Power(b,2) - a*c + 3*b*c + 3*Power(c,2))) + ax*(Power(by,2)*(Power(b,2) - a*c + b*c) - ay*by*(2*Power(b,2) - 2*a*c + 5*b*c + 3*Power(c,2)) + Power(ay,2)*(Power(b,2) - a*c + 4*b*c + 6*Power(c,2)))) - (6*(-(bx*(-(Power(by,3)*(Power(b,4) - 4*a*Power(b,2)*c + 2*Power(a,2)*Power(c,2))) + 3*ay*Power(by,2)*(Power(b,4) - 4*a*Power(b,2)*c + Power(b,3)*c + 2*Power(a,2)*Power(c,2) - 3*a*b*Power(c,2)) - 3*Power(ay,2)*by*(Power(b,4) + 2*Power(b,3)*c - 6*a*b*Power(c,2) + 2*a*(a - c)*Power(c,2) + Power(b,2)*c*(-4*a + c)) + Power(ay,3)*(Power(b,4) + 3*Power(b,3)*c + 2*a*(a - 3*c)*Power(c,2) + b*Power(c,2)*(-9*a + c) + Power(b,2)*c*(-4*a + 3*c)))) + ax*(-(Power(by,3)*(Power(b,4) - 4*a*Power(b,2)*c + Power(b,3)*c + 2*Power(a,2)*Power(c,2) - 3*a*b*Power(c,2))) + 3*ay*Power(by,2)*(Power(b,4) + 2*Power(b,3)*c - 6*a*b*Power(c,2) + 2*a*(a - c)*Power(c,2) + Power(b,2)*c*(-4*a + c)) - 3*Power(ay,2)*by*(Power(b,4) + 3*Power(b,3)*c + 2*a*(a - 3*c)*Power(c,2) + b*Power(c,2)*(-9*a + c) + Power(b,2)*c*(-4*a + 3*c)) + Power(ay,3)*(Power(b,4) + 4*Power(b,3)*c + 4*b*Power(c,2)*(-3*a + c) + 2*Power(c,2)*(Power(a,2) - 6*a*c + Power(c,2)) + Power(b,2)*(-4*a*c + 6*Power(c,2)))))*r2)/r1 + (6*(-(bx*(-(Power(by,3)*(Power(b,4) - 4*a*Power(b,2)*c + 2*Power(a,2)*Power(c,2))) + 3*ay*Power(by,2)*(Power(b,4) - 4*a*Power(b,2)*c + Power(b,3)*c + 2*Power(a,2)*Power(c,2) - 3*a*b*Power(c,2)) - 3*Power(ay,2)*by*(Power(b,4) + 2*Power(b,3)*c - 6*a*b*Power(c,2) + 2*a*(a - c)*Power(c,2) + Power(b,2)*c*(-4*a + c)) + Power(ay,3)*(Power(b,4) + 3*Power(b,3)*c + 2*a*(a - 3*c)*Power(c,2) + b*Power(c,2)*(-9*a + c) + Power(b,2)*c*(-4*a + 3*c)))) + ax*(-(Power(by,3)*(Power(b,4) - 4*a*Power(b,2)*c + Power(b,3)*c + 2*Power(a,2)*Power(c,2) - 3*a*b*Power(c,2))) + 3*ay*Power(by,2)*(Power(b,4) + 2*Power(b,3)*c - 6*a*b*Power(c,2) + 2*a*(a - c)*Power(c,2) + Power(b,2)*c*(-4*a + c)) - 3*Power(ay,2)*by*(Power(b,4) + 3*Power(b,3)*c + 2*a*(a - 3*c)*Power(c,2) + b*Power(c,2)*(-9*a + c) + Power(b,2)*c*(-4*a + 3*c)) + Power(ay,3)*(Power(b,4) + 4*Power(b,3)*c + 4*b*Power(c,2)*(-3*a + c) + 2*Power(c,2)*(Power(a,2) - 6*a*c + Power(c,2)) + Power(b,2)*(-4*a*c + 6*Power(c,2)))))*r3)/r1 - 3*(bx*(-(b*Power(by,3)*(Power(b,2) - 2*a*c)) - 3*Power(ay,2)*by*(b + c)*(Power(b,2) - 2*a*c + b*c) + 3*ay*Power(by,2)*(Power(b,3) - 2*a*b*c + Power(b,2)*c - a*Power(c,2)) + Power(ay,3)*(Power(b,3) + 3*Power(b,2)*c + Power(c,2)*(-3*a + c) + b*c*(-2*a + 3*c))) + ax*(-3*ay*Power(by,2)*(b + c)*(Power(b,2) - 2*a*c + b*c) + Power(by,3)*(Power(b,3) - 2*a*b*c + Power(b,2)*c - a*Power(c,2)) - Power(ay,3)*(b + 2*c)*(Power(b,2) + 2*b*c + 2*c*(-a + c)) + 3*Power(ay,2)*by*(Power(b,3) + 3*Power(b,2)*c + Power(c,2)*(-3*a + c) + b*c*(-2*a + 3*c))))*Log(a) + 3*(bx*(-(b*Power(by,3)*(Power(b,2) - 2*a*c)) - 3*Power(ay,2)*by*(b + c)*(Power(b,2) - 2*a*c + b*c) + 3*ay*Power(by,2)*(Power(b,3) - 2*a*b*c + Power(b,2)*c - a*Power(c,2)) + Power(ay,3)*(Power(b,3) + 3*Power(b,2)*c + Power(c,2)*(-3*a + c) + b*c*(-2*a + 3*c))) + ax*(-3*ay*Power(by,2)*(b + c)*(Power(b,2) - 2*a*c + b*c) + Power(by,3)*(Power(b,3) - 2*a*b*c + Power(b,2)*c - a*Power(c,2)) - Power(ay,3)*(b + 2*c)*(Power(b,2) + 2*b*c + 2*c*(-a + c)) + 3*Power(ay,2)*by*(Power(b,3) + 3*Power(b,2)*c + Power(c,2)*(-3*a + c) + b*c*(-2*a + 3*c))))*Log(r6))/(6.*Power(c,4));
		W(5,5) = (2*Power(ay - by,4)*Power(c,3) - 3*Power(ay - by,3)*Power(c,2)*(-(b*by) + ay*(b + 4*c)) + 6*Power(ay - by,2)*c*(Power(by,2)*(Power(b,2) - a*c) - 2*ay*by*(Power(b,2) - a*c + 2*b*c) + Power(ay,2)*(Power(b,2) + 4*b*c + c*(-a + 6*c))) - (6*(Power(by,4)*(Power(b,4) - 4*a*Power(b,2)*c + 2*Power(a,2)*Power(c,2)) - 4*ay*Power(by,3)*(Power(b,4) - 4*a*Power(b,2)*c + Power(b,3)*c + 2*Power(a,2)*Power(c,2) - 3*a*b*Power(c,2)) + 6*Power(ay,2)*Power(by,2)*(Power(b,4) + 2*Power(b,3)*c - 6*a*b*Power(c,2) + 2*a*(a - c)*Power(c,2) + Power(b,2)*c*(-4*a + c)) - 4*Power(ay,3)*by*(Power(b,4) + 3*Power(b,3)*c + 2*a*(a - 3*c)*Power(c,2) + b*Power(c,2)*(-9*a + c) + Power(b,2)*c*(-4*a + 3*c)) + Power(ay,4)*(Power(b,4) + 4*Power(b,3)*c + 4*b*Power(c,2)*(-3*a + c) + 2*Power(c,2)*(Power(a,2) - 6*a*c + Power(c,2)) + Power(b,2)*(-4*a*c + 6*Power(c,2))))*r2)/r1 + (6*(Power(by,4)*(Power(b,4) - 4*a*Power(b,2)*c + 2*Power(a,2)*Power(c,2)) - 4*ay*Power(by,3)*(Power(b,4) - 4*a*Power(b,2)*c + Power(b,3)*c + 2*Power(a,2)*Power(c,2) - 3*a*b*Power(c,2)) + 6*Power(ay,2)*Power(by,2)*(Power(b,4) + 2*Power(b,3)*c - 6*a*b*Power(c,2) + 2*a*(a - c)*Power(c,2) + Power(b,2)*c*(-4*a + c)) - 4*Power(ay,3)*by*(Power(b,4) + 3*Power(b,3)*c + 2*a*(a - 3*c)*Power(c,2) + b*Power(c,2)*(-9*a + c) + Power(b,2)*c*(-4*a + 3*c)) + Power(ay,4)*(Power(b,4) + 4*Power(b,3)*c + 4*b*Power(c,2)*(-3*a + c) + 2*Power(c,2)*(Power(a,2) - 6*a*c + Power(c,2)) + Power(b,2)*(-4*a*c + 6*Power(c,2))))*r3)/r1 + 3*(ay - by)*(-(b*by) + ay*(b + 2*c))*(Power(by,2)*(Power(b,2) - 2*a*c) - 2*ay*by*(Power(b,2) - 2*a*c + b*c) + Power(ay,2)*(Power(b,2) + 2*b*c + 2*c*(-a + c)))*Log(a) - 3*(ay - by)*(-(b*by) + ay*(b + 2*c))*(Power(by,2)*(Power(b,2) - 2*a*c) - 2*ay*by*(Power(b,2) - 2*a*c + b*c) + Power(ay,2)*(Power(b,2) + 2*b*c + 2*c*(-a + c)))*Log(r6))/(6.*Power(c,4));

		M += W*sqrt_c;

		// add derivative matrix P'tP'
		if (g.deriv >= 1)
		{
		W(0,0) = 0;
		W(0,1) = 0;
		W(0,2) = 0;
		W(0,3) = 0;
		W(0,4) = 0;
		W(0,5) = 0;
		W(1,1) = (-2*Power(ay - by,2)*(r2 - r3))/r1;
		W(1,2) = (2*(ax - bx)*(ay - by)*(r2 - r3))/r1;
		W(1,3) = (Power(ay - by,2)*(-2*(-(b*bx) + ax*(b + 2*c))*r2 + 2*(-(b*bx) + ax*(b + 2*c))*r3 + (ax - bx)*r1*(Log(a) - Log(r6))))/(c*r1);
		W(1,4) = ((ay - by)*((2*(b*(Power(bx,2) - Power(by,2)) - 2*ax*bx*(b + c) + 2*ay*by*(b + c) + Power(ax,2)*(b + 2*c) - Power(ay,2)*(b + 2*c))*r2)/r1 - (2*(b*(Power(bx,2) - Power(by,2)) - 2*ax*bx*(b + c) + 2*ay*by*(b + c) + Power(ax,2)*(b + 2*c) - Power(ay,2)*(b + 2*c))*r3)/r1 - (Power(ax,2) - Power(ay,2) - 2*ax*bx + Power(bx,2) + 2*ay*by - Power(by,2))*Log(a) + (Power(ax,2) - Power(ay,2) - 2*ax*bx + Power(bx,2) + 2*ay*by - Power(by,2))*Log(r6)))/(2.*c);
		W(1,5) = ((ax - bx)*(ay - by)*(2*(-(b*by) + ay*(b + 2*c))*r2 - 2*(-(b*by) + ay*(b + 2*c))*r3 - (ay - by)*r1*(Log(a) - Log(r6))))/(c*r1);
		W(2,2) = (-2*Power(ax - bx,2)*(r2 - r3))/r1;
		W(2,3) = ((ax - bx)*(ay - by)*(2*(-(b*bx) + ax*(b + 2*c))*r2 - 2*(-(b*bx) + ax*(b + 2*c))*r3 - (ax - bx)*r1*(Log(a) - Log(r6))))/(c*r1);
		W(2,4) = ((-ax + bx)*((2*(b*(Power(bx,2) - Power(by,2)) - 2*ax*bx*(b + c) + 2*ay*by*(b + c) + Power(ax,2)*(b + 2*c) - Power(ay,2)*(b + 2*c))*r2)/r1 - (2*(b*(Power(bx,2) - Power(by,2)) - 2*ax*bx*(b + c) + 2*ay*by*(b + c) + Power(ax,2)*(b + 2*c) - Power(ay,2)*(b + 2*c))*r3)/r1 - (Power(ax,2) - Power(ay,2) - 2*ax*bx + Power(bx,2) + 2*ay*by - Power(by,2))*Log(a) + (Power(ax,2) - Power(ay,2) - 2*ax*bx + Power(bx,2) + 2*ay*by - Power(by,2))*Log(r6)))/(2.*c);
		W(2,5) = (Power(ax - bx,2)*(-2*(-(b*by) + ay*(b + 2*c))*r2 + 2*(-(b*by) + ay*(b + 2*c))*r3 + (ay - by)*r1*(Log(a) - Log(r6))))/(c*r1);
		W(3,3) = (2*Power(ay - by,2)*(2*Power(ax - bx,2)*c - (2*(Power(bx,2)*(Power(b,2) - 2*a*c) - 2*ax*bx*(Power(b,2) - 2*a*c + b*c) + Power(ax,2)*(Power(b,2) + 2*b*c + 2*c*(-a + c)))*r2)/r1 + (2*(Power(bx,2)*(Power(b,2) - 2*a*c) - 2*ax*bx*(Power(b,2) - 2*a*c + b*c) + Power(ax,2)*(Power(b,2) + 2*b*c + 2*c*(-a + c)))*r3)/r1 + (ax - bx)*(-(b*bx) + ax*(b + 2*c))*Log(a) - (ax - bx)*(-(b*bx) + ax*(b + 2*c))*Log(r6)))/Power(c,2);
		W(3,4) = ((ay - by)*(-2*(ax - bx)*(Power(ax,2) - Power(ay,2) - 2*ax*bx + Power(bx,2) + 2*ay*by - Power(by,2))*c + (2*(-(Power(ax,2)*bx*(3*Power(b,2) + 4*b*c + 2*c*(-3*a + c))) + Power(ax,3)*(Power(b,2) + 2*b*c + 2*c*(-a + c)) + bx*(-((Power(bx,2) - Power(by,2))*(Power(b,2) - 2*a*c)) - ay*by*(2*Power(b,2) - 4*a*c + b*c) + Power(ay,2)*(Power(b,2) - 2*a*c + b*c)) - ax*(Power(b,2)*(-3*Power(bx,2) + Power(by,2)) + 2*a*(3*Power(bx,2) - Power(by,2))*c + b*(-2*Power(bx,2) + Power(by,2))*c - ay*by*(2*Power(b,2) + 3*b*c + 2*c*(-2*a + c)) + Power(ay,2)*(Power(b,2) + 2*b*c + 2*c*(-a + c))))*r2)/r1 - (2*(-(Power(ax,2)*bx*(3*Power(b,2) + 4*b*c + 2*c*(-3*a + c))) + Power(ax,3)*(Power(b,2) + 2*b*c + 2*c*(-a + c)) + bx*(-((Power(bx,2) - Power(by,2))*(Power(b,2) - 2*a*c)) - ay*by*(2*Power(b,2) - 4*a*c + b*c) + Power(ay,2)*(Power(b,2) - 2*a*c + b*c)) - ax*(Power(b,2)*(-3*Power(bx,2) + Power(by,2)) + 2*a*(3*Power(bx,2) - Power(by,2))*c + b*(-2*Power(bx,2) + Power(by,2))*c - ay*by*(2*Power(b,2) + 3*b*c + 2*c*(-2*a + c)) + Power(ay,2)*(Power(b,2) + 2*b*c + 2*c*(-a + c))))*r3)/r1 - (Power(ax,3)*(b + 2*c) - Power(ax,2)*bx*(3*b + 4*c) + bx*(b*(-Power(bx,2) + Power(by,2)) + Power(ay,2)*(b + c) - ay*by*(2*b + c)) - ax*(-3*b*Power(bx,2) + b*Power(by,2) - 2*Power(bx,2)*c + Power(by,2)*c + Power(ay,2)*(b + 2*c) - ay*by*(2*b + 3*c)))*Log(a) + (Power(ax,3)*(b + 2*c) - Power(ax,2)*bx*(3*b + 4*c) + bx*(b*(-Power(bx,2) + Power(by,2)) + Power(ay,2)*(b + c) - ay*by*(2*b + c)) - ax*(-3*b*Power(bx,2) + b*Power(by,2) - 2*Power(bx,2)*c + Power(by,2)*c + Power(ay,2)*(b + 2*c) - ay*by*(2*b + 3*c)))*Log(r6)))/Power(c,2);
		W(3,5) = (2*(-ax + bx)*(ay - by)*(2*(ax - bx)*(ay - by)*c - (2*(bx*(by*(Power(b,2) - 2*a*c) - ay*(Power(b,2) - 2*a*c + b*c)) + ax*(-(by*(Power(b,2) - 2*a*c + b*c)) + ay*(Power(b,2) + 2*b*c + 2*c*(-a + c))))*r2)/r1 + (2*(bx*(by*(Power(b,2) - 2*a*c) - ay*(Power(b,2) - 2*a*c + b*c)) + ax*(-(by*(Power(b,2) - 2*a*c + b*c)) + ay*(Power(b,2) + 2*b*c + 2*c*(-a + c))))*r3)/r1 - (bx*(-(b*by) + ay*(b + c)) + ax*(by*(b + c) - ay*(b + 2*c)))*Log(a) + (bx*(-(b*by) + ay*(b + c)) + ax*(by*(b + c) - ay*(b + 2*c)))*Log(r6)))/Power(c,2);
		W(4,4) = (2*Power(Power(ax,2) - Power(ay,2) - 2*ax*bx + Power(bx,2) + 2*ay*by - Power(by,2),2)*c - (2*(Power(Power(bx,2) - Power(by,2),2)*(Power(b,2) - 2*a*c) + 2*ay*by*(Power(bx,2) - Power(by,2))*(2*Power(b,2) - 4*a*c + b*c) - 2*Power(ax,3)*bx*(2*Power(b,2) + 3*b*c + 2*c*(-2*a + c)) - 2*Power(ay,3)*by*(2*Power(b,2) + 3*b*c + 2*c*(-2*a + c)) + Power(ax,4)*(Power(b,2) + 2*b*c + 2*c*(-a + c)) + Power(ay,4)*(Power(b,2) + 2*b*c + 2*c*(-a + c)) - 2*Power(ay,2)*(Power(b,2)*(Power(bx,2) - 3*Power(by,2)) + b*(Power(bx,2) - 3*Power(by,2))*c - c*(2*a*(Power(bx,2) - 3*Power(by,2)) + Power(by,2)*c)) + 2*ax*bx*(-((Power(bx,2) - Power(by,2))*(2*Power(b,2) - 4*a*c + b*c)) - 2*ay*by*(2*Power(b,2) + 2*b*c + c*(-4*a + c)) + Power(ay,2)*(2*Power(b,2) + 3*b*c + 2*c*(-2*a + c))) - 2*Power(ax,2)*(Power(b,2)*(-3*Power(bx,2) + Power(by,2)) + b*(-3*Power(bx,2) + Power(by,2))*c - c*(-6*a*Power(bx,2) + 2*a*Power(by,2) + Power(bx,2)*c) - ay*by*(2*Power(b,2) + 3*b*c + 2*c*(-2*a + c)) + Power(ay,2)*(Power(b,2) + 2*b*c + 2*c*(-a + c))))*r2)/r1 + (2*(Power(Power(bx,2) - Power(by,2),2)*(Power(b,2) - 2*a*c) + 2*ay*by*(Power(bx,2) - Power(by,2))*(2*Power(b,2) - 4*a*c + b*c) - 2*Power(ax,3)*bx*(2*Power(b,2) + 3*b*c + 2*c*(-2*a + c)) - 2*Power(ay,3)*by*(2*Power(b,2) + 3*b*c + 2*c*(-2*a + c)) + Power(ax,4)*(Power(b,2) + 2*b*c + 2*c*(-a + c)) + Power(ay,4)*(Power(b,2) + 2*b*c + 2*c*(-a + c)) - 2*Power(ay,2)*(Power(b,2)*(Power(bx,2) - 3*Power(by,2)) + b*(Power(bx,2) - 3*Power(by,2))*c - c*(2*a*(Power(bx,2) - 3*Power(by,2)) + Power(by,2)*c)) + 2*ax*bx*(-((Power(bx,2) - Power(by,2))*(2*Power(b,2) - 4*a*c + b*c)) - 2*ay*by*(2*Power(b,2) + 2*b*c + c*(-4*a + c)) + Power(ay,2)*(2*Power(b,2) + 3*b*c + 2*c*(-2*a + c))) - 2*Power(ax,2)*(Power(b,2)*(-3*Power(bx,2) + Power(by,2)) + b*(-3*Power(bx,2) + Power(by,2))*c - c*(-6*a*Power(bx,2) + 2*a*Power(by,2) + Power(bx,2)*c) - ay*by*(2*Power(b,2) + 3*b*c + 2*c*(-2*a + c)) + Power(ay,2)*(Power(b,2) + 2*b*c + 2*c*(-a + c))))*r3)/r1 + (Power(ax,2) - Power(ay,2) - 2*ax*bx + Power(bx,2) + 2*ay*by - Power(by,2))*(b*(Power(bx,2) - Power(by,2)) - 2*ax*bx*(b + c) + 2*ay*by*(b + c) + Power(ax,2)*(b + 2*c) - Power(ay,2)*(b + 2*c))*Log(a) - (Power(ax,2) - Power(ay,2) - 2*ax*bx + Power(bx,2) + 2*ay*by - Power(by,2))*(b*(Power(bx,2) - Power(by,2)) - 2*ax*bx*(b + c) + 2*ay*by*(b + c) + Power(ax,2)*(b + 2*c) - Power(ay,2)*(b + 2*c))*Log(r6))/(2.*Power(c,2));
		W(4,5) = ((-ax + bx)*(2*(ay - by)*(-Power(ax,2) + Power(ay,2) + 2*ax*bx - Power(bx,2) - 2*ay*by + Power(by,2))*c + (2*(by*(-Power(bx,2) + Power(by,2))*(Power(b,2) - 2*a*c) + ay*(Power(b,2)*(Power(bx,2) - 3*Power(by,2)) - 2*a*(Power(bx,2) - 3*Power(by,2))*c + b*(Power(bx,2) - 2*Power(by,2))*c) + Power(ay,2)*by*(3*Power(b,2) + 4*b*c + 2*c*(-3*a + c)) - Power(ay,3)*(Power(b,2) + 2*b*c + 2*c*(-a + c)) + ax*bx*(by*(2*Power(b,2) - 4*a*c + b*c) - ay*(2*Power(b,2) + 3*b*c + 2*c*(-2*a + c))) + Power(ax,2)*(-(by*(Power(b,2) - 2*a*c + b*c)) + ay*(Power(b,2) + 2*b*c + 2*c*(-a + c))))*r2)/r1 - (2*(by*(-Power(bx,2) + Power(by,2))*(Power(b,2) - 2*a*c) + ay*(Power(b,2)*(Power(bx,2) - 3*Power(by,2)) - 2*a*(Power(bx,2) - 3*Power(by,2))*c + b*(Power(bx,2) - 2*Power(by,2))*c) + Power(ay,2)*by*(3*Power(b,2) + 4*b*c + 2*c*(-3*a + c)) - Power(ay,3)*(Power(b,2) + 2*b*c + 2*c*(-a + c)) + ax*bx*(by*(2*Power(b,2) - 4*a*c + b*c) - ay*(2*Power(b,2) + 3*b*c + 2*c*(-2*a + c))) + Power(ax,2)*(-(by*(Power(b,2) - 2*a*c + b*c)) + ay*(Power(b,2) + 2*b*c + 2*c*(-a + c))))*r3)/r1 - (b*by*(-Power(bx,2) + Power(by,2)) - Power(ay,3)*(b + 2*c) + Power(ay,2)*by*(3*b + 4*c) + ax*bx*(-2*ay*b + 2*b*by - 3*ay*c + by*c) + ay*(b*Power(bx,2) - 3*b*Power(by,2) + Power(bx,2)*c - 2*Power(by,2)*c) + Power(ax,2)*(-(by*(b + c)) + ay*(b + 2*c)))*Log(a) + (b*by*(-Power(bx,2) + Power(by,2)) - Power(ay,3)*(b + 2*c) + Power(ay,2)*by*(3*b + 4*c) + ax*bx*(-2*ay*b + 2*b*by - 3*ay*c + by*c) + ay*(b*Power(bx,2) - 3*b*Power(by,2) + Power(bx,2)*c - 2*Power(by,2)*c) + Power(ax,2)*(-(by*(b + c)) + ay*(b + 2*c)))*Log(r6)))/Power(c,2);
		W(5,5) = (2*Power(ax - bx,2)*(2*Power(ay - by,2)*c - (2*(Power(by,2)*(Power(b,2) - 2*a*c) - 2*ay*by*(Power(b,2) - 2*a*c + b*c) + Power(ay,2)*(Power(b,2) + 2*b*c + 2*c*(-a + c)))*r2)/r1 + (2*(Power(by,2)*(Power(b,2) - 2*a*c) - 2*ay*by*(Power(b,2) - 2*a*c + b*c) + Power(ay,2)*(Power(b,2) + 2*b*c + 2*c*(-a + c)))*r3)/r1 + (ay - by)*(-(b*by) + ay*(b + 2*c))*Log(a) - (ay - by)*(-(b*by) + ay*(b + 2*c))*Log(r6)))/Power(c,2);

		M += W*(1.0/sqrt_c);
		}

		// add second derivative matrix P''tP''
		if (g.deriv >= 2)
		{
		W(0,0) = 0;
		W(0,1) = 0;
		W(0,2) = 0;
		W(0,3) = 0;
		W(0,4) = 0;
		W(0,5) = 0;
		W(1,1) = 0;
		W(1,2) = 0;
		W(1,3) = 0;
		W(1,4) = 0;
		W(1,5) = 0;
		W(2,2) = 0;
		W(2,3) = 0;
		W(2,4) = 0;
		W(2,5) = 0;
		W(3,3) = (8*Power(ay - by,4)*(-r2 + r3))/r1;
		W(3,4) = (8*(ax - bx)*Power(ay - by,3)*(r2 - r3))/r1;
		W(3,5) = (8*Power(ax - bx,2)*Power(ay - by,2)*(-r2 + r3))/r1;
		W(4,4) = (8*Power(ax - bx,2)*Power(ay - by,2)*(-r2 + r3))/r1;
		W(4,5) = (8*Power(ax - bx,3)*(ay - by)*(r2 - r3))/r1;
		W(5,5) = (8*Power(ax - bx,4)*(-r2 + r3))/r1;

		M += W*(1.0/(c*sqrt_c));
		}

		// copy symmetric parts
		M(1,0) = M(0,1);
		M(2,0) = M(0,2);
		M(3,0) = M(0,3);
		M(4,0) = M(0,4);
		M(5,0) = M(0,5);
		M(2,1) = M(1,2);
		M(3,1) = M(1,3);
		M(4,1) = M(1,4);
		M(5,1) = M(1,5);
		M(3,2) = M(2,3);
		M(4,2) = M(2,4);
		M(5,2) = M(2,5);
		M(4,3) = M(3,4);
		M(5,3) = M(3,5);
		M(5,4) = M(4,5);
	}

	Matrixd X(1,6);
	X(0,0) = 1;
	X(0,1) = x[0];
	X(0,2) = x[1];
	X(0,3) = x[0]*x[0];
	X(0,4) = x[1]*x[0];
	X(0,5) = x[1]*x[1];

	Matrixd k = X*M.pseudo_inverse();

	// calculate barycentric coordinates
	Matrixd B(6,1);
	for (int i = 0; i < n; i++)
	{
		// calculate some common terms
		vect2d &p1 = pv[p[i][0]];
		vect2d &p2 = pv[p[i][1]];
		double ax = p1[0];
		double ay = p1[1];
		double bx = p2[0];
		double by = p2[1];

		double a = (p1 - x)*(p1 - x);
		double b = (p1 - p2)*(p1 - x)*(-2);
		double c = (p1 - p2)*(p1 - p2);

		double sqrt_c = sqrt(c);

		double r5 = -b*b + 4*a*c;
		double r1 = sqrt(r5);
		double r4 = 1.0 / (r1*r5);
		double r2 = atan(b/r1);
		double r3 = atan((b+2*c)/r1);
		double r6 = a+b+c;

		// calc function coordinates
		B(0,0) = (-2*(b + 2*c)*r2 + 2*(b + 2*c)*r3 + r1*(Log(a) - Log(r6)))/(2.*c*r1);
		B(1,0) = (2*(ax - bx)*c + (2*(bx*(Power(b,2) - 2*a*c + b*c) - ax*(Power(b,2) + 2*b*c + 2*c*(-a + c)))*r2)/r1 + (2*(-(bx*(Power(b,2) - 2*a*c + b*c)) + ax*(Power(b,2) + 2*b*c + 2*c*(-a + c)))*r3)/r1 - (bx*(b + c) - ax*(b + 2*c))*Log(a) + (bx*(b + c) - ax*(b + 2*c))*Log(r6))/(2.*Power(c,2));
		B(2,0) = (2*(ay - by)*c + (2*(by*(Power(b,2) - 2*a*c + b*c) - ay*(Power(b,2) + 2*b*c + 2*c*(-a + c)))*r2)/r1 + (2*(-(by*(Power(b,2) - 2*a*c + b*c)) + ay*(Power(b,2) + 2*b*c + 2*c*(-a + c)))*r3)/r1 - (by*(b + c) - ay*(b + 2*c))*Log(a) + (by*(b + c) - ay*(b + 2*c))*Log(r6))/(2.*Power(c,2));
		B(3,0) = (-(Power(ax - bx,2)*Power(c,2)) + 2*(ax - bx)*c*(-(bx*(b + c)) + ax*(b + 3*c)) - (2*(Power(bx,2)*(Power(b,3) - 3*a*b*c + Power(b,2)*c - 2*a*Power(c,2)) + Power(ax,2)*(b + 2*c)*(Power(b,2) + b*c + c*(-3*a + c)) - 2*ax*bx*(Power(b,3) + 2*Power(b,2)*c - 4*a*Power(c,2) + b*c*(-3*a + c)))*r2)/r1 + (2*(Power(bx,2)*(Power(b,3) - 3*a*b*c + Power(b,2)*c - 2*a*Power(c,2)) + Power(ax,2)*(b + 2*c)*(Power(b,2) + b*c + c*(-3*a + c)) - 2*ax*bx*(Power(b,3) + 2*Power(b,2)*c - 4*a*Power(c,2) + b*c*(-3*a + c)))*r3)/r1 + (Power(bx,2)*(Power(b,2) - a*c + b*c) - 2*ax*bx*(Power(b,2) + 2*b*c + c*(-a + c)) + Power(ax,2)*(Power(b,2) + 3*b*c + c*(-a + 3*c)))*Log(a) - (Power(bx,2)*(Power(b,2) - a*c + b*c) - 2*ax*bx*(Power(b,2) + 2*b*c + c*(-a + c)) + Power(ax,2)*(Power(b,2) + 3*b*c + c*(-a + 3*c)))*Log(r6))/(2.*Power(c,3));
		B(4,0) = (-((ax - bx)*(ay - by)*Power(c,2)) + 2*c*(bx*(by*(b + c) - ay*(b + 2*c)) + ax*(-(by*(b + 2*c)) + ay*(b + 3*c))) - (2*(bx*(by*(Power(b,3) - 3*a*b*c + Power(b,2)*c - 2*a*Power(c,2)) - ay*(Power(b,3) + 2*Power(b,2)*c - 4*a*Power(c,2) + b*c*(-3*a + c))) + ax*(ay*(b + 2*c)*(Power(b,2) + b*c + c*(-3*a + c)) - by*(Power(b,3) + 2*Power(b,2)*c - 4*a*Power(c,2) + b*c*(-3*a + c))))*r2)/r1 + (2*(bx*(by*(Power(b,3) - 3*a*b*c + Power(b,2)*c - 2*a*Power(c,2)) - ay*(Power(b,3) + 2*Power(b,2)*c - 4*a*Power(c,2) + b*c*(-3*a + c))) + ax*(ay*(b + 2*c)*(Power(b,2) + b*c + c*(-3*a + c)) - by*(Power(b,3) + 2*Power(b,2)*c - 4*a*Power(c,2) + b*c*(-3*a + c))))*r3)/r1 + (bx*(by*(Power(b,2) - a*c + b*c) - ay*(Power(b,2) - a*c + 2*b*c + Power(c,2))) + ax*(-(by*(Power(b,2) - a*c + 2*b*c + Power(c,2))) + ay*(Power(b,2) - a*c + 3*b*c + 3*Power(c,2))))*Log(a) - (bx*(by*(Power(b,2) - a*c + b*c) - ay*(Power(b,2) - a*c + 2*b*c + Power(c,2))) + ax*(-(by*(Power(b,2) - a*c + 2*b*c + Power(c,2))) + ay*(Power(b,2) - a*c + 3*b*c + 3*Power(c,2))))*Log(r6))/(2.*Power(c,3));
		B(5,0) = (-(Power(ay - by,2)*Power(c,2)) + 2*(ay - by)*c*(-(by*(b + c)) + ay*(b + 3*c)) - (2*(Power(by,2)*(Power(b,3) - 3*a*b*c + Power(b,2)*c - 2*a*Power(c,2)) + Power(ay,2)*(b + 2*c)*(Power(b,2) + b*c + c*(-3*a + c)) - 2*ay*by*(Power(b,3) + 2*Power(b,2)*c - 4*a*Power(c,2) + b*c*(-3*a + c)))*r2)/r1 + (2*(Power(by,2)*(Power(b,3) - 3*a*b*c + Power(b,2)*c - 2*a*Power(c,2)) + Power(ay,2)*(b + 2*c)*(Power(b,2) + b*c + c*(-3*a + c)) - 2*ay*by*(Power(b,3) + 2*Power(b,2)*c - 4*a*Power(c,2) + b*c*(-3*a + c)))*r3)/r1 + (Power(by,2)*(Power(b,2) - a*c + b*c) - 2*ay*by*(Power(b,2) + 2*b*c + c*(-a + c)) + Power(ay,2)*(Power(b,2) + 3*b*c + c*(-a + 3*c)))*Log(a) - (Power(by,2)*(Power(b,2) - a*c + b*c) - 2*ay*by*(Power(b,2) + 2*b*c + c*(-a + c)) + Power(ay,2)*(Power(b,2) + 3*b*c + c*(-a + 3*c)))*Log(r6))/(2.*Power(c,3));
		bary[p[i][0]] += (k*B) * sqrt_c;

		B(0,0) = ((2*b*r2)/r1 - (2*b*r3)/r1 - Log(a) + Log(r6))/(2.*c);
		B(1,0) = (2*(-ax + bx)*c + (2*(-(bx*(Power(b,2) - 2*a*c)) + ax*(Power(b,2) - 2*a*c + b*c))*r2)/r1 - (2*(-(bx*(Power(b,2) - 2*a*c)) + ax*(Power(b,2) - 2*a*c + b*c))*r3)/r1 - (-(b*bx) + ax*(b + c))*Log(a) + (-(b*bx) + ax*(b + c))*Log(r6))/(2.*Power(c,2));
		B(2,0) = (2*(-ay + by)*c + (2*(-(by*(Power(b,2) - 2*a*c)) + ay*(Power(b,2) - 2*a*c + b*c))*r2)/r1 - (2*(-(by*(Power(b,2) - 2*a*c)) + ay*(Power(b,2) - 2*a*c + b*c))*r3)/r1 - (-(b*by) + ay*(b + c))*Log(a) + (-(b*by) + ay*(b + c))*Log(r6))/(2.*Power(c,2));
		B(3,0) = (Power(ax - bx,2)*Power(c,2) - 2*(ax - bx)*c*(-(b*bx) + ax*(b + 2*c)) + (2*(b*Power(bx,2)*(Power(b,2) - 3*a*c) - 2*ax*bx*(Power(b,3) - 3*a*b*c + Power(b,2)*c - 2*a*Power(c,2)) + Power(ax,2)*(Power(b,3) + 2*Power(b,2)*c - 4*a*Power(c,2) + b*c*(-3*a + c)))*r2)/r1 - (2*(b*Power(bx,2)*(Power(b,2) - 3*a*c) - 2*ax*bx*(Power(b,3) - 3*a*b*c + Power(b,2)*c - 2*a*Power(c,2)) + Power(ax,2)*(Power(b,3) + 2*Power(b,2)*c - 4*a*Power(c,2) + b*c*(-3*a + c)))*r3)/r1 - (Power(bx,2)*(Power(b,2) - a*c) - 2*ax*bx*(Power(b,2) - a*c + b*c) + Power(ax,2)*(Power(b,2) + 2*b*c + c*(-a + c)))*Log(a) + (Power(bx,2)*(Power(b,2) - a*c) - 2*ax*bx*(Power(b,2) - a*c + b*c) + Power(ax,2)*(Power(b,2) + 2*b*c + c*(-a + c)))*Log(r6))/(2.*Power(c,3));
		B(4,0) = ((ax - bx)*(ay - by)*Power(c,2) + 2*c*(bx*(-(b*by) + ay*(b + c)) + ax*(by*(b + c) - ay*(b + 2*c))) + (2*(bx*(b*by*(Power(b,2) - 3*a*c) - ay*(Power(b,3) - 3*a*b*c + Power(b,2)*c - 2*a*Power(c,2))) + ax*(-(by*(Power(b,3) - 3*a*b*c + Power(b,2)*c - 2*a*Power(c,2))) + ay*(Power(b,3) + 2*Power(b,2)*c - 4*a*Power(c,2) + b*c*(-3*a + c))))*r2)/r1 - (2*(bx*(b*by*(Power(b,2) - 3*a*c) - ay*(Power(b,3) - 3*a*b*c + Power(b,2)*c - 2*a*Power(c,2))) + ax*(-(by*(Power(b,3) - 3*a*b*c + Power(b,2)*c - 2*a*Power(c,2))) + ay*(Power(b,3) + 2*Power(b,2)*c - 4*a*Power(c,2) + b*c*(-3*a + c))))*r3)/r1 - (-(bx*(-(Power(b,2)*by) + a*by*c + ay*(Power(b,2) - a*c + b*c))) + ax*(-(by*(Power(b,2) - a*c + b*c)) + ay*(Power(b,2) - a*c + 2*b*c + Power(c,2))))*Log(a) + (-(bx*(-(Power(b,2)*by) + a*by*c + ay*(Power(b,2) - a*c + b*c))) + ax*(-(by*(Power(b,2) - a*c + b*c)) + ay*(Power(b,2) - a*c + 2*b*c + Power(c,2))))*Log(r6))/(2.*Power(c,3));
		B(5,0) = (Power(ay - by,2)*Power(c,2) - 2*(ay - by)*c*(-(b*by) + ay*(b + 2*c)) + (2*(b*Power(by,2)*(Power(b,2) - 3*a*c) - 2*ay*by*(Power(b,3) - 3*a*b*c + Power(b,2)*c - 2*a*Power(c,2)) + Power(ay,2)*(Power(b,3) + 2*Power(b,2)*c - 4*a*Power(c,2) + b*c*(-3*a + c)))*r2)/r1 - (2*(b*Power(by,2)*(Power(b,2) - 3*a*c) - 2*ay*by*(Power(b,3) - 3*a*b*c + Power(b,2)*c - 2*a*Power(c,2)) + Power(ay,2)*(Power(b,3) + 2*Power(b,2)*c - 4*a*Power(c,2) + b*c*(-3*a + c)))*r3)/r1 - (Power(by,2)*(Power(b,2) - a*c) - 2*ay*by*(Power(b,2) - a*c + b*c) + Power(ay,2)*(Power(b,2) + 2*b*c + c*(-a + c)))*Log(a) + (Power(by,2)*(Power(b,2) - a*c) - 2*ay*by*(Power(b,2) - a*c + b*c) + Power(ay,2)*(Power(b,2) + 2*b*c + c*(-a + c)))*Log(r6))/(2.*Power(c,3));
		bary[p[i][1]] += (k*B) * sqrt_c;

		// calc derivative coordinates
		if (g.deriv == 1)
		{
		B(0,0) = 0;
		B(1,0) = ((ay - by)*(2*c - (2*(Power(b,2) + 2*b*c + 2*c*(-a + c))*r2)/r1 + (2*(Power(b,2) + 2*b*c + 2*c*(-a + c))*r3)/r1 + (b + 2*c)*Log(a) - (b + 2*c)*Log(r6)))/(2.*Power(c,2));
		B(2,0) = ((ax - bx)*(-2*c + (2*(Power(b,2) + 2*b*c + 2*c*(-a + c))*r2)/r1 - (2*(Power(b,2) + 2*b*c + 2*c*(-a + c))*r3)/r1 - (b + 2*c)*Log(a) + (b + 2*c)*Log(r6)))/(2.*Power(c,2));
		B(3,0) = ((ay - by)*((ax - bx)*Power(c,2) + 2*c*(-(bx*(b + c)) + ax*(b + 2*c)) - (2*(ax*(b + 2*c)*(Power(b,2) + b*c + c*(-3*a + c)) - bx*(Power(b,3) + 2*Power(b,2)*c - 4*a*Power(c,2) + b*c*(-3*a + c)))*r2)/r1 + (2*(ax*(b + 2*c)*(Power(b,2) + b*c + c*(-3*a + c)) - bx*(Power(b,3) + 2*Power(b,2)*c - 4*a*Power(c,2) + b*c*(-3*a + c)))*r3)/r1 + (-(bx*(Power(b,2) + 2*b*c + c*(-a + c))) + ax*(Power(b,2) + 3*b*c + c*(-a + 3*c)))*Log(a) - (-(bx*(Power(b,2) + 2*b*c + c*(-a + c))) + ax*(Power(b,2) + 3*b*c + c*(-a + 3*c)))*Log(r6)))/Power(c,3);
		B(4,0) = (-((Power(ax,2) - Power(ay,2) - 2*ax*bx + Power(bx,2) + 2*ay*by - Power(by,2))*Power(c,2)) + 2*c*(-((Power(bx,2) - Power(by,2))*(b + c)) - Power(ax,2)*(b + 2*c) + Power(ay,2)*(b + 2*c) + ax*bx*(2*b + 3*c) - ay*by*(2*b + 3*c)) + (2*(Power(ax,2)*(b + 2*c)*(Power(b,2) + b*c + c*(-3*a + c)) - Power(ay,2)*(b + 2*c)*(Power(b,2) + b*c + c*(-3*a + c)) + (Power(bx,2) - Power(by,2))*(Power(b,3) + 2*Power(b,2)*c - 4*a*Power(c,2) + b*c*(-3*a + c)) - ax*bx*(2*Power(b,3) + 5*Power(b,2)*c + 2*Power(c,2)*(-5*a + c) + b*(-6*a*c + 4*Power(c,2))) + ay*by*(2*Power(b,3) + 5*Power(b,2)*c + 2*Power(c,2)*(-5*a + c) + b*(-6*a*c + 4*Power(c,2))))*r2)/r1 - (2*(Power(ax,2)*(b + 2*c)*(Power(b,2) + b*c + c*(-3*a + c)) - Power(ay,2)*(b + 2*c)*(Power(b,2) + b*c + c*(-3*a + c)) + (Power(bx,2) - Power(by,2))*(Power(b,3) + 2*Power(b,2)*c - 4*a*Power(c,2) + b*c*(-3*a + c)) - ax*bx*(2*Power(b,3) + 5*Power(b,2)*c + 2*Power(c,2)*(-5*a + c) + b*(-6*a*c + 4*Power(c,2))) + ay*by*(2*Power(b,3) + 5*Power(b,2)*c + 2*Power(c,2)*(-5*a + c) + b*(-6*a*c + 4*Power(c,2))))*r3)/r1 - (-(ax*bx*(2*Power(b,2) + 5*b*c - 2*(a - 2*c)*c)) + (Power(bx,2) - Power(by,2))*(Power(b,2) - a*c + 2*b*c + Power(c,2)) - Power(ay,2)*(Power(b,2) - a*c + 3*b*c + 3*Power(c,2)) + ay*by*(2*Power(b,2) - 2*a*c + 5*b*c + 4*Power(c,2)) + Power(ax,2)*(Power(b,2) + 3*b*c + c*(-a + 3*c)))*Log(a) + (-(ax*bx*(2*Power(b,2) + 5*b*c - 2*(a - 2*c)*c)) + (Power(bx,2) - Power(by,2))*(Power(b,2) - a*c + 2*b*c + Power(c,2)) - Power(ay,2)*(Power(b,2) - a*c + 3*b*c + 3*Power(c,2)) + ay*by*(2*Power(b,2) - 2*a*c + 5*b*c + 4*Power(c,2)) + Power(ax,2)*(Power(b,2) + 3*b*c + c*(-a + 3*c)))*Log(r6))/(2.*Power(c,3));
		B(5,0) = ((ax - bx)*(-((ay - by)*Power(c,2)) - 2*c*(-(by*(b + c)) + ay*(b + 2*c)) + (2*(ay*(b + 2*c)*(Power(b,2) + b*c + c*(-3*a + c)) - by*(Power(b,3) + 2*Power(b,2)*c - 4*a*Power(c,2) + b*c*(-3*a + c)))*r2)/r1 - (2*(ay*(b + 2*c)*(Power(b,2) + b*c + c*(-3*a + c)) - by*(Power(b,3) + 2*Power(b,2)*c - 4*a*Power(c,2) + b*c*(-3*a + c)))*r3)/r1 - (-(by*(Power(b,2) + 2*b*c + c*(-a + c))) + ay*(Power(b,2) + 3*b*c + c*(-a + 3*c)))*Log(a) + (-(by*(Power(b,2) + 2*b*c + c*(-a + c))) + ay*(Power(b,2) + 3*b*c + c*(-a + 3*c)))*Log(r6)))/Power(c,3);
		bary2[i*3 + 0] += k*B;
		
		B(0,0) = 0;
		B(1,0) = ((ay - by)*(-2*c + (2*(Power(b,2) - 2*a*c + b*c)*r2)/r1 - (2*(Power(b,2) - 2*a*c + b*c)*r3)/r1 - (b + c)*Log(a) + (b + c)*Log(r6)))/Power(c,2);
		B(2,0) = ((ax - bx)*(2*c - (2*(Power(b,2) - 2*a*c + b*c)*r2)/r1 + (2*(Power(b,2) - 2*a*c + b*c)*r3)/r1 + (b + c)*Log(a) - (b + c)*Log(r6)))/Power(c,2);
		B(3,0) = (2*(ay - by)*((ax - bx)*Power(c,2) + 2*c*(bx*(b + c) - ax*(b + 2*c)) + (2*(-(bx*(Power(b,3) - 3*a*b*c + Power(b,2)*c - 2*a*Power(c,2))) + ax*(Power(b,3) + 2*Power(b,2)*c - 4*a*Power(c,2) + b*c*(-3*a + c)))*r2)/r1 - (2*(-(bx*(Power(b,3) - 3*a*b*c + Power(b,2)*c - 2*a*Power(c,2))) + ax*(Power(b,3) + 2*Power(b,2)*c - 4*a*Power(c,2) + b*c*(-3*a + c)))*r3)/r1 - (-(bx*(Power(b,2) - a*c + b*c)) + ax*(Power(b,2) + 2*b*c + c*(-a + c)))*Log(a) + (-(bx*(Power(b,2) - a*c + b*c)) + ax*(Power(b,2) + 2*b*c + c*(-a + c)))*Log(r6)))/Power(c,3);
		B(4,0) = ((-Power(ax,2) + Power(ay,2) + 2*ax*bx - Power(bx,2) - 2*ay*by + Power(by,2))*Power(c,2) + 2*c*((Power(bx,2) - Power(by,2))*(b + c) + Power(ax,2)*(b + 2*c) - Power(ay,2)*(b + 2*c) - ax*bx*(2*b + 3*c) + ay*by*(2*b + 3*c)) - (2*(-(ax*bx*(b + c)*(2*Power(b,2) - 6*a*c + b*c)) + ay*by*(b + c)*(2*Power(b,2) - 6*a*c + b*c) + (Power(bx,2) - Power(by,2))*(Power(b,3) - 3*a*b*c + Power(b,2)*c - 2*a*Power(c,2)) + Power(ax,2)*(Power(b,3) + 2*Power(b,2)*c - 4*a*Power(c,2) + b*c*(-3*a + c)) - Power(ay,2)*(Power(b,3) + 2*Power(b,2)*c - 4*a*Power(c,2) + b*c*(-3*a + c)))*r2)/r1 + (2*(-(ax*bx*(b + c)*(2*Power(b,2) - 6*a*c + b*c)) + ay*by*(b + c)*(2*Power(b,2) - 6*a*c + b*c) + (Power(bx,2) - Power(by,2))*(Power(b,3) - 3*a*b*c + Power(b,2)*c - 2*a*Power(c,2)) + Power(ax,2)*(Power(b,3) + 2*Power(b,2)*c - 4*a*Power(c,2) + b*c*(-3*a + c)) - Power(ay,2)*(Power(b,3) + 2*Power(b,2)*c - 4*a*Power(c,2) + b*c*(-3*a + c)))*r3)/r1 - (-((Power(bx,2) - Power(by,2))*(Power(b,2) - a*c + b*c)) + Power(ay,2)*(Power(b,2) - a*c + 2*b*c + Power(c,2)) - ay*by*(2*Power(b,2) - 2*a*c + 3*b*c + Power(c,2)) + ax*bx*(2*Power(b,2) + 3*b*c + c*(-2*a + c)) - Power(ax,2)*(Power(b,2) + 2*b*c + c*(-a + c)))*Log(a) + (-((Power(bx,2) - Power(by,2))*(Power(b,2) - a*c + b*c)) + Power(ay,2)*(Power(b,2) - a*c + 2*b*c + Power(c,2)) - ay*by*(2*Power(b,2) - 2*a*c + 3*b*c + Power(c,2)) + ax*bx*(2*Power(b,2) + 3*b*c + c*(-2*a + c)) - Power(ax,2)*(Power(b,2) + 2*b*c + c*(-a + c)))*Log(r6))/Power(c,3);
		B(5,0) = (2*(ax - bx)*(-((ay - by)*Power(c,2)) - 2*c*(by*(b + c) - ay*(b + 2*c)) - (2*(-(by*(Power(b,3) - 3*a*b*c + Power(b,2)*c - 2*a*Power(c,2))) + ay*(Power(b,3) + 2*Power(b,2)*c - 4*a*Power(c,2) + b*c*(-3*a + c)))*r2)/r1 + (2*(-(by*(Power(b,3) - 3*a*b*c + Power(b,2)*c - 2*a*Power(c,2))) + ay*(Power(b,3) + 2*Power(b,2)*c - 4*a*Power(c,2) + b*c*(-3*a + c)))*r3)/r1 + (-(by*(Power(b,2) - a*c + b*c)) + ay*(Power(b,2) + 2*b*c + c*(-a + c)))*Log(a) - (-(by*(Power(b,2) - a*c + b*c)) + ay*(Power(b,2) + 2*b*c + c*(-a + c)))*Log(r6)))/Power(c,3);
		bary2[i*3 + 1] += k*B;
		
		B(0,0) = 0;
		B(1,0) = ((ay - by)*(-2*(Power(b,2) - 2*a*c)*r2 + 2*(Power(b,2) - 2*a*c)*r3 + r1*(2*c + b*Log(a) - b*Log(r6))))/(2.*Power(c,2)*r1);
		B(2,0) = -((ax - bx)*(-2*(Power(b,2) - 2*a*c)*r2 + 2*(Power(b,2) - 2*a*c)*r3 + r1*(2*c + b*Log(a) - b*Log(r6))))/(2.*Power(c,2)*r1);
		B(3,0) = ((ay - by)*((-ax + bx)*Power(c,2) + 2*c*(-(b*bx) + ax*(b + c)) - (2*(-(b*bx*(Power(b,2) - 3*a*c)) + ax*(Power(b,3) - 3*a*b*c + Power(b,2)*c - 2*a*Power(c,2)))*r2)/r1 + (2*(-(b*bx*(Power(b,2) - 3*a*c)) + ax*(Power(b,3) - 3*a*b*c + Power(b,2)*c - 2*a*Power(c,2)))*r3)/r1 + (-(Power(b,2)*bx) + a*bx*c + ax*(Power(b,2) - a*c + b*c))*Log(a) - (-(Power(b,2)*bx) + a*bx*c + ax*(Power(b,2) - a*c + b*c))*Log(r6)))/Power(c,3);
		B(4,0) = ((Power(ax,2) - Power(ay,2) - 2*ax*bx + Power(bx,2) + 2*ay*by - Power(by,2))*Power(c,2) + 2*c*(b*(-Power(bx,2) + Power(by,2)) - Power(ax,2)*(b + c) + Power(ay,2)*(b + c) + ax*bx*(2*b + c) - ay*by*(2*b + c)) + (2*(b*(Power(bx,2) - Power(by,2))*(Power(b,2) - 3*a*c) + ay*by*(2*Power(b,3) - 6*a*b*c + Power(b,2)*c - 2*a*Power(c,2)) + Power(ax,2)*(Power(b,3) - 3*a*b*c + Power(b,2)*c - 2*a*Power(c,2)) - Power(ay,2)*(Power(b,3) - 3*a*b*c + Power(b,2)*c - 2*a*Power(c,2)) + ax*bx*(-2*Power(b,3) + 6*a*b*c - Power(b,2)*c + 2*a*Power(c,2)))*r2)/r1 - (2*(b*(Power(bx,2) - Power(by,2))*(Power(b,2) - 3*a*c) + ay*by*(2*Power(b,3) - 6*a*b*c + Power(b,2)*c - 2*a*Power(c,2)) + Power(ax,2)*(Power(b,3) - 3*a*b*c + Power(b,2)*c - 2*a*Power(c,2)) - Power(ay,2)*(Power(b,3) - 3*a*b*c + Power(b,2)*c - 2*a*Power(c,2)) + ax*bx*(-2*Power(b,3) + 6*a*b*c - Power(b,2)*c + 2*a*Power(c,2)))*r3)/r1 - ((Power(bx,2) - Power(by,2))*(Power(b,2) - a*c) - ax*bx*(2*Power(b,2) - 2*a*c + b*c) + ay*by*(2*Power(b,2) - 2*a*c + b*c) + Power(ax,2)*(Power(b,2) - a*c + b*c) - Power(ay,2)*(Power(b,2) - a*c + b*c))*Log(a) + ((Power(bx,2) - Power(by,2))*(Power(b,2) - a*c) - ax*bx*(2*Power(b,2) - 2*a*c + b*c) + ay*by*(2*Power(b,2) - 2*a*c + b*c) + Power(ax,2)*(Power(b,2) - a*c + b*c) - Power(ay,2)*(Power(b,2) - a*c + b*c))*Log(r6))/(2.*Power(c,3));
		B(5,0) = ((ax - bx)*((ay - by)*Power(c,2) - 2*c*(-(b*by) + ay*(b + c)) + (2*(-(b*by*(Power(b,2) - 3*a*c)) + ay*(Power(b,3) - 3*a*b*c + Power(b,2)*c - 2*a*Power(c,2)))*r2)/r1 - (2*(-(b*by*(Power(b,2) - 3*a*c)) + ay*(Power(b,3) - 3*a*b*c + Power(b,2)*c - 2*a*Power(c,2)))*r3)/r1 - (-(Power(b,2)*by) + a*by*c + ay*(Power(b,2) - a*c + b*c))*Log(a) + (-(Power(b,2)*by) + a*by*c + ay*(Power(b,2) - a*c + b*c))*Log(r6)))/Power(c,3);
		bary2[i*3 + 2] += k*B;
		}

		// calc second derivative coordinates
		if (g.deriv == 2)
		{
		B(0,0) = 0;
		B(1,0) = 0;
		B(2,0) = 0;
		B(3,0) = (Power(ay - by,2)*(2*c - (2*(Power(b,2) + 2*b*c + 2*c*(-a + c))*r2)/r1 + (2*(Power(b,2) + 2*b*c + 2*c*(-a + c))*r3)/r1 + (b + 2*c)*Log(a) - (b + 2*c)*Log(r6)))/Power(c,2);
		B(4,0) = ((ax - bx)*(ay - by)*(-2*c + (2*(Power(b,2) + 2*b*c + 2*c*(-a + c))*r2)/r1 - (2*(Power(b,2) + 2*b*c + 2*c*(-a + c))*r3)/r1 - (b + 2*c)*Log(a) + (b + 2*c)*Log(r6)))/Power(c,2);
		B(5,0) = (Power(ax - bx,2)*(2*c - (2*(Power(b,2) + 2*b*c + 2*c*(-a + c))*r2)/r1 + (2*(Power(b,2) + 2*b*c + 2*c*(-a + c))*r3)/r1 + (b + 2*c)*Log(a) - (b + 2*c)*Log(r6)))/Power(c,2);
		bary2[i*3 + 0] += (k*B) / sqrt_c;
		
		B(0,0) = 0;
		B(1,0) = 0;
		B(2,0) = 0;
		B(3,0) = (2*Power(ay - by,2)*(-2*c + (2*(Power(b,2) - 2*a*c + b*c)*r2)/r1 - (2*(Power(b,2) - 2*a*c + b*c)*r3)/r1 - (b + c)*Log(a) + (b + c)*Log(r6)))/Power(c,2);
		B(4,0) = (2*(ax - bx)*(ay - by)*(2*c - (2*(Power(b,2) - 2*a*c + b*c)*r2)/r1 + (2*(Power(b,2) - 2*a*c + b*c)*r3)/r1 + (b + c)*Log(a) - (b + c)*Log(r6)))/Power(c,2);
		B(5,0) = (2*Power(ax - bx,2)*(-2*c + (2*(Power(b,2) - 2*a*c + b*c)*r2)/r1 - (2*(Power(b,2) - 2*a*c + b*c)*r3)/r1 - (b + c)*Log(a) + (b + c)*Log(r6)))/Power(c,2);
		bary2[i*3 + 1] += (k*B) / sqrt_c;
		
		B(0,0) = 0;
		B(1,0) = 0;
		B(2,0) = 0;
		B(3,0) = (Power(ay - by,2)*(-2*(Power(b,2) - 2*a*c)*r2 + 2*(Power(b,2) - 2*a*c)*r3 + r1*(2*c + b*Log(a) - b*Log(r6))))/(Power(c,2)*r1);
		B(4,0) = -(((ax - bx)*(ay - by)*(-2*(Power(b,2) - 2*a*c)*r2 + 2*(Power(b,2) - 2*a*c)*r3 + r1*(2*c + b*Log(a) - b*Log(r6))))/(Power(c,2)*r1));
		B(5,0) = (Power(ax - bx,2)*(-2*(Power(b,2) - 2*a*c)*r2 + 2*(Power(b,2) - 2*a*c)*r3 + r1*(2*c + b*Log(a) - b*Log(r6))))/(Power(c,2)*r1);
		bary2[i*3 + 2] += (k*B) / sqrt_c;
		}
	}

	// bary coords must sum to one
	double sum = 0;
	for (int i = 0; i < n; i++)
		sum += bary[i];
	for (int i = 0; i < n; i++)
		bary[i] /= sum;
}

void calcBaryQuad_Alpha2(double *bary, double *bary2, vect2d &x, vector<vect2d> &pv, vector<vect2i> &p, vector<vect2d> &r)
{
	double epsilon = 1e-3;

	int n = pv.size();
	if (n < 3)
		return;

	// calculate distances from line and projection onto line
	vector<double> dists(n);
	vector<double> projs(n);
	for (int i = 0; i < n; i++)
	{
		vect2d vv = pv[p[i][1]] - pv[p[i][0]];
		vect2d xx = x - pv[p[i][0]];
		projs[i] = xx*vv/vv.length2();
		dists[i] = (xx - vv*projs[i]).length();
	}

	// fix degenerate cases (force to interpolate the line)
	for (int i = 0; i < n; i++)
	{
		if (dists[i] < epsilon && projs[i] >= 0 && projs[i] <= 1)
		{
			bary[p[i][0]] += 1 - projs[i];
			bary[p[i][1]] += projs[i];
			return;
		}
	}

	// calculate constant
	Matrixd M(6,6), W(6,6);
	M.set_zero();
	for (int i = 0; i < n; i++)
	{
		// calculate some common terms
		vect2d &p1 = pv[p[i][0]];
		vect2d &p2 = pv[p[i][1]];
		double ax = p1[0];
		double ay = p1[1];
		double bx = p2[0];
		double by = p2[1];

		double a = (p1 - x)*(p1 - x);
		double b = (p1 - p2)*(p1 - x)*(-2);
		double c = (p1 - p2)*(p1 - p2);

		double sqrt_c = sqrt(c);

		double r5 = -b*b + 4*a*c;
		double r1 = sqrt(r5);
		double r4 = 1.0 / (r1*r5);
		double r2 = atan(b/r1);
		double r3 = atan((b+2*c)/r1);
		double r6 = a+b+c;

		// calc matrix of PtP
		W(0,0) = -((b/a + (4*c*r2)/r1 - (4*c*r3)/r1 - (b + 2*c)/r6)/r5);
		W(0,1) = -((ax*(2 + b/a) - 2*bx + (2*(-(b*bx) + ax*(b + 2*c))*r2)/r1 - (2*(-(b*bx) + ax*(b + 2*c))*r3)/r1 + (-2*a*(ax - bx) + b*bx - 2*ax*(b + c))/r6)/r5);
		W(0,2) = -((ay*(2 + b/a) - 2*by + (2*(-(b*by) + ay*(b + 2*c))*r2)/r1 - (2*(-(b*by) + ay*(b + 2*c))*r3)/r1 + (-2*a*(ay - by) + b*by - 2*ay*(b + c))/r6)/r5);
		W(0,3) = -(((Power(ax,2)*b*c + a*(ax - bx)*(-(b*bx) + ax*(b + 4*c)))/(a*c) + (4*(a*Power(ax - bx,2) + ax*(-(b*bx) + ax*(b + c)))*r2)/r1 - (4*(a*Power(ax - bx,2) + ax*(-(b*bx) + ax*(b + c)))*r3)/r1 - (Power(b,2)*Power(bx,2) - 2*ax*b*bx*(b + c) + a*(ax - bx)*(ax*b - b*bx + 2*ax*c + 2*bx*c) + Power(ax,2)*(Power(b,2) + 3*b*c + 2*Power(c,2)))/(c*r6))/r5);
		W(0,4) = -(((ax*ay*b*c + a*(bx*(b*by - ay*(b + 2*c)) + ax*(-(by*(b + 2*c)) + ay*(b + 4*c))))/(a*c) - (2*(ay*b*bx - 2*a*(ax - bx)*(ay - by) + ax*(b*by - 2*ay*(b + c)))*r2)/r1 + (2*(ay*b*bx - 2*a*(ax - bx)*(ay - by) + ax*(b*by - 2*ay*(b + c)))*r3)/r1 + (b*bx*(-(b*by) + ay*(b + c)) + ax*(b + c)*(b*by - ay*(b + 2*c)) + a*(bx*(ay*b - b*by + 2*by*c) + ax*(b*by - ay*(b + 2*c))))/(c*r6))/r5);
		W(0,5) = -(((Power(ay,2)*b*c + a*(ay - by)*(-(b*by) + ay*(b + 4*c)))/(a*c) + (4*(a*Power(ay - by,2) + ay*(-(b*by) + ay*(b + c)))*r2)/r1 - (4*(a*Power(ay - by,2) + ay*(-(b*by) + ay*(b + c)))*r3)/r1 - (Power(b,2)*Power(by,2) - 2*ay*b*by*(b + c) + a*(ay - by)*(ay*b - b*by + 2*ay*c + 2*by*c) + Power(ay,2)*(Power(b,2) + 3*b*c + 2*Power(c,2)))/(c*r6))/r5);
		W(1,1) = -(((Power(ax,2)*b*c + a*(ax - bx)*(-(b*bx) + ax*(b + 4*c)))/(a*c) + (4*(a*Power(ax - bx,2) + ax*(-(b*bx) + ax*(b + c)))*r2)/r1 - (4*(a*Power(ax - bx,2) + ax*(-(b*bx) + ax*(b + c)))*r3)/r1 - (Power(b,2)*Power(bx,2) - 2*ax*b*bx*(b + c) + a*(ax - bx)*(ax*b - b*bx + 2*ax*c + 2*bx*c) + Power(ax,2)*(Power(b,2) + 3*b*c + 2*Power(c,2)))/(c*r6))/r5);
		W(1,2) = -(((ax*ay*b*c + a*(bx*(b*by - ay*(b + 2*c)) + ax*(-(by*(b + 2*c)) + ay*(b + 4*c))))/(a*c) - (2*(ay*b*bx - 2*a*(ax - bx)*(ay - by) + ax*(b*by - 2*ay*(b + c)))*r2)/r1 + (2*(ay*b*bx - 2*a*(ax - bx)*(ay - by) + ax*(b*by - 2*ay*(b + c)))*r3)/r1 + (b*bx*(-(b*by) + ay*(b + c)) + ax*(b + c)*(b*by - ay*(b + 2*c)) + a*(bx*(ay*b - b*by + 2*by*c) + ax*(b*by - ay*(b + 2*c))))/(c*r6))/r5);
		W(1,3) = ((-2*(-(Power(b,3)*Power(bx,3)) - 2*Power(a,2)*Power(ax - bx,3)*c + 3*ax*Power(b,2)*Power(bx,2)*(b + c) - 3*Power(ax,2)*b*bx*Power(b + c,2) + Power(ax,3)*(Power(b,3) + 3*Power(b,2)*c + 4*b*Power(c,2) + 2*Power(c,3)) + a*(ax - bx)*(Power(ax,2)*Power(b,2) + b*Power(bx,2)*(b - 3*c) + ax*bx*(-2*Power(b,2) + 3*b*c + 6*Power(c,2)))))/(-4*Power(a,2)*c + Power(b,2)*(b + c) + a*(Power(b,2) - 4*b*c - 4*Power(c,2))) + 2*(-(b*bx) + ax*(b + 2*c))*(Power(bx,2)*(Power(b,2) - 6*a*c) + 2*ax*bx*(-Power(b,2) + 6*a*c + b*c) + Power(ax,2)*(Power(b,2) - 2*b*c - 2*c*(3*a + c)))*r2*r4 - 2*(-(b*bx) + ax*(b + 2*c))*(Power(bx,2)*(Power(b,2) - 6*a*c) + 2*ax*bx*(-Power(b,2) + 6*a*c + b*c) + Power(ax,2)*(Power(b,2) - 2*b*c - 2*c*(3*a + c)))*r3*r4 - (2*(-2*Power(a,2)*Power(ax - bx,3)*c + Power(ax,3)*b*Power(c,2) + a*(ax - bx)*(Power(b,2)*Power(bx,2) - ax*b*bx*(2*b + 3*c) + Power(ax,2)*(Power(b,2) + 3*b*c + 6*Power(c,2)))))/(a*r5) + Power(ax - bx,3)*Log(a) - Power(ax - bx,3)*Log(r6))/(2.*Power(c,2));
		W(1,4) = ((-2*(2*Power(a,2)*Power(ax - bx,2)*(ay - by)*c - Power(b,2)*Power(bx,2)*(-(b*by) + ay*(b + c)) + 2*ax*b*bx*(b + c)*(-(b*by) + ay*(b + c)) - Power(ax,2)*(b + c)*(-(b*by*(b + c)) + ay*(Power(b,2) + 2*b*c + 2*Power(c,2))) - a*(Power(bx,2)*(-(b*by*(b - 3*c)) + ay*(Power(b,2) - 2*b*c - 2*Power(c,2))) + 2*ax*bx*(by*(Power(b,2) - 2*b*c - 2*Power(c,2)) + ay*(-Power(b,2) + b*c + 2*Power(c,2))) + Power(ax,2)*(ay*Power(b,2) + by*(-Power(b,2) + b*c + 2*Power(c,2))))))/(4*Power(a,2)*c - Power(b,2)*(b + c) + a*(-Power(b,2) + 4*b*c + 4*Power(c,2))) + 2*(Power(bx,2)*(-(b*by*(Power(b,2) - 6*a*c)) + ay*(Power(b,3) - 6*a*b*c - 4*a*Power(c,2))) - 2*ax*bx*(by*(-Power(b,3) + 6*a*b*c + 4*a*Power(c,2)) + ay*(Power(b,3) - 8*a*Power(c,2) - 2*b*c*(3*a + c))) + Power(ax,2)*(ay*(b + 2*c)*(Power(b,2) - 2*b*c - 2*c*(3*a + c)) + by*(-Power(b,3) + 8*a*Power(c,2) + 2*b*c*(3*a + c))))*r2*r4 - 2*(Power(bx,2)*(-(b*by*(Power(b,2) - 6*a*c)) + ay*(Power(b,3) - 6*a*b*c - 4*a*Power(c,2))) - 2*ax*bx*(by*(-Power(b,3) + 6*a*b*c + 4*a*Power(c,2)) + ay*(Power(b,3) - 8*a*Power(c,2) - 2*b*c*(3*a + c))) + Power(ax,2)*(ay*(b + 2*c)*(Power(b,2) - 2*b*c - 2*c*(3*a + c)) + by*(-Power(b,3) + 8*a*Power(c,2) + 2*b*c*(3*a + c))))*r3*r4 + (2*(2*Power(a,2)*Power(ax - bx,2)*(ay - by)*c - Power(ax,2)*ay*b*Power(c,2) + a*(-(b*Power(bx,2)*(-(b*by) + ay*(b + c))) + 2*ax*bx*(-(b*by*(b + c)) + ay*(Power(b,2) + 2*b*c + 2*Power(c,2))) + Power(ax,2)*(by*(Power(b,2) + 2*b*c + 2*Power(c,2)) - ay*(Power(b,2) + 3*b*c + 6*Power(c,2))))))/(a*r5) + Power(ax - bx,2)*(ay - by)*Log(a) - Power(ax - bx,2)*(ay - by)*Log(r6))/(2.*Power(c,2));
		W(1,5) = ((-2*(2*Power(a,2)*(ax - bx)*Power(ay - by,2)*c + b*bx*Power(b*by - ay*(b + c),2) - ax*(b + c)*(Power(b,2)*Power(by,2) - 2*ay*b*by*(b + c) + Power(ay,2)*(Power(b,2) + 2*b*c + 2*Power(c,2))) - a*(bx*(-(b*Power(by,2)*(b - 3*c)) + 2*ay*by*(Power(b,2) - 2*b*c - 2*Power(c,2)) + Power(ay,2)*(-Power(b,2) + b*c + 2*Power(c,2))) + ax*(Power(ay,2)*Power(b,2) + Power(by,2)*(Power(b,2) - 2*b*c - 2*Power(c,2)) + 2*ay*by*(-Power(b,2) + b*c + 2*Power(c,2))))))/(4*Power(a,2)*c - Power(b,2)*(b + c) + a*(-Power(b,2) + 4*b*c + 4*Power(c,2))) + 2*(bx*(-(b*Power(by,2)*(Power(b,2) - 6*a*c)) + 2*ay*by*(Power(b,3) - 6*a*b*c - 4*a*Power(c,2)) + Power(ay,2)*(-Power(b,3) + 8*a*Power(c,2) + 2*b*c*(3*a + c))) + ax*(Power(by,2)*(Power(b,3) - 6*a*b*c - 4*a*Power(c,2)) + Power(ay,2)*(b + 2*c)*(Power(b,2) - 2*b*c - 2*c*(3*a + c)) + 2*ay*by*(-Power(b,3) + 8*a*Power(c,2) + 2*b*c*(3*a + c))))*r2*r4 - 2*(bx*(-(b*Power(by,2)*(Power(b,2) - 6*a*c)) + 2*ay*by*(Power(b,3) - 6*a*b*c - 4*a*Power(c,2)) + Power(ay,2)*(-Power(b,3) + 8*a*Power(c,2) + 2*b*c*(3*a + c))) + ax*(Power(by,2)*(Power(b,3) - 6*a*b*c - 4*a*Power(c,2)) + Power(ay,2)*(b + 2*c)*(Power(b,2) - 2*b*c - 2*c*(3*a + c)) + 2*ay*by*(-Power(b,3) + 8*a*Power(c,2) + 2*b*c*(3*a + c))))*r3*r4 + (2*(2*Power(a,2)*(ax - bx)*Power(ay - by,2)*c - ax*Power(ay,2)*b*Power(c,2) + a*(bx*(Power(b,2)*Power(by,2) - 2*ay*b*by*(b + c) + Power(ay,2)*(Power(b,2) + 2*b*c + 2*Power(c,2))) - ax*(b*Power(by,2)*(b + c) - 2*ay*by*(Power(b,2) + 2*b*c + 2*Power(c,2)) + Power(ay,2)*(Power(b,2) + 3*b*c + 6*Power(c,2))))))/(a*r5) + (ax - bx)*Power(ay - by,2)*Log(a) - (ax - bx)*Power(ay - by,2)*Log(r6))/(2.*Power(c,2));
		W(2,2) = -(((Power(ay,2)*b*c + a*(ay - by)*(-(b*by) + ay*(b + 4*c)))/(a*c) + (4*(a*Power(ay - by,2) + ay*(-(b*by) + ay*(b + c)))*r2)/r1 - (4*(a*Power(ay - by,2) + ay*(-(b*by) + ay*(b + c)))*r3)/r1 - (Power(b,2)*Power(by,2) - 2*ay*b*by*(b + c) + a*(ay - by)*(ay*b - b*by + 2*ay*c + 2*by*c) + Power(ay,2)*(Power(b,2) + 3*b*c + 2*Power(c,2)))/(c*r6))/r5);
		W(2,3) = ((-2*(2*Power(a,2)*Power(ax - bx,2)*(ay - by)*c - Power(b,2)*Power(bx,2)*(-(b*by) + ay*(b + c)) + 2*ax*b*bx*(b + c)*(-(b*by) + ay*(b + c)) - Power(ax,2)*(b + c)*(-(b*by*(b + c)) + ay*(Power(b,2) + 2*b*c + 2*Power(c,2))) - a*(Power(bx,2)*(-(b*by*(b - 3*c)) + ay*(Power(b,2) - 2*b*c - 2*Power(c,2))) + 2*ax*bx*(by*(Power(b,2) - 2*b*c - 2*Power(c,2)) + ay*(-Power(b,2) + b*c + 2*Power(c,2))) + Power(ax,2)*(ay*Power(b,2) + by*(-Power(b,2) + b*c + 2*Power(c,2))))))/(4*Power(a,2)*c - Power(b,2)*(b + c) + a*(-Power(b,2) + 4*b*c + 4*Power(c,2))) + 2*(Power(bx,2)*(-(b*by*(Power(b,2) - 6*a*c)) + ay*(Power(b,3) - 6*a*b*c - 4*a*Power(c,2))) - 2*ax*bx*(by*(-Power(b,3) + 6*a*b*c + 4*a*Power(c,2)) + ay*(Power(b,3) - 8*a*Power(c,2) - 2*b*c*(3*a + c))) + Power(ax,2)*(ay*(b + 2*c)*(Power(b,2) - 2*b*c - 2*c*(3*a + c)) + by*(-Power(b,3) + 8*a*Power(c,2) + 2*b*c*(3*a + c))))*r2*r4 - 2*(Power(bx,2)*(-(b*by*(Power(b,2) - 6*a*c)) + ay*(Power(b,3) - 6*a*b*c - 4*a*Power(c,2))) - 2*ax*bx*(by*(-Power(b,3) + 6*a*b*c + 4*a*Power(c,2)) + ay*(Power(b,3) - 8*a*Power(c,2) - 2*b*c*(3*a + c))) + Power(ax,2)*(ay*(b + 2*c)*(Power(b,2) - 2*b*c - 2*c*(3*a + c)) + by*(-Power(b,3) + 8*a*Power(c,2) + 2*b*c*(3*a + c))))*r3*r4 + (2*(2*Power(a,2)*Power(ax - bx,2)*(ay - by)*c - Power(ax,2)*ay*b*Power(c,2) + a*(-(b*Power(bx,2)*(-(b*by) + ay*(b + c))) + 2*ax*bx*(-(b*by*(b + c)) + ay*(Power(b,2) + 2*b*c + 2*Power(c,2))) + Power(ax,2)*(by*(Power(b,2) + 2*b*c + 2*Power(c,2)) - ay*(Power(b,2) + 3*b*c + 6*Power(c,2))))))/(a*r5) + Power(ax - bx,2)*(ay - by)*Log(a) - Power(ax - bx,2)*(ay - by)*Log(r6))/(2.*Power(c,2));
		W(2,4) = ((-2*(2*Power(a,2)*(ax - bx)*Power(ay - by,2)*c + b*bx*Power(b*by - ay*(b + c),2) - ax*(b + c)*(Power(b,2)*Power(by,2) - 2*ay*b*by*(b + c) + Power(ay,2)*(Power(b,2) + 2*b*c + 2*Power(c,2))) - a*(bx*(-(b*Power(by,2)*(b - 3*c)) + 2*ay*by*(Power(b,2) - 2*b*c - 2*Power(c,2)) + Power(ay,2)*(-Power(b,2) + b*c + 2*Power(c,2))) + ax*(Power(ay,2)*Power(b,2) + Power(by,2)*(Power(b,2) - 2*b*c - 2*Power(c,2)) + 2*ay*by*(-Power(b,2) + b*c + 2*Power(c,2))))))/(4*Power(a,2)*c - Power(b,2)*(b + c) + a*(-Power(b,2) + 4*b*c + 4*Power(c,2))) + 2*(bx*(-(b*Power(by,2)*(Power(b,2) - 6*a*c)) + 2*ay*by*(Power(b,3) - 6*a*b*c - 4*a*Power(c,2)) + Power(ay,2)*(-Power(b,3) + 8*a*Power(c,2) + 2*b*c*(3*a + c))) + ax*(Power(by,2)*(Power(b,3) - 6*a*b*c - 4*a*Power(c,2)) + Power(ay,2)*(b + 2*c)*(Power(b,2) - 2*b*c - 2*c*(3*a + c)) + 2*ay*by*(-Power(b,3) + 8*a*Power(c,2) + 2*b*c*(3*a + c))))*r2*r4 - 2*(bx*(-(b*Power(by,2)*(Power(b,2) - 6*a*c)) + 2*ay*by*(Power(b,3) - 6*a*b*c - 4*a*Power(c,2)) + Power(ay,2)*(-Power(b,3) + 8*a*Power(c,2) + 2*b*c*(3*a + c))) + ax*(Power(by,2)*(Power(b,3) - 6*a*b*c - 4*a*Power(c,2)) + Power(ay,2)*(b + 2*c)*(Power(b,2) - 2*b*c - 2*c*(3*a + c)) + 2*ay*by*(-Power(b,3) + 8*a*Power(c,2) + 2*b*c*(3*a + c))))*r3*r4 + (2*(2*Power(a,2)*(ax - bx)*Power(ay - by,2)*c - ax*Power(ay,2)*b*Power(c,2) + a*(bx*(Power(b,2)*Power(by,2) - 2*ay*b*by*(b + c) + Power(ay,2)*(Power(b,2) + 2*b*c + 2*Power(c,2))) - ax*(b*Power(by,2)*(b + c) - 2*ay*by*(Power(b,2) + 2*b*c + 2*Power(c,2)) + Power(ay,2)*(Power(b,2) + 3*b*c + 6*Power(c,2))))))/(a*r5) + (ax - bx)*Power(ay - by,2)*Log(a) - (ax - bx)*Power(ay - by,2)*Log(r6))/(2.*Power(c,2));
		W(2,5) = ((-2*(-(Power(b,3)*Power(by,3)) - 2*Power(a,2)*Power(ay - by,3)*c + 3*ay*Power(b,2)*Power(by,2)*(b + c) - 3*Power(ay,2)*b*by*Power(b + c,2) + Power(ay,3)*(Power(b,3) + 3*Power(b,2)*c + 4*b*Power(c,2) + 2*Power(c,3)) + a*(ay - by)*(Power(ay,2)*Power(b,2) + b*Power(by,2)*(b - 3*c) + ay*by*(-2*Power(b,2) + 3*b*c + 6*Power(c,2)))))/(-4*Power(a,2)*c + Power(b,2)*(b + c) + a*(Power(b,2) - 4*b*c - 4*Power(c,2))) + 2*(-(b*by) + ay*(b + 2*c))*(Power(by,2)*(Power(b,2) - 6*a*c) + 2*ay*by*(-Power(b,2) + 6*a*c + b*c) + Power(ay,2)*(Power(b,2) - 2*b*c - 2*c*(3*a + c)))*r2*r4 - 2*(-(b*by) + ay*(b + 2*c))*(Power(by,2)*(Power(b,2) - 6*a*c) + 2*ay*by*(-Power(b,2) + 6*a*c + b*c) + Power(ay,2)*(Power(b,2) - 2*b*c - 2*c*(3*a + c)))*r3*r4 - (2*(-2*Power(a,2)*Power(ay - by,3)*c + Power(ay,3)*b*Power(c,2) + a*(ay - by)*(Power(b,2)*Power(by,2) - ay*b*by*(2*b + 3*c) + Power(ay,2)*(Power(b,2) + 3*b*c + 6*Power(c,2)))))/(a*r5) + Power(ay - by,3)*Log(a) - Power(ay - by,3)*Log(r6))/(2.*Power(c,2));
		W(3,3) = (Power(ax - bx,4)*c + (Power(b,4)*Power(bx,4) - 4*ax*Power(b,3)*Power(bx,3)*(b + c) + 6*Power(ax,2)*Power(b,2)*Power(bx,2)*Power(b + c,2) - 4*Power(ax,3)*b*bx*Power(b + c,3) + Power(ax,4)*(Power(b,4) + 4*Power(b,3)*c + 6*Power(b,2)*Power(c,2) + 5*b*Power(c,3) + 2*Power(c,4)) - Power(a,2)*Power(ax - bx,3)*c*(-3*b*bx + 2*bx*c + 3*ax*(b + 2*c)) + a*(ax - bx)*(-(Power(b,2)*Power(bx,3)*(b - 4*c)) + ax*b*Power(bx,2)*(3*Power(b,2) - 8*b*c - 12*Power(c,2)) + Power(ax,3)*(Power(b,3) - 6*b*Power(c,2) - 4*Power(c,3)) + Power(ax,2)*bx*(-3*Power(b,3) + 4*Power(b,2)*c + 18*b*Power(c,2) + 12*Power(c,3))))/(4*Power(a,2)*c - Power(b,2)*(b + c) + a*(-Power(b,2) + 4*b*c + 4*Power(c,2))) + 2*(Power(bx,4)*(Power(b,4) - 6*a*Power(b,2)*c + 6*Power(a,2)*Power(c,2)) - 2*ax*Power(bx,3)*(2*Power(b,4) - 12*a*Power(b,2)*c + Power(b,3)*c + 12*Power(a,2)*Power(c,2) - 6*a*b*Power(c,2)) + 6*Power(ax,2)*Power(bx,2)*(Power(b,4) - 6*a*Power(b,2)*c + Power(b,3)*c - 6*a*b*Power(c,2) + 2*a*(3*a - c)*Power(c,2)) + 2*Power(ax,3)*bx*(-2*Power(b,4) + 12*a*Power(b,2)*c - 3*Power(b,3)*c + 12*a*Power(c,2)*(-a + c) + 2*b*Power(c,2)*(9*a + c)) + Power(ax,4)*(Power(b,4) - 6*a*Power(b,2)*c + 2*Power(b,3)*c - 4*b*Power(c,2)*(3*a + c) - 2*Power(c,2)*(-3*Power(a,2) + 6*a*c + Power(c,2))))*r2*r4 - 2*(Power(bx,4)*(Power(b,4) - 6*a*Power(b,2)*c + 6*Power(a,2)*Power(c,2)) - 2*ax*Power(bx,3)*(2*Power(b,4) - 12*a*Power(b,2)*c + Power(b,3)*c + 12*Power(a,2)*Power(c,2) - 6*a*b*Power(c,2)) + 6*Power(ax,2)*Power(bx,2)*(Power(b,4) - 6*a*Power(b,2)*c + Power(b,3)*c - 6*a*b*Power(c,2) + 2*a*(3*a - c)*Power(c,2)) + 2*Power(ax,3)*bx*(-2*Power(b,4) + 12*a*Power(b,2)*c - 3*Power(b,3)*c + 12*a*Power(c,2)*(-a + c) + 2*b*Power(c,2)*(9*a + c)) + Power(ax,4)*(Power(b,4) - 6*a*Power(b,2)*c + 2*Power(b,3)*c - 4*b*Power(c,2)*(3*a + c) - 2*Power(c,2)*(-3*Power(a,2) + 6*a*c + Power(c,2))))*r3*r4 - (Power(ax,4)*b*Power(c,3) - Power(a,2)*Power(ax - bx,3)*c*(3*ax*b - 3*b*bx + 8*ax*c) + a*(ax - bx)*(-(Power(b,3)*Power(bx,3)) + ax*Power(b,2)*Power(bx,2)*(3*b + 4*c) - Power(ax,2)*b*bx*(3*Power(b,2) + 8*b*c + 6*Power(c,2)) + Power(ax,3)*(Power(b,3) + 4*Power(b,2)*c + 6*b*Power(c,2) + 8*Power(c,3))))/(a*r5) + Power(ax - bx,3)*(-(b*bx) + ax*(b + 2*c))*Log(a) - Power(ax - bx,3)*(-(b*bx) + ax*(b + 2*c))*Log(r6))/Power(c,3);
		W(3,4) = (2*Power(ax - bx,3)*(ay - by)*c + (2*(-(Power(b,3)*Power(bx,3)*(-(b*by) + ay*(b + c))) + 3*ax*Power(b,2)*Power(bx,2)*(b + c)*(-(b*by) + ay*(b + c)) - 3*Power(ax,2)*b*bx*Power(b + c,2)*(-(b*by) + ay*(b + c)) - Power(a,2)*Power(ax - bx,2)*c*(ax*(3*ay*b - 3*b*by + 6*ay*c - 4*by*c) + bx*(-3*ay*b + 3*b*by - 2*by*c)) + Power(ax,3)*(b + c)*(-(b*by*Power(b + c,2)) + ay*(Power(b,3) + 3*Power(b,2)*c + 3*b*Power(c,2) + 2*Power(c,3))) + a*(-3*Power(ax,2)*bx*(b + c)*(ay*(Power(b,2) - 2*b*c - 4*Power(c,2)) + by*(-Power(b,2) + 3*b*c + 2*Power(c,2))) + b*Power(bx,3)*(b*by*(b - 4*c) + ay*(-Power(b,2) + 3*b*c + 3*Power(c,2))) + 3*ax*Power(bx,2)*(b*by*(-Power(b,2) + 3*b*c + 3*Power(c,2)) + ay*(Power(b,3) - 2*Power(b,2)*c - 5*b*Power(c,2) - 2*Power(c,3))) + Power(ax,3)*(ay*(Power(b,3) - 6*b*Power(c,2) - 4*Power(c,3)) + by*(-Power(b,3) + Power(b,2)*c + 6*b*Power(c,2) + 4*Power(c,3))))))/(4*Power(a,2)*c - Power(b,2)*(b + c) + a*(-Power(b,2) + 4*b*c + 4*Power(c,2))) + 2*(Power(bx,3)*(2*by*(Power(b,4) - 6*a*Power(b,2)*c + 6*Power(a,2)*Power(c,2)) - ay*(2*Power(b,4) - 12*a*Power(b,2)*c + Power(b,3)*c + 12*Power(a,2)*Power(c,2) - 6*a*b*Power(c,2))) + 3*ax*Power(bx,2)*(-(by*(2*Power(b,4) - 12*a*Power(b,2)*c + Power(b,3)*c + 12*Power(a,2)*Power(c,2) - 6*a*b*Power(c,2))) + 2*ay*(Power(b,4) - 6*a*Power(b,2)*c + Power(b,3)*c - 6*a*b*Power(c,2) + 2*a*(3*a - c)*Power(c,2))) + 3*Power(ax,2)*bx*(2*by*(Power(b,4) - 6*a*Power(b,2)*c + Power(b,3)*c - 6*a*b*Power(c,2) + 2*a*(3*a - c)*Power(c,2)) + ay*(-2*Power(b,4) + 12*a*Power(b,2)*c - 3*Power(b,3)*c + 12*a*Power(c,2)*(-a + c) + 2*b*Power(c,2)*(9*a + c))) + Power(ax,3)*(by*(-2*Power(b,4) + 12*a*Power(b,2)*c - 3*Power(b,3)*c + 12*a*Power(c,2)*(-a + c) + 2*b*Power(c,2)*(9*a + c)) + 2*ay*(Power(b,4) - 6*a*Power(b,2)*c + 2*Power(b,3)*c - 4*b*Power(c,2)*(3*a + c) - 2*Power(c,2)*(-3*Power(a,2) + 6*a*c + Power(c,2)))))*r2*r4 - 2*(Power(bx,3)*(2*by*(Power(b,4) - 6*a*Power(b,2)*c + 6*Power(a,2)*Power(c,2)) - ay*(2*Power(b,4) - 12*a*Power(b,2)*c + Power(b,3)*c + 12*Power(a,2)*Power(c,2) - 6*a*b*Power(c,2))) + 3*ax*Power(bx,2)*(-(by*(2*Power(b,4) - 12*a*Power(b,2)*c + Power(b,3)*c + 12*Power(a,2)*Power(c,2) - 6*a*b*Power(c,2))) + 2*ay*(Power(b,4) - 6*a*Power(b,2)*c + Power(b,3)*c - 6*a*b*Power(c,2) + 2*a*(3*a - c)*Power(c,2))) + 3*Power(ax,2)*bx*(2*by*(Power(b,4) - 6*a*Power(b,2)*c + Power(b,3)*c - 6*a*b*Power(c,2) + 2*a*(3*a - c)*Power(c,2)) + ay*(-2*Power(b,4) + 12*a*Power(b,2)*c - 3*Power(b,3)*c + 12*a*Power(c,2)*(-a + c) + 2*b*Power(c,2)*(9*a + c))) + Power(ax,3)*(by*(-2*Power(b,4) + 12*a*Power(b,2)*c - 3*Power(b,3)*c + 12*a*Power(c,2)*(-a + c) + 2*b*Power(c,2)*(9*a + c)) + 2*ay*(Power(b,4) - 6*a*Power(b,2)*c + 2*Power(b,3)*c - 4*b*Power(c,2)*(3*a + c) - 2*Power(c,2)*(-3*Power(a,2) + 6*a*c + Power(c,2)))))*r3*r4 + (2*(-(Power(ax,3)*ay*b*Power(c,3)) + Power(a,2)*Power(ax - bx,2)*c*(bx*(-3*ay*b + 3*b*by - 2*ay*c) + ax*(-3*by*(b + 2*c) + ay*(3*b + 8*c))) + a*(Power(b,2)*Power(bx,3)*(-(b*by) + ay*(b + c)) - 3*ax*b*Power(bx,2)*(b + c)*(-(b*by) + ay*(b + c)) + 3*Power(ax,2)*bx*(-(b*by*Power(b + c,2)) + ay*(Power(b,3) + 3*Power(b,2)*c + 3*b*Power(c,2) + 2*Power(c,3))) + Power(ax,3)*(by*(Power(b,3) + 3*Power(b,2)*c + 3*b*Power(c,2) + 2*Power(c,3)) - ay*(Power(b,3) + 4*Power(b,2)*c + 6*b*Power(c,2) + 8*Power(c,3))))))/(a*r5) + Power(ax - bx,2)*(ax*(2*ay*b - 2*b*by + 4*ay*c - 3*by*c) + bx*(2*b*by - ay*(2*b + c)))*Log(a) - Power(ax - bx,2)*(ax*(2*ay*b - 2*b*by + 4*ay*c - 3*by*c) + bx*(2*b*by - ay*(2*b + c)))*Log(r6))/(2.*Power(c,3));
		W(3,5) = (Power(ax - bx,2)*Power(ay - by,2)*c - (Power(b,2)*Power(bx,2)*Power(b*by - ay*(b + c),2) - 2*ax*b*bx*(b + c)*Power(b*by - ay*(b + c),2) - Power(a,2)*(ax - bx)*(ay - by)*c*(ax*(3*ay*b - 3*b*by + 6*ay*c - 2*by*c) - bx*(3*ay*b - 3*b*by + 2*ay*c + 2*by*c)) + Power(ax,2)*(Power(b,2)*Power(by,2)*Power(b + c,2) - 2*ay*b*by*Power(b + c,3) + Power(ay,2)*(Power(b,4) + 4*Power(b,3)*c + 6*Power(b,2)*Power(c,2) + 5*b*Power(c,3) + 2*Power(c,4))) + a*(Power(bx,2)*(-(b*by) + ay*(b + c))*(-(b*by*(b - 4*c)) + ay*(Power(b,2) - 3*b*c - 2*Power(c,2))) - 2*ax*bx*(b*Power(by,2)*(Power(b,2) - 3*b*c - 3*Power(c,2)) - 2*ay*by*(b + c)*(Power(b,2) - 3*b*c - 2*Power(c,2)) + Power(ay,2)*(Power(b,3) - Power(b,2)*c - 6*b*Power(c,2) - 4*Power(c,3))) + Power(ax,2)*(Power(by,2)*(b + c)*(Power(b,2) - 3*b*c - 2*Power(c,2)) + Power(ay,2)*(Power(b,3) - 6*b*Power(c,2) - 4*Power(c,3)) - 2*ay*by*(Power(b,3) - Power(b,2)*c - 6*b*Power(c,2) - 4*Power(c,3)))))/(-4*Power(a,2)*c + Power(b,2)*(b + c) + a*(Power(b,2) - 4*b*c - 4*Power(c,2))) + 2*(Power(bx,2)*(Power(by,2)*(Power(b,4) - 6*a*Power(b,2)*c + 6*Power(a,2)*Power(c,2)) - ay*by*(2*Power(b,4) - 12*a*Power(b,2)*c + Power(b,3)*c + 12*Power(a,2)*Power(c,2) - 6*a*b*Power(c,2)) + Power(ay,2)*(Power(b,4) - 6*a*Power(b,2)*c + Power(b,3)*c - 6*a*b*Power(c,2) + 2*a*(3*a - c)*Power(c,2))) + ax*bx*(-(Power(by,2)*(2*Power(b,4) - 12*a*Power(b,2)*c + Power(b,3)*c + 12*Power(a,2)*Power(c,2) - 6*a*b*Power(c,2))) + 4*ay*by*(Power(b,4) - 6*a*Power(b,2)*c + Power(b,3)*c - 6*a*b*Power(c,2) + 2*a*(3*a - c)*Power(c,2)) + Power(ay,2)*(-2*Power(b,4) + 12*a*Power(b,2)*c - 3*Power(b,3)*c + 12*a*Power(c,2)*(-a + c) + 2*b*Power(c,2)*(9*a + c))) + Power(ax,2)*(Power(by,2)*(Power(b,4) - 6*a*Power(b,2)*c + Power(b,3)*c - 6*a*b*Power(c,2) + 2*a*(3*a - c)*Power(c,2)) + ay*by*(-2*Power(b,4) + 12*a*Power(b,2)*c - 3*Power(b,3)*c + 12*a*Power(c,2)*(-a + c) + 2*b*Power(c,2)*(9*a + c)) + Power(ay,2)*(Power(b,4) - 6*a*Power(b,2)*c + 2*Power(b,3)*c - 4*b*Power(c,2)*(3*a + c) - 2*Power(c,2)*(-3*Power(a,2) + 6*a*c + Power(c,2)))))*r2*r4 - 2*(Power(bx,2)*(Power(by,2)*(Power(b,4) - 6*a*Power(b,2)*c + 6*Power(a,2)*Power(c,2)) - ay*by*(2*Power(b,4) - 12*a*Power(b,2)*c + Power(b,3)*c + 12*Power(a,2)*Power(c,2) - 6*a*b*Power(c,2)) + Power(ay,2)*(Power(b,4) - 6*a*Power(b,2)*c + Power(b,3)*c - 6*a*b*Power(c,2) + 2*a*(3*a - c)*Power(c,2))) + ax*bx*(-(Power(by,2)*(2*Power(b,4) - 12*a*Power(b,2)*c + Power(b,3)*c + 12*Power(a,2)*Power(c,2) - 6*a*b*Power(c,2))) + 4*ay*by*(Power(b,4) - 6*a*Power(b,2)*c + Power(b,3)*c - 6*a*b*Power(c,2) + 2*a*(3*a - c)*Power(c,2)) + Power(ay,2)*(-2*Power(b,4) + 12*a*Power(b,2)*c - 3*Power(b,3)*c + 12*a*Power(c,2)*(-a + c) + 2*b*Power(c,2)*(9*a + c))) + Power(ax,2)*(Power(by,2)*(Power(b,4) - 6*a*Power(b,2)*c + Power(b,3)*c - 6*a*b*Power(c,2) + 2*a*(3*a - c)*Power(c,2)) + ay*by*(-2*Power(b,4) + 12*a*Power(b,2)*c - 3*Power(b,3)*c + 12*a*Power(c,2)*(-a + c) + 2*b*Power(c,2)*(9*a + c)) + Power(ay,2)*(Power(b,4) - 6*a*Power(b,2)*c + 2*Power(b,3)*c - 4*b*Power(c,2)*(3*a + c) - 2*Power(c,2)*(-3*Power(a,2) + 6*a*c + Power(c,2)))))*r3*r4 - (Power(ax,2)*Power(ay,2)*b*Power(c,3) - Power(a,2)*(ax - bx)*(ay - by)*c*(bx*(-3*ay*b + 3*b*by - 4*ay*c) + ax*(3*ay*b - 3*b*by + 8*ay*c - 4*by*c)) + a*(b*Power(bx,2)*Power(b*by - ay*(b + c),2) - 2*ax*bx*(Power(b,2)*Power(by,2)*(b + c) - 2*ay*b*by*Power(b + c,2) + Power(ay,2)*(Power(b,3) + 3*Power(b,2)*c + 3*b*Power(c,2) + 2*Power(c,3))) + Power(ax,2)*(b*Power(by,2)*Power(b + c,2) - 2*ay*by*(Power(b,3) + 3*Power(b,2)*c + 3*b*Power(c,2) + 2*Power(c,3)) + Power(ay,2)*(Power(b,3) + 4*Power(b,2)*c + 6*b*Power(c,2) + 8*Power(c,3)))))/(a*r5) + (ax - bx)*(ay - by)*(-(bx*(-(b*by) + ay*(b + c))) + ax*(-(by*(b + c)) + ay*(b + 2*c)))*Log(a) - (ax - bx)*(ay - by)*(-(bx*(-(b*by) + ay*(b + c))) + ax*(-(by*(b + c)) + ay*(b + 2*c)))*Log(r6))/Power(c,3);
		W(4,4) = (Power(ax - bx,2)*Power(ay - by,2)*c - (Power(b,2)*Power(bx,2)*Power(b*by - ay*(b + c),2) - 2*ax*b*bx*(b + c)*Power(b*by - ay*(b + c),2) - Power(a,2)*(ax - bx)*(ay - by)*c*(ax*(3*ay*b - 3*b*by + 6*ay*c - 2*by*c) - bx*(3*ay*b - 3*b*by + 2*ay*c + 2*by*c)) + Power(ax,2)*(Power(b,2)*Power(by,2)*Power(b + c,2) - 2*ay*b*by*Power(b + c,3) + Power(ay,2)*(Power(b,4) + 4*Power(b,3)*c + 6*Power(b,2)*Power(c,2) + 5*b*Power(c,3) + 2*Power(c,4))) + a*(Power(bx,2)*(-(b*by) + ay*(b + c))*(-(b*by*(b - 4*c)) + ay*(Power(b,2) - 3*b*c - 2*Power(c,2))) - 2*ax*bx*(b*Power(by,2)*(Power(b,2) - 3*b*c - 3*Power(c,2)) - 2*ay*by*(b + c)*(Power(b,2) - 3*b*c - 2*Power(c,2)) + Power(ay,2)*(Power(b,3) - Power(b,2)*c - 6*b*Power(c,2) - 4*Power(c,3))) + Power(ax,2)*(Power(by,2)*(b + c)*(Power(b,2) - 3*b*c - 2*Power(c,2)) + Power(ay,2)*(Power(b,3) - 6*b*Power(c,2) - 4*Power(c,3)) - 2*ay*by*(Power(b,3) - Power(b,2)*c - 6*b*Power(c,2) - 4*Power(c,3)))))/(-4*Power(a,2)*c + Power(b,2)*(b + c) + a*(Power(b,2) - 4*b*c - 4*Power(c,2))) + 2*(Power(bx,2)*(Power(by,2)*(Power(b,4) - 6*a*Power(b,2)*c + 6*Power(a,2)*Power(c,2)) - ay*by*(2*Power(b,4) - 12*a*Power(b,2)*c + Power(b,3)*c + 12*Power(a,2)*Power(c,2) - 6*a*b*Power(c,2)) + Power(ay,2)*(Power(b,4) - 6*a*Power(b,2)*c + Power(b,3)*c - 6*a*b*Power(c,2) + 2*a*(3*a - c)*Power(c,2))) + ax*bx*(-(Power(by,2)*(2*Power(b,4) - 12*a*Power(b,2)*c + Power(b,3)*c + 12*Power(a,2)*Power(c,2) - 6*a*b*Power(c,2))) + 4*ay*by*(Power(b,4) - 6*a*Power(b,2)*c + Power(b,3)*c - 6*a*b*Power(c,2) + 2*a*(3*a - c)*Power(c,2)) + Power(ay,2)*(-2*Power(b,4) + 12*a*Power(b,2)*c - 3*Power(b,3)*c + 12*a*Power(c,2)*(-a + c) + 2*b*Power(c,2)*(9*a + c))) + Power(ax,2)*(Power(by,2)*(Power(b,4) - 6*a*Power(b,2)*c + Power(b,3)*c - 6*a*b*Power(c,2) + 2*a*(3*a - c)*Power(c,2)) + ay*by*(-2*Power(b,4) + 12*a*Power(b,2)*c - 3*Power(b,3)*c + 12*a*Power(c,2)*(-a + c) + 2*b*Power(c,2)*(9*a + c)) + Power(ay,2)*(Power(b,4) - 6*a*Power(b,2)*c + 2*Power(b,3)*c - 4*b*Power(c,2)*(3*a + c) - 2*Power(c,2)*(-3*Power(a,2) + 6*a*c + Power(c,2)))))*r2*r4 - 2*(Power(bx,2)*(Power(by,2)*(Power(b,4) - 6*a*Power(b,2)*c + 6*Power(a,2)*Power(c,2)) - ay*by*(2*Power(b,4) - 12*a*Power(b,2)*c + Power(b,3)*c + 12*Power(a,2)*Power(c,2) - 6*a*b*Power(c,2)) + Power(ay,2)*(Power(b,4) - 6*a*Power(b,2)*c + Power(b,3)*c - 6*a*b*Power(c,2) + 2*a*(3*a - c)*Power(c,2))) + ax*bx*(-(Power(by,2)*(2*Power(b,4) - 12*a*Power(b,2)*c + Power(b,3)*c + 12*Power(a,2)*Power(c,2) - 6*a*b*Power(c,2))) + 4*ay*by*(Power(b,4) - 6*a*Power(b,2)*c + Power(b,3)*c - 6*a*b*Power(c,2) + 2*a*(3*a - c)*Power(c,2)) + Power(ay,2)*(-2*Power(b,4) + 12*a*Power(b,2)*c - 3*Power(b,3)*c + 12*a*Power(c,2)*(-a + c) + 2*b*Power(c,2)*(9*a + c))) + Power(ax,2)*(Power(by,2)*(Power(b,4) - 6*a*Power(b,2)*c + Power(b,3)*c - 6*a*b*Power(c,2) + 2*a*(3*a - c)*Power(c,2)) + ay*by*(-2*Power(b,4) + 12*a*Power(b,2)*c - 3*Power(b,3)*c + 12*a*Power(c,2)*(-a + c) + 2*b*Power(c,2)*(9*a + c)) + Power(ay,2)*(Power(b,4) - 6*a*Power(b,2)*c + 2*Power(b,3)*c - 4*b*Power(c,2)*(3*a + c) - 2*Power(c,2)*(-3*Power(a,2) + 6*a*c + Power(c,2)))))*r3*r4 - (Power(ax,2)*Power(ay,2)*b*Power(c,3) - Power(a,2)*(ax - bx)*(ay - by)*c*(bx*(-3*ay*b + 3*b*by - 4*ay*c) + ax*(3*ay*b - 3*b*by + 8*ay*c - 4*by*c)) + a*(b*Power(bx,2)*Power(b*by - ay*(b + c),2) - 2*ax*bx*(Power(b,2)*Power(by,2)*(b + c) - 2*ay*b*by*Power(b + c,2) + Power(ay,2)*(Power(b,3) + 3*Power(b,2)*c + 3*b*Power(c,2) + 2*Power(c,3))) + Power(ax,2)*(b*Power(by,2)*Power(b + c,2) - 2*ay*by*(Power(b,3) + 3*Power(b,2)*c + 3*b*Power(c,2) + 2*Power(c,3)) + Power(ay,2)*(Power(b,3) + 4*Power(b,2)*c + 6*b*Power(c,2) + 8*Power(c,3)))))/(a*r5) + (ax - bx)*(ay - by)*(-(bx*(-(b*by) + ay*(b + c))) + ax*(-(by*(b + c)) + ay*(b + 2*c)))*Log(a) - (ax - bx)*(ay - by)*(-(bx*(-(b*by) + ay*(b + c))) + ax*(-(by*(b + c)) + ay*(b + 2*c)))*Log(r6))/Power(c,3);
		W(4,5) = (2*(ax - bx)*Power(ay - by,3)*c + (2*(b*bx*Power(-(b*by) + ay*(b + c),3) - ax*(-(Power(b,3)*Power(by,3)*(b + c)) + 3*ay*Power(b,2)*Power(by,2)*Power(b + c,2) - 3*Power(ay,2)*b*by*Power(b + c,3) + Power(ay,3)*(Power(b,4) + 4*Power(b,3)*c + 6*Power(b,2)*Power(c,2) + 5*b*Power(c,3) + 2*Power(c,4))) + Power(a,2)*Power(ay - by,2)*c*(-(bx*(3*ay*b - 3*b*by + 4*ay*c + 2*by*c)) + 3*ax*(-(b*by) + ay*(b + 2*c))) + a*(bx*(-(Power(b,2)*Power(by,3)*(b - 4*c)) + 3*ay*b*Power(by,2)*(Power(b,2) - 3*b*c - 3*Power(c,2)) - 3*Power(ay,2)*by*(b + c)*(Power(b,2) - 3*b*c - 2*Power(c,2)) + Power(ay,3)*(Power(b,3) - Power(b,2)*c - 6*b*Power(c,2) - 4*Power(c,3))) + ax*(b*Power(by,3)*(Power(b,2) - 3*b*c - 3*Power(c,2)) - 3*ay*Power(by,2)*(b + c)*(Power(b,2) - 3*b*c - 2*Power(c,2)) - Power(ay,3)*(Power(b,3) - 6*b*Power(c,2) - 4*Power(c,3)) + 3*Power(ay,2)*by*(Power(b,3) - Power(b,2)*c - 6*b*Power(c,2) - 4*Power(c,3))))))/(-4*Power(a,2)*c + Power(b,2)*(b + c) + a*(Power(b,2) - 4*b*c - 4*Power(c,2))) + 2*(bx*(2*Power(by,3)*(Power(b,4) - 6*a*Power(b,2)*c + 6*Power(a,2)*Power(c,2)) - 3*ay*Power(by,2)*(2*Power(b,4) - 12*a*Power(b,2)*c + Power(b,3)*c + 12*Power(a,2)*Power(c,2) - 6*a*b*Power(c,2)) + 6*Power(ay,2)*by*(Power(b,4) - 6*a*Power(b,2)*c + Power(b,3)*c - 6*a*b*Power(c,2) + 2*a*(3*a - c)*Power(c,2)) + Power(ay,3)*(-2*Power(b,4) + 12*a*Power(b,2)*c - 3*Power(b,3)*c + 12*a*Power(c,2)*(-a + c) + 2*b*Power(c,2)*(9*a + c))) + ax*(-(Power(by,3)*(2*Power(b,4) - 12*a*Power(b,2)*c + Power(b,3)*c + 12*Power(a,2)*Power(c,2) - 6*a*b*Power(c,2))) + 6*ay*Power(by,2)*(Power(b,4) - 6*a*Power(b,2)*c + Power(b,3)*c - 6*a*b*Power(c,2) + 2*a*(3*a - c)*Power(c,2)) + 3*Power(ay,2)*by*(-2*Power(b,4) + 12*a*Power(b,2)*c - 3*Power(b,3)*c + 12*a*Power(c,2)*(-a + c) + 2*b*Power(c,2)*(9*a + c)) + 2*Power(ay,3)*(Power(b,4) - 6*a*Power(b,2)*c + 2*Power(b,3)*c - 4*b*Power(c,2)*(3*a + c) - 2*Power(c,2)*(-3*Power(a,2) + 6*a*c + Power(c,2)))))*r2*r4 - 2*(bx*(2*Power(by,3)*(Power(b,4) - 6*a*Power(b,2)*c + 6*Power(a,2)*Power(c,2)) - 3*ay*Power(by,2)*(2*Power(b,4) - 12*a*Power(b,2)*c + Power(b,3)*c + 12*Power(a,2)*Power(c,2) - 6*a*b*Power(c,2)) + 6*Power(ay,2)*by*(Power(b,4) - 6*a*Power(b,2)*c + Power(b,3)*c - 6*a*b*Power(c,2) + 2*a*(3*a - c)*Power(c,2)) + Power(ay,3)*(-2*Power(b,4) + 12*a*Power(b,2)*c - 3*Power(b,3)*c + 12*a*Power(c,2)*(-a + c) + 2*b*Power(c,2)*(9*a + c))) + ax*(-(Power(by,3)*(2*Power(b,4) - 12*a*Power(b,2)*c + Power(b,3)*c + 12*Power(a,2)*Power(c,2) - 6*a*b*Power(c,2))) + 6*ay*Power(by,2)*(Power(b,4) - 6*a*Power(b,2)*c + Power(b,3)*c - 6*a*b*Power(c,2) + 2*a*(3*a - c)*Power(c,2)) + 3*Power(ay,2)*by*(-2*Power(b,4) + 12*a*Power(b,2)*c - 3*Power(b,3)*c + 12*a*Power(c,2)*(-a + c) + 2*b*Power(c,2)*(9*a + c)) + 2*Power(ay,3)*(Power(b,4) - 6*a*Power(b,2)*c + 2*Power(b,3)*c - 4*b*Power(c,2)*(3*a + c) - 2*Power(c,2)*(-3*Power(a,2) + 6*a*c + Power(c,2)))))*r3*r4 + (2*(-(ax*Power(ay,3)*b*Power(c,3)) + Power(a,2)*Power(ay - by,2)*c*(ax*(3*ay*b - 3*b*by + 8*ay*c - 2*by*c) + 3*bx*(b*by - ay*(b + 2*c))) + a*(bx*(-(Power(b,3)*Power(by,3)) + 3*ay*Power(b,2)*Power(by,2)*(b + c) - 3*Power(ay,2)*b*by*Power(b + c,2) + Power(ay,3)*(Power(b,3) + 3*Power(b,2)*c + 3*b*Power(c,2) + 2*Power(c,3))) + ax*(Power(b,2)*Power(by,3)*(b + c) - 3*ay*b*Power(by,2)*Power(b + c,2) + 3*Power(ay,2)*by*(Power(b,3) + 3*Power(b,2)*c + 3*b*Power(c,2) + 2*Power(c,3)) - Power(ay,3)*(Power(b,3) + 4*Power(b,2)*c + 6*b*Power(c,2) + 8*Power(c,3))))))/(a*r5) - Power(ay - by,2)*(bx*(2*ay*b - 2*b*by + 3*ay*c) + ax*(by*(2*b + c) - 2*ay*(b + 2*c)))*Log(a) + Power(ay - by,2)*(bx*(2*ay*b - 2*b*by + 3*ay*c) + ax*(by*(2*b + c) - 2*ay*(b + 2*c)))*Log(r6))/(2.*Power(c,3));
		W(5,5) = (Power(ay - by,4)*c + (Power(b,4)*Power(by,4) - 4*ay*Power(b,3)*Power(by,3)*(b + c) + 6*Power(ay,2)*Power(b,2)*Power(by,2)*Power(b + c,2) - 4*Power(ay,3)*b*by*Power(b + c,3) + Power(ay,4)*(Power(b,4) + 4*Power(b,3)*c + 6*Power(b,2)*Power(c,2) + 5*b*Power(c,3) + 2*Power(c,4)) - Power(a,2)*Power(ay - by,3)*c*(-3*b*by + 2*by*c + 3*ay*(b + 2*c)) + a*(ay - by)*(-(Power(b,2)*Power(by,3)*(b - 4*c)) + ay*b*Power(by,2)*(3*Power(b,2) - 8*b*c - 12*Power(c,2)) + Power(ay,3)*(Power(b,3) - 6*b*Power(c,2) - 4*Power(c,3)) + Power(ay,2)*by*(-3*Power(b,3) + 4*Power(b,2)*c + 18*b*Power(c,2) + 12*Power(c,3))))/(4*Power(a,2)*c - Power(b,2)*(b + c) + a*(-Power(b,2) + 4*b*c + 4*Power(c,2))) + 2*(Power(by,4)*(Power(b,4) - 6*a*Power(b,2)*c + 6*Power(a,2)*Power(c,2)) - 2*ay*Power(by,3)*(2*Power(b,4) - 12*a*Power(b,2)*c + Power(b,3)*c + 12*Power(a,2)*Power(c,2) - 6*a*b*Power(c,2)) + 6*Power(ay,2)*Power(by,2)*(Power(b,4) - 6*a*Power(b,2)*c + Power(b,3)*c - 6*a*b*Power(c,2) + 2*a*(3*a - c)*Power(c,2)) + 2*Power(ay,3)*by*(-2*Power(b,4) + 12*a*Power(b,2)*c - 3*Power(b,3)*c + 12*a*Power(c,2)*(-a + c) + 2*b*Power(c,2)*(9*a + c)) + Power(ay,4)*(Power(b,4) - 6*a*Power(b,2)*c + 2*Power(b,3)*c - 4*b*Power(c,2)*(3*a + c) - 2*Power(c,2)*(-3*Power(a,2) + 6*a*c + Power(c,2))))*r2*r4 - 2*(Power(by,4)*(Power(b,4) - 6*a*Power(b,2)*c + 6*Power(a,2)*Power(c,2)) - 2*ay*Power(by,3)*(2*Power(b,4) - 12*a*Power(b,2)*c + Power(b,3)*c + 12*Power(a,2)*Power(c,2) - 6*a*b*Power(c,2)) + 6*Power(ay,2)*Power(by,2)*(Power(b,4) - 6*a*Power(b,2)*c + Power(b,3)*c - 6*a*b*Power(c,2) + 2*a*(3*a - c)*Power(c,2)) + 2*Power(ay,3)*by*(-2*Power(b,4) + 12*a*Power(b,2)*c - 3*Power(b,3)*c + 12*a*Power(c,2)*(-a + c) + 2*b*Power(c,2)*(9*a + c)) + Power(ay,4)*(Power(b,4) - 6*a*Power(b,2)*c + 2*Power(b,3)*c - 4*b*Power(c,2)*(3*a + c) - 2*Power(c,2)*(-3*Power(a,2) + 6*a*c + Power(c,2))))*r3*r4 - (Power(ay,4)*b*Power(c,3) - Power(a,2)*Power(ay - by,3)*c*(3*ay*b - 3*b*by + 8*ay*c) + a*(ay - by)*(-(Power(b,3)*Power(by,3)) + ay*Power(b,2)*Power(by,2)*(3*b + 4*c) - Power(ay,2)*b*by*(3*Power(b,2) + 8*b*c + 6*Power(c,2)) + Power(ay,3)*(Power(b,3) + 4*Power(b,2)*c + 6*b*Power(c,2) + 8*Power(c,3))))/(a*r5) + Power(ay - by,3)*(-(b*by) + ay*(b + 2*c))*Log(a) - Power(ay - by,3)*(-(b*by) + ay*(b + 2*c))*Log(r6))/Power(c,3);

		M += W*sqrt_c;

		// add derivative matrix P'tP'
		if (g.deriv >= 1)
		{
		W(0,0) = 0;
		W(0,1) = 0;
		W(0,2) = 0;
		W(0,3) = 0;
		W(0,4) = 0;
		W(0,5) = 0;
		W(1,1) = -((Power(ay - by,2)*(b/a + (4*c*r2)/r1 - (4*c*r3)/r1 - (b + 2*c)/r6))/r5);
		W(1,2) = -(((-ax + bx)*(ay - by)*(b/a + (4*c*r2)/r1 - (4*c*r3)/r1 - (b + 2*c)/r6))/r5);
		W(1,3) = (-2*Power(ay - by,2)*(ax*(2 + b/a) - 2*bx + (2*(-(b*bx) + ax*(b + 2*c))*r2)/r1 - (2*(-(b*bx) + ax*(b + 2*c))*r3)/r1 + (-2*a*(ax - bx) + b*bx - 2*ax*(b + c))/r6))/r5;
		W(1,4) = (ay - by)*((-2*Power(ay,2)*b + b*Power(bx,2) + 3*ay*b*by - b*Power(by,2) + 2*a*(Power(ax,2) - Power(ay,2) - 2*ax*bx + Power(bx,2) + 2*ay*by - Power(by,2)) - 2*Power(ay,2)*c + 2*ay*by*c + 2*Power(ax,2)*(b + c) - ax*bx*(3*b + 2*c))/(-4*Power(a,2)*c + Power(b,2)*(b + c) + a*(Power(b,2) - 4*b*c - 4*Power(c,2))) + 2*(b*(Power(bx,2) - Power(by,2)) - 2*ax*bx*(b + c) + 2*ay*by*(b + c) + Power(ax,2)*(b + 2*c) - Power(ay,2)*(b + 2*c))*r2*r4 - 2*(b*(Power(bx,2) - Power(by,2)) - 2*ax*bx*(b + c) + 2*ay*by*(b + c) + Power(ax,2)*(b + 2*c) - Power(ay,2)*(b + 2*c))*r3*r4 + (2*a*(Power(ax,2) - Power(ay,2) - 2*ax*bx + Power(bx,2) + 2*ay*by - Power(by,2)) + b*(Power(ax,2) - ax*bx + ay*(-ay + by)))/(a*r5));
		W(1,5) = (-2*(ax - bx)*(ay - by)*(-((ay*(2*a + b))/a) + 2*by - (2*(-(b*by) + ay*(b + 2*c))*r2)/r1 + (2*(-(b*by) + ay*(b + 2*c))*r3)/r1 + (2*a*(ay - by) - b*by + 2*ay*(b + c))/r6))/r5;
		W(2,2) = -((Power(ax - bx,2)*(b/a + (4*c*r2)/r1 - (4*c*r3)/r1 - (b + 2*c)/r6))/r5);
		W(2,3) = (-2*(ax - bx)*(ay - by)*(-((ax*(2*a + b))/a) + 2*bx - (2*(-(b*bx) + ax*(b + 2*c))*r2)/r1 + (2*(-(b*bx) + ax*(b + 2*c))*r3)/r1 + (2*a*(ax - bx) - b*bx + 2*ax*(b + c))/r6))/r5;
		W(2,4) = (-ax + bx)*((-2*Power(ay,2)*b + b*Power(bx,2) + 3*ay*b*by - b*Power(by,2) + 2*a*(Power(ax,2) - Power(ay,2) - 2*ax*bx + Power(bx,2) + 2*ay*by - Power(by,2)) - 2*Power(ay,2)*c + 2*ay*by*c + 2*Power(ax,2)*(b + c) - ax*bx*(3*b + 2*c))/(-4*Power(a,2)*c + Power(b,2)*(b + c) + a*(Power(b,2) - 4*b*c - 4*Power(c,2))) + 2*(b*(Power(bx,2) - Power(by,2)) - 2*ax*bx*(b + c) + 2*ay*by*(b + c) + Power(ax,2)*(b + 2*c) - Power(ay,2)*(b + 2*c))*r2*r4 - 2*(b*(Power(bx,2) - Power(by,2)) - 2*ax*bx*(b + c) + 2*ay*by*(b + c) + Power(ax,2)*(b + 2*c) - Power(ay,2)*(b + 2*c))*r3*r4 + (2*a*(Power(ax,2) - Power(ay,2) - 2*ax*bx + Power(bx,2) + 2*ay*by - Power(by,2)) + b*(Power(ax,2) - ax*bx + ay*(-ay + by)))/(a*r5));
		W(2,5) = (-2*Power(ax - bx,2)*(ay*(2 + b/a) - 2*by + (2*(-(b*by) + ay*(b + 2*c))*r2)/r1 - (2*(-(b*by) + ay*(b + 2*c))*r3)/r1 + (-2*a*(ay - by) + b*by - 2*ay*(b + c))/r6))/r5;
		W(3,3) = (-4*Power(ay - by,2)*((Power(ax,2)*b*c + a*(ax - bx)*(-(b*bx) + ax*(b + 4*c)))/(a*c) + (4*(a*Power(ax - bx,2) + ax*(-(b*bx) + ax*(b + c)))*r2)/r1 - (4*(a*Power(ax - bx,2) + ax*(-(b*bx) + ax*(b + c)))*r3)/r1 - (Power(b,2)*Power(bx,2) - 2*ax*b*bx*(b + c) + a*(ax - bx)*(ax*b - b*bx + 2*ax*c + 2*bx*c) + Power(ax,2)*(Power(b,2) + 3*b*c + 2*Power(c,2)))/(c*r6)))/r5;
		W(3,4) = (-2*(ay - by)*((ax*b*(-Power(ax,2) + ax*bx + ay*(ay - by))*c + a*(-(Power(ax,3)*(b + 4*c)) + Power(ax,2)*bx*(3*b + 8*c) + bx*(b*(Power(bx,2) - Power(by,2)) + 2*ay*by*(b + c) - Power(ay,2)*(b + 2*c)) + ax*(-3*b*Power(bx,2) + b*Power(by,2) - 4*Power(bx,2)*c + 2*Power(by,2)*c - 2*ay*by*(b + 3*c) + Power(ay,2)*(b + 4*c))))/(a*c) - (2*(ay*b*bx*(ay - by) + 2*a*(ax - bx)*(Power(ax,2) - Power(ay,2) - 2*ax*bx + Power(bx,2) + 2*ay*by - Power(by,2)) + 2*Power(ax,3)*(b + c) - 2*Power(ax,2)*bx*(2*b + c) + ax*(b*(2*Power(bx,2) - Power(by,2)) - 2*Power(ay,2)*(b + c) + ay*by*(3*b + 2*c)))*r2)/r1 + (2*(ay*b*bx*(ay - by) + 2*a*(ax - bx)*(Power(ax,2) - Power(ay,2) - 2*ax*bx + Power(bx,2) + 2*ay*by - Power(by,2)) + 2*Power(ax,3)*(b + c) - 2*Power(ax,2)*bx*(2*b + c) + ax*(b*(2*Power(bx,2) - Power(by,2)) - 2*Power(ay,2)*(b + c) + ay*by*(3*b + 2*c)))*r3)/r1 + (Power(ax,3)*(Power(b,2) + 3*b*c + 2*Power(c,2)) - Power(ax,2)*bx*(3*Power(b,2) + 5*b*c + 2*Power(c,2)) + b*bx*(b*(-Power(bx,2) + Power(by,2)) + Power(ay,2)*(b + c) - ay*by*(2*b + c)) - ax*(-2*ay*by*Power(b + c,2) + b*(-3*b*Power(bx,2) + b*Power(by,2) - 2*Power(bx,2)*c + Power(by,2)*c) + Power(ay,2)*(Power(b,2) + 3*b*c + 2*Power(c,2))) + a*(Power(ax,3)*(b + 2*c) - Power(ax,2)*bx*(3*b + 2*c) + bx*(Power(ay,2)*b - (Power(bx,2) - Power(by,2))*(b - 2*c) + 2*ay*by*(-b + c)) - ax*(-3*b*Power(bx,2) + b*Power(by,2) + 2*Power(bx,2)*c - 2*ay*by*(b + c) + Power(ay,2)*(b + 2*c))))/(c*r6)))/r5;
		W(3,5) = (-4*(-ax + bx)*(ay - by)*((ax*ay*b*c + a*(bx*(b*by - ay*(b + 2*c)) + ax*(-(by*(b + 2*c)) + ay*(b + 4*c))))/(a*c) - (2*(ay*b*bx - 2*a*(ax - bx)*(ay - by) + ax*(b*by - 2*ay*(b + c)))*r2)/r1 + (2*(ay*b*bx - 2*a*(ax - bx)*(ay - by) + ax*(b*by - 2*ay*(b + c)))*r3)/r1 + (b*bx*(-(b*by) + ay*(b + c)) + ax*(b + c)*(b*by - ay*(b + 2*c)) + a*(bx*(ay*b - b*by + 2*by*c) + ax*(b*by - ay*(b + 2*c))))/(c*r6)))/r5;
		W(4,4) = -(((Power(ax,4)*b*c + Power(ay,4)*b*c - 2*Power(ax,3)*b*bx*c + 2*ax*ay*b*bx*(ay - by)*c - 2*Power(ay,3)*b*by*c + Power(ay,2)*b*Power(by,2)*c + Power(ax,2)*b*(-2*Power(ay,2) + Power(bx,2) + 2*ay*by)*c + a*(Power(ax,2) - Power(ay,2) - 2*ax*bx + Power(bx,2) + 2*ay*by - Power(by,2))*(b*(Power(bx,2) - Power(by,2)) - 2*ax*bx*(b + 2*c) + 2*ay*by*(b + 2*c) + Power(ax,2)*(b + 4*c) - Power(ay,2)*(b + 4*c)))/(a*c) + (4*(a*Power(Power(ax,2) - Power(ay,2) - 2*ax*bx + Power(bx,2) + 2*ay*by - Power(by,2),2) + (Power(ax,2) - ax*bx + ay*(-ay + by))*(b*(Power(bx,2) - Power(by,2)) + Power(ax,2)*(b + c) - Power(ay,2)*(b + c) - ax*bx*(2*b + c) + ay*by*(2*b + c)))*r2)/r1 - (4*(a*Power(Power(ax,2) - Power(ay,2) - 2*ax*bx + Power(bx,2) + 2*ay*by - Power(by,2),2) + (Power(ax,2) - ax*bx + ay*(-ay + by))*(b*(Power(bx,2) - Power(by,2)) + Power(ax,2)*(b + c) - Power(ay,2)*(b + c) - ax*bx*(2*b + c) + ay*by*(2*b + c)))*r3)/r1 - (Power(ay,4)*Power(b,2) - 2*Power(ay,2)*Power(b,2)*Power(bx,2) + Power(b,2)*Power(bx,4) - 4*Power(ay,3)*Power(b,2)*by + 4*ay*Power(b,2)*Power(bx,2)*by + 6*Power(ay,2)*Power(b,2)*Power(by,2) - 2*Power(b,2)*Power(bx,2)*Power(by,2) - 4*ay*Power(b,2)*Power(by,3) + Power(b,2)*Power(by,4) + 3*Power(ay,4)*b*c - 2*Power(ay,2)*b*Power(bx,2)*c - 8*Power(ay,3)*b*by*c + 2*ay*b*Power(bx,2)*by*c + 7*Power(ay,2)*b*Power(by,2)*c - 2*ay*b*Power(by,3)*c + 2*Power(ay,4)*Power(c,2) - 4*Power(ay,3)*by*Power(c,2) + 2*Power(ay,2)*Power(by,2)*Power(c,2) - 4*Power(ax,3)*bx*Power(b + c,2) + Power(ax,4)*(Power(b,2) + 3*b*c + 2*Power(c,2)) + a*(Power(ax,2) - Power(ay,2) - 2*ax*bx + Power(bx,2) + 2*ay*by - Power(by,2))*(-2*ax*b*bx + 2*ay*b*by + (Power(bx,2) - Power(by,2))*(b - 2*c) + Power(ax,2)*(b + 2*c) - Power(ay,2)*(b + 2*c)) + Power(ax,2)*(Power(b,2)*(6*Power(bx,2) - 2*Power(by,2)) + b*(7*Power(bx,2) - 2*Power(by,2))*c + 2*Power(bx,2)*Power(c,2) + 4*ay*by*Power(b + c,2) - 2*Power(ay,2)*(Power(b,2) + 3*b*c + 2*Power(c,2))) + 2*ax*bx*(2*Power(ay,2)*Power(b + c,2) - b*(Power(bx,2) - Power(by,2))*(2*b + c) - ay*by*(4*Power(b,2) + 5*b*c + 2*Power(c,2))))/(c*r6))/r5);
		W(4,5) = (-2*(-ax + bx)*((ay*b*(-Power(ax,2) + ax*bx + ay*(ay - by))*c + a*(b*by*(Power(bx,2) - Power(by,2)) + Power(ay,3)*(b + 4*c) - Power(ay,2)*by*(3*b + 8*c) + ay*(-(b*Power(bx,2)) + 3*b*Power(by,2) - 2*Power(bx,2)*c + 4*Power(by,2)*c) + 2*ax*bx*(-(by*(b + c)) + ay*(b + 3*c)) + Power(ax,2)*(by*(b + 2*c) - ay*(b + 4*c))))/(a*c) - (2*(2*a*(ay - by)*(Power(ax,2) - Power(ay,2) - 2*ax*bx + Power(bx,2) + 2*ay*by - Power(by,2)) + ax*bx*(-3*ay*b + b*by - 2*ay*c) + Power(ax,2)*(-(b*by) + 2*ay*(b + c)) + ay*(b*(Power(bx,2) - 2*Power(by,2)) - 2*Power(ay,2)*(b + c) + 2*ay*by*(2*b + c)))*r2)/r1 + (2*(2*a*(ay - by)*(Power(ax,2) - Power(ay,2) - 2*ax*bx + Power(bx,2) + 2*ay*by - Power(by,2)) + ax*bx*(-3*ay*b + b*by - 2*ay*c) + Power(ax,2)*(-(b*by) + 2*ay*(b + c)) + ay*(b*(Power(bx,2) - 2*Power(by,2)) - 2*Power(ay,2)*(b + c) + 2*ay*by*(2*b + c)))*r3)/r1 + (-(Power(ay,3)*Power(b,2)) + ay*Power(b,2)*Power(bx,2) + 3*Power(ay,2)*Power(b,2)*by - Power(b,2)*Power(bx,2)*by - 3*ay*Power(b,2)*Power(by,2) + Power(b,2)*Power(by,3) - 3*Power(ay,3)*b*c + ay*b*Power(bx,2)*c + 5*Power(ay,2)*b*by*c - 2*ay*b*Power(by,2)*c - 2*Power(ay,3)*Power(c,2) + 2*Power(ay,2)*by*Power(c,2) + ax*bx*(-2*ay*Power(b + c,2) + b*by*(2*b + c)) + Power(ax,2)*(b + c)*(-(b*by) + ay*(b + 2*c)) + a*(by*(-Power(bx,2) + Power(by,2))*(b - 2*c) - Power(ay,3)*(b + 2*c) + Power(ay,2)*by*(3*b + 2*c) + ay*(b*Power(bx,2) - 3*b*Power(by,2) + 2*Power(by,2)*c) - 2*ax*bx*(by*(-b + c) + ay*(b + c)) + Power(ax,2)*(-(b*by) + ay*(b + 2*c))))/(c*r6)))/r5;
		W(5,5) = (-4*Power(ax - bx,2)*((Power(ay,2)*b*c + a*(ay - by)*(-(b*by) + ay*(b + 4*c)))/(a*c) + (4*(a*Power(ay - by,2) + ay*(-(b*by) + ay*(b + c)))*r2)/r1 - (4*(a*Power(ay - by,2) + ay*(-(b*by) + ay*(b + c)))*r3)/r1 - (Power(b,2)*Power(by,2) - 2*ay*b*by*(b + c) + a*(ay - by)*(ay*b - b*by + 2*ay*c + 2*by*c) + Power(ay,2)*(Power(b,2) + 3*b*c + 2*Power(c,2)))/(c*r6)))/r5;

		M += W*(1.0/sqrt_c);
		}

		// add second derivative matrix P''tP''
		if (g.deriv >= 2)
		{
		W(0,0) = 0;
		W(0,1) = 0;
		W(0,2) = 0;
		W(0,3) = 0;
		W(0,4) = 0;
		W(0,5) = 0;
		W(1,1) = 0;
		W(1,2) = 0;
		W(1,3) = 0;
		W(1,4) = 0;
		W(1,5) = 0;
		W(2,2) = 0;
		W(2,3) = 0;
		W(2,4) = 0;
		W(2,5) = 0;
		W(3,3) = (-4*Power(ay - by,4)*(b/a + (4*c*r2)/r1 - (4*c*r3)/r1 - (b + 2*c)/r6))/r5;
		W(3,4) = (-4*(-ax + bx)*Power(ay - by,3)*(b/a + (4*c*r2)/r1 - (4*c*r3)/r1 - (b + 2*c)/r6))/r5;
		W(3,5) = (-4*Power(ax - bx,2)*Power(ay - by,2)*(b/a + (4*c*r2)/r1 - (4*c*r3)/r1 - (b + 2*c)/r6))/r5;
		W(4,4) = (-4*Power(ax - bx,2)*Power(ay - by,2)*(b/a + (4*c*r2)/r1 - (4*c*r3)/r1 - (b + 2*c)/r6))/r5;
		W(4,5) = (-4*Power(-ax + bx,3)*(ay - by)*(b/a + (4*c*r2)/r1 - (4*c*r3)/r1 - (b + 2*c)/r6))/r5;
		W(5,5) = (-4*Power(ax - bx,4)*(b/a + (4*c*r2)/r1 - (4*c*r3)/r1 - (b + 2*c)/r6))/r5;

		M += W*(1.0/(c*sqrt_c));
		}

		// copy symmetric parts
		M(1,0) = M(0,1);
		M(2,0) = M(0,2);
		M(3,0) = M(0,3);
		M(4,0) = M(0,4);
		M(5,0) = M(0,5);
		M(2,1) = M(1,2);
		M(3,1) = M(1,3);
		M(4,1) = M(1,4);
		M(5,1) = M(1,5);
		M(3,2) = M(2,3);
		M(4,2) = M(2,4);
		M(5,2) = M(2,5);
		M(4,3) = M(3,4);
		M(5,3) = M(3,5);
		M(5,4) = M(4,5);
	}

	Matrixd X(1,6);
	X(0,0) = 1;
	X(0,1) = x[0];
	X(0,2) = x[1];
	X(0,3) = x[0]*x[0];
	X(0,4) = x[1]*x[0];
	X(0,5) = x[1]*x[1];

	Matrixd k = X*M.pseudo_inverse();

	// calculate barycentric coordinates
	Matrixd B(6,1);
	for (int i = 0; i < n; i++)
	{
		// calculate some common terms
		vect2d &p1 = pv[p[i][0]];
		vect2d &p2 = pv[p[i][1]];
		double ax = p1[0];
		double ay = p1[1];
		double bx = p2[0];
		double by = p2[1];

		double a = (p1 - x)*(p1 - x);
		double b = (p1 - p2)*(p1 - x)*(-2);
		double c = (p1 - p2)*(p1 - p2);

		double sqrt_c = sqrt(c);

		double r5 = -b*b + 4*a*c;
		double r1 = sqrt(r5);
		double r4 = 1.0 / (r1*r5);
		double r2 = atan(b/r1);
		double r3 = atan((b+2*c)/r1);
		double r6 = a+b+c;

		// calc function coordinates
#ifdef QUADRIC_BOUNDARY_DERIV_FIGURE
		B(0,0) = -((r4*((2*a + b)*r1 + 4*a*r2*r6 - 4*a*r3*r6))/a);
		B(1,0) = ((2*(2*Power(a,2)*(ax - bx)*c - ax*b*Power(c,2) + a*(bx*(Power(b,2) + 2*b*c + 2*Power(c,2)) - ax*(Power(b,2) + 3*b*c + 6*Power(c,2)))))/(-(a*Power(b,2)) + 4*Power(a,2)*c) + 2*(ax*(b + 2*c)*(Power(b,2) - 2*b*c - 2*c*(3*a + c)) + bx*(-Power(b,3) + 8*a*Power(c,2) + 2*b*c*(3*a + c)))*r2*r4 - 2*(ax*(b + 2*c)*(Power(b,2) - 2*b*c - 2*c*(3*a + c)) + bx*(-Power(b,3) + 8*a*Power(c,2) + 2*b*c*(3*a + c)))*r3*r4 - (2*(bx*(Power(b,2) - 2*a*c + b*c) - ax*(Power(b,2) + 2*b*c + 2*c*(-a + c))))/r5 + (ax - bx)*Log(a) - (ax - bx)*Log(r6))/(2.*Power(c,2));
		B(2,0) = ((2*(2*Power(a,2)*(ay - by)*c - ay*b*Power(c,2) + a*(by*(Power(b,2) + 2*b*c + 2*Power(c,2)) - ay*(Power(b,2) + 3*b*c + 6*Power(c,2)))))/(-(a*Power(b,2)) + 4*Power(a,2)*c) + 2*(ay*(b + 2*c)*(Power(b,2) - 2*b*c - 2*c*(3*a + c)) + by*(-Power(b,3) + 8*a*Power(c,2) + 2*b*c*(3*a + c)))*r2*r4 - 2*(ay*(b + 2*c)*(Power(b,2) - 2*b*c - 2*c*(3*a + c)) + by*(-Power(b,3) + 8*a*Power(c,2) + 2*b*c*(3*a + c)))*r3*r4 - (2*(by*(Power(b,2) - 2*a*c + b*c) - ay*(Power(b,2) + 2*b*c + 2*c*(-a + c))))/r5 + (ay - by)*Log(a) - (ay - by)*Log(r6))/(2.*Power(c,2));
		B(3,0) = (Power(ax - bx,2)*c + 2*(Power(bx,2)*(Power(b,4) - 6*a*Power(b,2)*c + Power(b,3)*c - 6*a*b*Power(c,2) + 2*a*(3*a - c)*Power(c,2)) + ax*bx*(-2*Power(b,4) + 12*a*Power(b,2)*c - 3*Power(b,3)*c + 12*a*Power(c,2)*(-a + c) + 2*b*Power(c,2)*(9*a + c)) + Power(ax,2)*(Power(b,4) - 6*a*Power(b,2)*c + 2*Power(b,3)*c - 4*b*Power(c,2)*(3*a + c) - 2*Power(c,2)*(-3*Power(a,2) + 6*a*c + Power(c,2))))*r2*r4 - 2*(Power(bx,2)*(Power(b,4) - 6*a*Power(b,2)*c + Power(b,3)*c - 6*a*b*Power(c,2) + 2*a*(3*a - c)*Power(c,2)) + ax*bx*(-2*Power(b,4) + 12*a*Power(b,2)*c - 3*Power(b,3)*c + 12*a*Power(c,2)*(-a + c) + 2*b*Power(c,2)*(9*a + c)) + Power(ax,2)*(Power(b,4) - 6*a*Power(b,2)*c + 2*Power(b,3)*c - 4*b*Power(c,2)*(3*a + c) - 2*Power(c,2)*(-3*Power(a,2) + 6*a*c + Power(c,2))))*r3*r4 + (Power(bx,2)*(Power(b,3) - 3*a*b*c + Power(b,2)*c - 2*a*Power(c,2)) + Power(ax,2)*(b + 2*c)*(Power(b,2) + b*c + c*(-3*a + c)) - 2*ax*bx*(Power(b,3) + 2*Power(b,2)*c - 4*a*Power(c,2) + b*c*(-3*a + c)))/r5 + (-(Power(ax,2)*b*Power(c,3)) + Power(a,2)*(ax - bx)*c*(3*ax*b - 3*b*bx + 8*ax*c - 4*bx*c) - a*(b*Power(bx,2)*Power(b + c,2) - 2*ax*bx*(Power(b,3) + 3*Power(b,2)*c + 3*b*Power(c,2) + 2*Power(c,3)) + Power(ax,2)*(Power(b,3) + 4*Power(b,2)*c + 6*b*Power(c,2) + 8*Power(c,3))))/(a*r5) + (ax - bx)*(-(bx*(b + c)) + ax*(b + 2*c))*Log(a) - (ax - bx)*(-(bx*(b + c)) + ax*(b + 2*c))*Log(r6))/Power(c,3);
		B(4,0) = (2*(ax - bx)*(ay - by)*c + 2*(bx*(2*by*(Power(b,4) - 6*a*Power(b,2)*c + Power(b,3)*c - 6*a*b*Power(c,2) + 2*a*(3*a - c)*Power(c,2)) + ay*(-2*Power(b,4) + 12*a*Power(b,2)*c - 3*Power(b,3)*c + 12*a*Power(c,2)*(-a + c) + 2*b*Power(c,2)*(9*a + c))) + ax*(by*(-2*Power(b,4) + 12*a*Power(b,2)*c - 3*Power(b,3)*c + 12*a*Power(c,2)*(-a + c) + 2*b*Power(c,2)*(9*a + c)) + 2*ay*(Power(b,4) - 6*a*Power(b,2)*c + 2*Power(b,3)*c - 4*b*Power(c,2)*(3*a + c) - 2*Power(c,2)*(-3*Power(a,2) + 6*a*c + Power(c,2)))))*r2*r4 - 2*(bx*(2*by*(Power(b,4) - 6*a*Power(b,2)*c + Power(b,3)*c - 6*a*b*Power(c,2) + 2*a*(3*a - c)*Power(c,2)) + ay*(-2*Power(b,4) + 12*a*Power(b,2)*c - 3*Power(b,3)*c + 12*a*Power(c,2)*(-a + c) + 2*b*Power(c,2)*(9*a + c))) + ax*(by*(-2*Power(b,4) + 12*a*Power(b,2)*c - 3*Power(b,3)*c + 12*a*Power(c,2)*(-a + c) + 2*b*Power(c,2)*(9*a + c)) + 2*ay*(Power(b,4) - 6*a*Power(b,2)*c + 2*Power(b,3)*c - 4*b*Power(c,2)*(3*a + c) - 2*Power(c,2)*(-3*Power(a,2) + 6*a*c + Power(c,2)))))*r3*r4 - (2*(bx*(-(by*(Power(b,3) - 3*a*b*c + Power(b,2)*c - 2*a*Power(c,2))) + ay*(Power(b,3) + 2*Power(b,2)*c - 4*a*Power(c,2) + b*c*(-3*a + c))) + ax*(-(ay*(b + 2*c)*(Power(b,2) + b*c + c*(-3*a + c))) + by*(Power(b,3) + 2*Power(b,2)*c - 4*a*Power(c,2) + b*c*(-3*a + c)))))/r5 + (2*(-(ax*ay*b*Power(c,3)) + Power(a,2)*c*(ax*(3*ay*b - 3*b*by + 8*ay*c - 6*by*c) + bx*(-3*ay*b + 3*b*by - 6*ay*c + 4*by*c)) + a*(bx*(-(b*by*Power(b + c,2)) + ay*(Power(b,3) + 3*Power(b,2)*c + 3*b*Power(c,2) + 2*Power(c,3))) + ax*(by*(Power(b,3) + 3*Power(b,2)*c + 3*b*Power(c,2) + 2*Power(c,3)) - ay*(Power(b,3) + 4*Power(b,2)*c + 6*b*Power(c,2) + 8*Power(c,3))))))/(a*r5) - (bx*(-2*by*(b + c) + ay*(2*b + 3*c)) + ax*(-2*ay*(b + 2*c) + by*(2*b + 3*c)))*Log(a) + (bx*(-2*by*(b + c) + ay*(2*b + 3*c)) + ax*(-2*ay*(b + 2*c) + by*(2*b + 3*c)))*Log(r6))/(2.*Power(c,3));
		B(5,0) = (Power(ay - by,2)*c + 2*(Power(by,2)*(Power(b,4) - 6*a*Power(b,2)*c + Power(b,3)*c - 6*a*b*Power(c,2) + 2*a*(3*a - c)*Power(c,2)) + ay*by*(-2*Power(b,4) + 12*a*Power(b,2)*c - 3*Power(b,3)*c + 12*a*Power(c,2)*(-a + c) + 2*b*Power(c,2)*(9*a + c)) + Power(ay,2)*(Power(b,4) - 6*a*Power(b,2)*c + 2*Power(b,3)*c - 4*b*Power(c,2)*(3*a + c) - 2*Power(c,2)*(-3*Power(a,2) + 6*a*c + Power(c,2))))*r2*r4 - 2*(Power(by,2)*(Power(b,4) - 6*a*Power(b,2)*c + Power(b,3)*c - 6*a*b*Power(c,2) + 2*a*(3*a - c)*Power(c,2)) + ay*by*(-2*Power(b,4) + 12*a*Power(b,2)*c - 3*Power(b,3)*c + 12*a*Power(c,2)*(-a + c) + 2*b*Power(c,2)*(9*a + c)) + Power(ay,2)*(Power(b,4) - 6*a*Power(b,2)*c + 2*Power(b,3)*c - 4*b*Power(c,2)*(3*a + c) - 2*Power(c,2)*(-3*Power(a,2) + 6*a*c + Power(c,2))))*r3*r4 + (Power(by,2)*(Power(b,3) - 3*a*b*c + Power(b,2)*c - 2*a*Power(c,2)) + Power(ay,2)*(b + 2*c)*(Power(b,2) + b*c + c*(-3*a + c)) - 2*ay*by*(Power(b,3) + 2*Power(b,2)*c - 4*a*Power(c,2) + b*c*(-3*a + c)))/r5 + (-(Power(ay,2)*b*Power(c,3)) + Power(a,2)*(ay - by)*c*(3*ay*b - 3*b*by + 8*ay*c - 4*by*c) - a*(b*Power(by,2)*Power(b + c,2) - 2*ay*by*(Power(b,3) + 3*Power(b,2)*c + 3*b*Power(c,2) + 2*Power(c,3)) + Power(ay,2)*(Power(b,3) + 4*Power(b,2)*c + 6*b*Power(c,2) + 8*Power(c,3))))/(a*r5) + (ay - by)*(-(by*(b + c)) + ay*(b + 2*c))*Log(a) - (ay - by)*(-(by*(b + c)) + ay*(b + 2*c))*Log(r6))/Power(c,3);
		bary[i*3 + 0] = (k*B) * sqrt_c;
		
		B(0,0) = 2*(r1 + (2*a + b)*r2 - (2*a + b)*r3)*r4;
		B(1,0) = (r4*(-2*(bx*(-Power(b,3) + 6*a*b*c + 4*a*Power(c,2)) + ax*(Power(b,3) - 8*a*Power(c,2) - 2*b*c*(3*a + c)))*r2 + 2*(bx*(-Power(b,3) + 6*a*b*c + 4*a*Power(c,2)) + ax*(Power(b,3) - 8*a*Power(c,2) - 2*b*c*(3*a + c)))*r3 + r1*(2*c*(ax*b - b*bx + 2*ax*c) - (ax - bx)*r5*Log(a) + (ax - bx)*r5*Log(r6))))/(2.*Power(c,2));
		B(2,0) = (r4*(-2*(by*(-Power(b,3) + 6*a*b*c + 4*a*Power(c,2)) + ay*(Power(b,3) - 8*a*Power(c,2) - 2*b*c*(3*a + c)))*r2 + 2*(by*(-Power(b,3) + 6*a*b*c + 4*a*Power(c,2)) + ay*(Power(b,3) - 8*a*Power(c,2) - 2*b*c*(3*a + c)))*r3 + r1*(2*c*(ay*b - b*by + 2*ay*c) - (ay - by)*r5*Log(a) + (ay - by)*r5*Log(r6))))/(2.*Power(c,2));
		B(3,0) = (-2*Power(ax - bx,2)*c - 2*(Power(bx,2)*(2*Power(b,4) - 12*a*Power(b,2)*c + Power(b,3)*c + 12*Power(a,2)*Power(c,2) - 6*a*b*Power(c,2)) - 4*ax*bx*(Power(b,4) - 6*a*Power(b,2)*c + Power(b,3)*c - 6*a*b*Power(c,2) + 2*a*(3*a - c)*Power(c,2)) + Power(ax,2)*(2*Power(b,4) - 12*a*Power(b,2)*c + 3*Power(b,3)*c + 12*a*(a - c)*Power(c,2) - 2*b*Power(c,2)*(9*a + c)))*r2*r4 + 2*(Power(bx,2)*(2*Power(b,4) - 12*a*Power(b,2)*c + Power(b,3)*c + 12*Power(a,2)*Power(c,2) - 6*a*b*Power(c,2)) - 4*ax*bx*(Power(b,4) - 6*a*Power(b,2)*c + Power(b,3)*c - 6*a*b*Power(c,2) + 2*a*(3*a - c)*Power(c,2)) + Power(ax,2)*(2*Power(b,4) - 12*a*Power(b,2)*c + 3*Power(b,3)*c + 12*a*(a - c)*Power(c,2) - 2*b*Power(c,2)*(9*a + c)))*r3*r4 - (2*(b*Power(bx,2)*(Power(b,2) - 3*a*c) - 2*ax*bx*(Power(b,3) - 3*a*b*c + Power(b,2)*c - 2*a*Power(c,2)) + Power(ax,2)*(Power(b,3) + 2*Power(b,2)*c - 4*a*Power(c,2) + b*c*(-3*a + c))))/r5 + (2*(Power(bx,2)*(Power(b,3) - 3*a*b*c + Power(b,2)*c - 2*a*Power(c,2)) + Power(ax,2)*(b + 2*c)*(Power(b,2) + b*c + c*(-3*a + c)) - 2*ax*bx*(Power(b,3) + 2*Power(b,2)*c - 4*a*Power(c,2) + b*c*(-3*a + c))))/r5 - (ax - bx)*(-(bx*(2*b + c)) + ax*(2*b + 3*c))*Log(a) + (ax - bx)*(-(bx*(2*b + c)) + ax*(2*b + 3*c))*Log(r6))/(2.*Power(c,3));
		B(4,0) = (-2*(ax - bx)*(ay - by)*c - 2*(bx*(by*(2*Power(b,4) - 12*a*Power(b,2)*c + Power(b,3)*c + 12*Power(a,2)*Power(c,2) - 6*a*b*Power(c,2)) - 2*ay*(Power(b,4) - 6*a*Power(b,2)*c + Power(b,3)*c - 6*a*b*Power(c,2) + 2*a*(3*a - c)*Power(c,2))) + ax*(-2*by*(Power(b,4) - 6*a*Power(b,2)*c + Power(b,3)*c - 6*a*b*Power(c,2) + 2*a*(3*a - c)*Power(c,2)) + ay*(2*Power(b,4) - 12*a*Power(b,2)*c + 3*Power(b,3)*c + 12*a*(a - c)*Power(c,2) - 2*b*Power(c,2)*(9*a + c))))*r2*r4 + 2*(bx*(by*(2*Power(b,4) - 12*a*Power(b,2)*c + Power(b,3)*c + 12*Power(a,2)*Power(c,2) - 6*a*b*Power(c,2)) - 2*ay*(Power(b,4) - 6*a*Power(b,2)*c + Power(b,3)*c - 6*a*b*Power(c,2) + 2*a*(3*a - c)*Power(c,2))) + ax*(-2*by*(Power(b,4) - 6*a*Power(b,2)*c + Power(b,3)*c - 6*a*b*Power(c,2) + 2*a*(3*a - c)*Power(c,2)) + ay*(2*Power(b,4) - 12*a*Power(b,2)*c + 3*Power(b,3)*c + 12*a*(a - c)*Power(c,2) - 2*b*Power(c,2)*(9*a + c))))*r3*r4 - (2*(bx*(b*by*(Power(b,2) - 3*a*c) - ay*(Power(b,3) - 3*a*b*c + Power(b,2)*c - 2*a*Power(c,2))) + ax*(-(by*(Power(b,3) - 3*a*b*c + Power(b,2)*c - 2*a*Power(c,2))) + ay*(Power(b,3) + 2*Power(b,2)*c - 4*a*Power(c,2) + b*c*(-3*a + c)))))/r5 - (2*(bx*(-(by*(Power(b,3) - 3*a*b*c + Power(b,2)*c - 2*a*Power(c,2))) + ay*(Power(b,3) + 2*Power(b,2)*c - 4*a*Power(c,2) + b*c*(-3*a + c))) + ax*(-(ay*(b + 2*c)*(Power(b,2) + b*c + c*(-3*a + c))) + by*(Power(b,3) + 2*Power(b,2)*c - 4*a*Power(c,2) + b*c*(-3*a + c)))))/r5 + (bx*(2*ay*(b + c) - by*(2*b + c)) + ax*(2*by*(b + c) - ay*(2*b + 3*c)))*Log(a) - (bx*(2*ay*(b + c) - by*(2*b + c)) + ax*(2*by*(b + c) - ay*(2*b + 3*c)))*Log(r6))/(2.*Power(c,3));
		B(5,0) = (-2*Power(ay - by,2)*c - 2*(Power(by,2)*(2*Power(b,4) - 12*a*Power(b,2)*c + Power(b,3)*c + 12*Power(a,2)*Power(c,2) - 6*a*b*Power(c,2)) - 4*ay*by*(Power(b,4) - 6*a*Power(b,2)*c + Power(b,3)*c - 6*a*b*Power(c,2) + 2*a*(3*a - c)*Power(c,2)) + Power(ay,2)*(2*Power(b,4) - 12*a*Power(b,2)*c + 3*Power(b,3)*c + 12*a*(a - c)*Power(c,2) - 2*b*Power(c,2)*(9*a + c)))*r2*r4 + 2*(Power(by,2)*(2*Power(b,4) - 12*a*Power(b,2)*c + Power(b,3)*c + 12*Power(a,2)*Power(c,2) - 6*a*b*Power(c,2)) - 4*ay*by*(Power(b,4) - 6*a*Power(b,2)*c + Power(b,3)*c - 6*a*b*Power(c,2) + 2*a*(3*a - c)*Power(c,2)) + Power(ay,2)*(2*Power(b,4) - 12*a*Power(b,2)*c + 3*Power(b,3)*c + 12*a*(a - c)*Power(c,2) - 2*b*Power(c,2)*(9*a + c)))*r3*r4 - (2*(b*Power(by,2)*(Power(b,2) - 3*a*c) - 2*ay*by*(Power(b,3) - 3*a*b*c + Power(b,2)*c - 2*a*Power(c,2)) + Power(ay,2)*(Power(b,3) + 2*Power(b,2)*c - 4*a*Power(c,2) + b*c*(-3*a + c))))/r5 + (2*(Power(by,2)*(Power(b,3) - 3*a*b*c + Power(b,2)*c - 2*a*Power(c,2)) + Power(ay,2)*(b + 2*c)*(Power(b,2) + b*c + c*(-3*a + c)) - 2*ay*by*(Power(b,3) + 2*Power(b,2)*c - 4*a*Power(c,2) + b*c*(-3*a + c))))/r5 - (ay - by)*(-(by*(2*b + c)) + ay*(2*b + 3*c))*Log(a) + (ay - by)*(-(by*(2*b + c)) + ay*(2*b + 3*c))*Log(r6))/(2.*Power(c,3));
		bary[i*3 + 1] = 2*(k*B) * sqrt_c; // oops i forgot the factor of 2 in my derivation in mathematica
		
		B(0,0) = -((r4*((2*a + b)*r1 + 4*a*r2*r6 - 4*a*r3*r6))/r6);
		B(1,0) = (-2*(b*bx*(Power(b,2) - 6*a*c) + ax*(-Power(b,3) + 6*a*b*c + 4*a*Power(c,2)))*r2*r4 + 2*(b*bx*(Power(b,2) - 6*a*c) + ax*(-Power(b,3) + 6*a*b*c + 4*a*Power(c,2)))*r3*r4 - (2*(-(bx*(Power(b,2) - 2*a*c)) + ax*(Power(b,2) - 2*a*c + b*c)))/r5 + (2*(2*Power(a,2)*(-ax + bx)*c + Power(b,2)*(-(b*bx) + ax*(b + c)) + a*(-(b*bx*(b - 3*c)) + ax*(Power(b,2) - 2*b*c - 2*Power(c,2)))))/(r5*r6) + (ax - bx)*Log(a) + (-ax + bx)*Log(r6))/(2.*Power(c,2));
		B(2,0) = (-2*(b*by*(Power(b,2) - 6*a*c) + ay*(-Power(b,3) + 6*a*b*c + 4*a*Power(c,2)))*r2*r4 + 2*(b*by*(Power(b,2) - 6*a*c) + ay*(-Power(b,3) + 6*a*b*c + 4*a*Power(c,2)))*r3*r4 - (2*(-(by*(Power(b,2) - 2*a*c)) + ay*(Power(b,2) - 2*a*c + b*c)))/r5 + (2*(2*Power(a,2)*(-ay + by)*c + Power(b,2)*(-(b*by) + ay*(b + c)) + a*(-(b*by*(b - 3*c)) + ay*(Power(b,2) - 2*b*c - 2*Power(c,2)))))/(r5*r6) + (ay - by)*Log(a) + (-ay + by)*Log(r6))/(2.*Power(c,2));
		B(3,0) = (Power(ax - bx,2)*c + (Power(a,2)*(ax - bx)*c*(3*ax*b - 3*b*bx + 2*ax*c + 2*bx*c) - Power(b,2)*Power(b*bx - ax*(b + c),2) - a*(-(b*bx) + ax*(b + c))*(-(b*bx*(b - 4*c)) + ax*(Power(b,2) - 3*b*c - 2*Power(c,2))))/(-4*Power(a,2)*c + Power(b,2)*(b + c) + a*(Power(b,2) - 4*b*c - 4*Power(c,2))) + 2*(Power(bx,2)*(Power(b,4) - 6*a*Power(b,2)*c + 6*Power(a,2)*Power(c,2)) - ax*bx*(2*Power(b,4) - 12*a*Power(b,2)*c + Power(b,3)*c + 12*Power(a,2)*Power(c,2) - 6*a*b*Power(c,2)) + Power(ax,2)*(Power(b,4) - 6*a*Power(b,2)*c + Power(b,3)*c - 6*a*b*Power(c,2) + 2*a*(3*a - c)*Power(c,2)))*r2*r4 - 2*(Power(bx,2)*(Power(b,4) - 6*a*Power(b,2)*c + 6*Power(a,2)*Power(c,2)) - ax*bx*(2*Power(b,4) - 12*a*Power(b,2)*c + Power(b,3)*c + 12*Power(a,2)*Power(c,2) - 6*a*b*Power(c,2)) + Power(ax,2)*(Power(b,4) - 6*a*Power(b,2)*c + Power(b,3)*c - 6*a*b*Power(c,2) + 2*a*(3*a - c)*Power(c,2)))*r3*r4 + (a*(ax - bx)*c*(3*ax*b - 3*b*bx + 4*ax*c) - b*Power(b*bx - ax*(b + c),2))/r5 + (ax - bx)*(-(b*bx) + ax*(b + c))*Log(a) - (ax - bx)*(-(b*bx) + ax*(b + c))*Log(r6))/Power(c,3);
		B(4,0) = (2*(ax - bx)*(ay - by)*c + (2*(-(Power(b,2)*(-(b*bx) + ax*(b + c))*(-(b*by) + ay*(b + c))) + Power(a,2)*c*(ax*(3*ay*b - 3*b*by + 2*ay*c) + bx*(-3*ay*b + 3*b*by - 2*by*c)) + a*(b*bx*(-(b*by*(b - 4*c)) + ay*(Power(b,2) - 3*b*c - 3*Power(c,2))) + ax*(b*by*(Power(b,2) - 3*b*c - 3*Power(c,2)) - ay*(b + c)*(Power(b,2) - 3*b*c - 2*Power(c,2))))))/(-4*Power(a,2)*c + Power(b,2)*(b + c) + a*(Power(b,2) - 4*b*c - 4*Power(c,2))) + 2*(bx*(2*by*(Power(b,4) - 6*a*Power(b,2)*c + 6*Power(a,2)*Power(c,2)) - ay*(2*Power(b,4) - 12*a*Power(b,2)*c + Power(b,3)*c + 12*Power(a,2)*Power(c,2) - 6*a*b*Power(c,2))) + ax*(-(by*(2*Power(b,4) - 12*a*Power(b,2)*c + Power(b,3)*c + 12*Power(a,2)*Power(c,2) - 6*a*b*Power(c,2))) + 2*ay*(Power(b,4) - 6*a*Power(b,2)*c + Power(b,3)*c - 6*a*b*Power(c,2) + 2*a*(3*a - c)*Power(c,2))))*r2*r4 - 2*(bx*(2*by*(Power(b,4) - 6*a*Power(b,2)*c + 6*Power(a,2)*Power(c,2)) - ay*(2*Power(b,4) - 12*a*Power(b,2)*c + Power(b,3)*c + 12*Power(a,2)*Power(c,2) - 6*a*b*Power(c,2))) + ax*(-(by*(2*Power(b,4) - 12*a*Power(b,2)*c + Power(b,3)*c + 12*Power(a,2)*Power(c,2) - 6*a*b*Power(c,2))) + 2*ay*(Power(b,4) - 6*a*Power(b,2)*c + Power(b,3)*c - 6*a*b*Power(c,2) + 2*a*(3*a - c)*Power(c,2))))*r3*r4 - (2*(bx*(b*by*(Power(b,2) - 3*a*c) - ay*(Power(b,3) - 3*a*b*c + Power(b,2)*c - 2*a*Power(c,2))) + ax*(-(by*(Power(b,3) - 3*a*b*c + Power(b,2)*c - 2*a*Power(c,2))) + ay*(Power(b,3) + 2*Power(b,2)*c - 4*a*Power(c,2) + b*c*(-3*a + c)))))/r5 - (bx*(-2*b*by + ay*(2*b + c)) + ax*(-2*ay*(b + c) + by*(2*b + c)))*Log(a) + (bx*(-2*b*by + ay*(2*b + c)) + ax*(-2*ay*(b + c) + by*(2*b + c)))*Log(r6))/(2.*Power(c,3));
		B(5,0) = (Power(ay - by,2)*c + (Power(a,2)*(ay - by)*c*(3*ay*b - 3*b*by + 2*ay*c + 2*by*c) - Power(b,2)*Power(b*by - ay*(b + c),2) - a*(-(b*by) + ay*(b + c))*(-(b*by*(b - 4*c)) + ay*(Power(b,2) - 3*b*c - 2*Power(c,2))))/(-4*Power(a,2)*c + Power(b,2)*(b + c) + a*(Power(b,2) - 4*b*c - 4*Power(c,2))) + 2*(Power(by,2)*(Power(b,4) - 6*a*Power(b,2)*c + 6*Power(a,2)*Power(c,2)) - ay*by*(2*Power(b,4) - 12*a*Power(b,2)*c + Power(b,3)*c + 12*Power(a,2)*Power(c,2) - 6*a*b*Power(c,2)) + Power(ay,2)*(Power(b,4) - 6*a*Power(b,2)*c + Power(b,3)*c - 6*a*b*Power(c,2) + 2*a*(3*a - c)*Power(c,2)))*r2*r4 - 2*(Power(by,2)*(Power(b,4) - 6*a*Power(b,2)*c + 6*Power(a,2)*Power(c,2)) - ay*by*(2*Power(b,4) - 12*a*Power(b,2)*c + Power(b,3)*c + 12*Power(a,2)*Power(c,2) - 6*a*b*Power(c,2)) + Power(ay,2)*(Power(b,4) - 6*a*Power(b,2)*c + Power(b,3)*c - 6*a*b*Power(c,2) + 2*a*(3*a - c)*Power(c,2)))*r3*r4 + (a*(ay - by)*c*(3*ay*b - 3*b*by + 4*ay*c) - b*Power(b*by - ay*(b + c),2))/r5 + (ay - by)*(-(b*by) + ay*(b + c))*Log(a) - (ay - by)*(-(b*by) + ay*(b + c))*Log(r6))/Power(c,3);
		bary[i*3 + 2] = (k*B) * sqrt_c;
#else
		B(0,0) = -(((b*r1 + 2*a*(b + 2*c)*r2 - 2*a*(b + 2*c)*r3)*r4)/a);
		B(1,0) = -((((ax*b + 2*a*(ax - bx))*r1 + 2*a*(2*a*(ax - bx) - b*bx + 2*ax*(b + c))*r2 - 2*a*(2*a*(ax - bx) - b*bx + 2*ax*(b + c))*r3)*r4)/a);
		B(2,0) = -((((ay*b + 2*a*(ay - by))*r1 + 2*a*(2*a*(ay - by) - b*by + 2*ay*(b + c))*r2 - 2*a*(2*a*(ay - by) - b*by + 2*ay*(b + c))*r3)*r4)/a);
		B(3,0) = (2*(Power(bx,2)*(Power(b,3) - 6*a*b*c - 4*a*Power(c,2)) + Power(ax,2)*(b + 2*c)*(Power(b,2) - 2*b*c - 2*c*(3*a + c)) + 2*ax*bx*(-Power(b,3) + 8*a*Power(c,2) + 2*b*c*(3*a + c)))*r2*r4 - 2*(Power(bx,2)*(Power(b,3) - 6*a*b*c - 4*a*Power(c,2)) + Power(ax,2)*(b + 2*c)*(Power(b,2) - 2*b*c - 2*c*(3*a + c)) + 2*ax*bx*(-Power(b,3) + 8*a*Power(c,2) + 2*b*c*(3*a + c)))*r3*r4 + (2*(Power(bx,2)*(Power(b,2) - 2*a*c) - 2*ax*bx*(Power(b,2) - 2*a*c + b*c) + Power(ax,2)*(Power(b,2) + 2*b*c + 2*c*(-a + c))))/r5 - (2*(-2*Power(a,2)*Power(ax - bx,2)*c + Power(ax,2)*b*Power(c,2) + a*(b*Power(bx,2)*(b + c) - 2*ax*bx*(Power(b,2) + 2*b*c + 2*Power(c,2)) + Power(ax,2)*(Power(b,2) + 3*b*c + 6*Power(c,2)))))/(a*r5) + Power(ax - bx,2)*Log(a) - Power(ax - bx,2)*Log(r6))/(2.*Power(c,2));
		B(4,0) = (2*(bx*(by*(Power(b,3) - 6*a*b*c - 4*a*Power(c,2)) + ay*(-Power(b,3) + 8*a*Power(c,2) + 2*b*c*(3*a + c))) + ax*(ay*(b + 2*c)*(Power(b,2) - 2*b*c - 2*c*(3*a + c)) + by*(-Power(b,3) + 8*a*Power(c,2) + 2*b*c*(3*a + c))))*r2*r4 - 2*(bx*(by*(Power(b,3) - 6*a*b*c - 4*a*Power(c,2)) + ay*(-Power(b,3) + 8*a*Power(c,2) + 2*b*c*(3*a + c))) + ax*(ay*(b + 2*c)*(Power(b,2) - 2*b*c - 2*c*(3*a + c)) + by*(-Power(b,3) + 8*a*Power(c,2) + 2*b*c*(3*a + c))))*r3*r4 - (2*(bx*(-(by*(Power(b,2) - 2*a*c)) + ay*(Power(b,2) - 2*a*c + b*c)) + ax*(by*(Power(b,2) - 2*a*c + b*c) - ay*(Power(b,2) + 2*b*c + 2*c*(-a + c)))))/r5 - (2*(-2*Power(a,2)*(ax - bx)*(ay - by)*c + ax*ay*b*Power(c,2) + a*(bx*(b*by*(b + c) - ay*(Power(b,2) + 2*b*c + 2*Power(c,2))) + ax*(-(by*(Power(b,2) + 2*b*c + 2*Power(c,2))) + ay*(Power(b,2) + 3*b*c + 6*Power(c,2))))))/(a*r5) + (ax - bx)*(ay - by)*Log(a) - (ax - bx)*(ay - by)*Log(r6))/(2.*Power(c,2));
		B(5,0) = (2*(Power(by,2)*(Power(b,3) - 6*a*b*c - 4*a*Power(c,2)) + Power(ay,2)*(b + 2*c)*(Power(b,2) - 2*b*c - 2*c*(3*a + c)) + 2*ay*by*(-Power(b,3) + 8*a*Power(c,2) + 2*b*c*(3*a + c)))*r2*r4 - 2*(Power(by,2)*(Power(b,3) - 6*a*b*c - 4*a*Power(c,2)) + Power(ay,2)*(b + 2*c)*(Power(b,2) - 2*b*c - 2*c*(3*a + c)) + 2*ay*by*(-Power(b,3) + 8*a*Power(c,2) + 2*b*c*(3*a + c)))*r3*r4 + (2*(Power(by,2)*(Power(b,2) - 2*a*c) - 2*ay*by*(Power(b,2) - 2*a*c + b*c) + Power(ay,2)*(Power(b,2) + 2*b*c + 2*c*(-a + c))))/r5 - (2*(-2*Power(a,2)*Power(ay - by,2)*c + Power(ay,2)*b*Power(c,2) + a*(b*Power(by,2)*(b + c) - 2*ay*by*(Power(b,2) + 2*b*c + 2*Power(c,2)) + Power(ay,2)*(Power(b,2) + 3*b*c + 6*Power(c,2)))))/(a*r5) + Power(ay - by,2)*Log(a) - Power(ay - by,2)*Log(r6))/(2.*Power(c,2));
		bary[p[i][0]] += (k*B) * sqrt_c;

		B(0,0) = -((r4*(-((b + 2*c)*r1) - 2*b*r2*r6 + 2*b*r3*r6))/r6);
		B(1,0) = -(((b*bx - ax*(b + 2*c))/c - (2*(ax*b + 2*a*(ax - bx))*r2)/r1 + (2*(ax*b + 2*a*(ax - bx))*r3)/r1 + (a*(ax*b - b*bx + 2*bx*c) + b*(-(b*bx) + ax*(b + c)))/(c*r6))/r5);
		B(2,0) = -(((b*by - ay*(b + 2*c))/c - (2*(ay*b + 2*a*(ay - by))*r2)/r1 + (2*(ay*b + 2*a*(ay - by))*r3)/r1 + (a*(ay*b - b*by + 2*by*c) + b*(-(b*by) + ay*(b + c)))/(c*r6))/r5);
		B(3,0) = ((2*(-2*Power(a,2)*Power(ax - bx,2)*c + b*Power(b*bx - ax*(b + c),2) + a*(b*Power(bx,2)*(b - 3*c) - 2*ax*bx*(Power(b,2) - 2*b*c - 2*Power(c,2)) + Power(ax,2)*(Power(b,2) - b*c - 2*Power(c,2)))))/(-4*Power(a,2)*c + Power(b,2)*(b + c) + a*(Power(b,2) - 4*b*c - 4*Power(c,2))) - 2*(b*Power(bx,2)*(Power(b,2) - 6*a*c) - 2*ax*bx*(Power(b,3) - 6*a*b*c - 4*a*Power(c,2)) + Power(ax,2)*(Power(b,3) - 8*a*Power(c,2) - 2*b*c*(3*a + c)))*r2*r4 + 2*(b*Power(bx,2)*(Power(b,2) - 6*a*c) - 2*ax*bx*(Power(b,3) - 6*a*b*c - 4*a*Power(c,2)) + Power(ax,2)*(Power(b,3) - 8*a*Power(c,2) - 2*b*c*(3*a + c)))*r3*r4 + (2*(Power(bx,2)*(Power(b,2) - 2*a*c) - 2*ax*bx*(Power(b,2) - 2*a*c + b*c) + Power(ax,2)*(Power(b,2) + 2*b*c + 2*c*(-a + c))))/r5 - Power(ax - bx,2)*Log(a) + Power(ax - bx,2)*Log(r6))/(2.*Power(c,2));
		B(4,0) = ((2*(2*Power(a,2)*(ax - bx)*(ay - by)*c - b*(-(b*bx) + ax*(b + c))*(-(b*by) + ay*(b + c)) + a*(bx*(-(b*by*(b - 3*c)) + ay*(Power(b,2) - 2*b*c - 2*Power(c,2))) + ax*(by*(Power(b,2) - 2*b*c - 2*Power(c,2)) + ay*(-Power(b,2) + b*c + 2*Power(c,2))))))/(4*Power(a,2)*c - Power(b,2)*(b + c) + a*(-Power(b,2) + 4*b*c + 4*Power(c,2))) - 2*(bx*(b*by*(Power(b,2) - 6*a*c) + ay*(-Power(b,3) + 6*a*b*c + 4*a*Power(c,2))) + ax*(by*(-Power(b,3) + 6*a*b*c + 4*a*Power(c,2)) + ay*(Power(b,3) - 8*a*Power(c,2) - 2*b*c*(3*a + c))))*r2*r4 + 2*(bx*(b*by*(Power(b,2) - 6*a*c) + ay*(-Power(b,3) + 6*a*b*c + 4*a*Power(c,2))) + ax*(by*(-Power(b,3) + 6*a*b*c + 4*a*Power(c,2)) + ay*(Power(b,3) - 8*a*Power(c,2) - 2*b*c*(3*a + c))))*r3*r4 - (2*(bx*(-(by*(Power(b,2) - 2*a*c)) + ay*(Power(b,2) - 2*a*c + b*c)) + ax*(by*(Power(b,2) - 2*a*c + b*c) - ay*(Power(b,2) + 2*b*c + 2*c*(-a + c)))))/r5 - (ax - bx)*(ay - by)*Log(a) + (ax - bx)*(ay - by)*Log(r6))/(2.*Power(c,2));
		B(5,0) = ((2*(-2*Power(a,2)*Power(ay - by,2)*c + b*Power(b*by - ay*(b + c),2) + a*(b*Power(by,2)*(b - 3*c) - 2*ay*by*(Power(b,2) - 2*b*c - 2*Power(c,2)) + Power(ay,2)*(Power(b,2) - b*c - 2*Power(c,2)))))/(-4*Power(a,2)*c + Power(b,2)*(b + c) + a*(Power(b,2) - 4*b*c - 4*Power(c,2))) - 2*(b*Power(by,2)*(Power(b,2) - 6*a*c) - 2*ay*by*(Power(b,3) - 6*a*b*c - 4*a*Power(c,2)) + Power(ay,2)*(Power(b,3) - 8*a*Power(c,2) - 2*b*c*(3*a + c)))*r2*r4 + 2*(b*Power(by,2)*(Power(b,2) - 6*a*c) - 2*ay*by*(Power(b,3) - 6*a*b*c - 4*a*Power(c,2)) + Power(ay,2)*(Power(b,3) - 8*a*Power(c,2) - 2*b*c*(3*a + c)))*r3*r4 + (2*(Power(by,2)*(Power(b,2) - 2*a*c) - 2*ay*by*(Power(b,2) - 2*a*c + b*c) + Power(ay,2)*(Power(b,2) + 2*b*c + 2*c*(-a + c))))/r5 - Power(ay - by,2)*Log(a) + Power(ay - by,2)*Log(r6))/(2.*Power(c,2));
		bary[p[i][1]] += (k*B) * sqrt_c;
#endif
		// calc derivative coordinates
		if (g.deriv == 1)
		{
		B(0,0) = 0;
		B(1,0) = -(((ay - by)*r4*((2*a + b)*r1 + 4*a*r2*r6 - 4*a*r3*r6))/a);
		B(2,0) = ((ax - bx)*r4*((2*a + b)*r1 + 4*a*r2*r6 - 4*a*r3*r6))/a;
		B(3,0) = ((ay - by)*((2*(2*Power(a,2)*(ax - bx)*c - ax*b*Power(c,2) + a*(bx*(Power(b,2) + 2*b*c + 2*Power(c,2)) - ax*(Power(b,2) + 3*b*c + 6*Power(c,2)))))/(-(a*Power(b,2)) + 4*Power(a,2)*c) + 2*(ax*(b + 2*c)*(Power(b,2) - 2*b*c - 2*c*(3*a + c)) + bx*(-Power(b,3) + 8*a*Power(c,2) + 2*b*c*(3*a + c)))*r2*r4 - 2*(ax*(b + 2*c)*(Power(b,2) - 2*b*c - 2*c*(3*a + c)) + bx*(-Power(b,3) + 8*a*Power(c,2) + 2*b*c*(3*a + c)))*r3*r4 - (2*(bx*(Power(b,2) - 2*a*c + b*c) - ax*(Power(b,2) + 2*b*c + 2*c*(-a + c))))/r5 + (ax - bx)*Log(a) - (ax - bx)*Log(r6)))/Power(c,2);
		B(4,0) = (-2*(Power(ax,2)*(b + 2*c)*(Power(b,2) - 2*b*c - 2*c*(3*a + c)) - Power(ay,2)*(b + 2*c)*(Power(b,2) - 2*b*c - 2*c*(3*a + c)) + (Power(bx,2) - Power(by,2))*(Power(b,3) - 8*a*Power(c,2) - 2*b*c*(3*a + c)) + 2*ay*by*(Power(b,3) - 2*Power(c,2)*(5*a + c) - 2*b*c*(3*a + 2*c)) + 2*ax*bx*(-Power(b,3) + 2*Power(c,2)*(5*a + c) + 2*b*c*(3*a + 2*c)))*r2*r4 + 2*(Power(ax,2)*(b + 2*c)*(Power(b,2) - 2*b*c - 2*c*(3*a + c)) - Power(ay,2)*(b + 2*c)*(Power(b,2) - 2*b*c - 2*c*(3*a + c)) + (Power(bx,2) - Power(by,2))*(Power(b,3) - 8*a*Power(c,2) - 2*b*c*(3*a + c)) + 2*ay*by*(Power(b,3) - 2*Power(c,2)*(5*a + c) - 2*b*c*(3*a + 2*c)) + 2*ax*bx*(-Power(b,3) + 2*Power(c,2)*(5*a + c) + 2*b*c*(3*a + 2*c)))*r3*r4 - (2*((Power(bx,2) - Power(by,2))*(Power(b,2) - 2*a*c + b*c) - ax*bx*(2*Power(b,2) + 3*b*c + 2*c*(-2*a + c)) + ay*by*(2*Power(b,2) + 3*b*c + 2*c*(-2*a + c)) + Power(ax,2)*(Power(b,2) + 2*b*c + 2*c*(-a + c)) - Power(ay,2)*(Power(b,2) + 2*b*c + 2*c*(-a + c))))/r5 - (2*(2*Power(a,2)*(Power(ax,2) - Power(ay,2) - 2*ax*bx + Power(bx,2) + 2*ay*by - Power(by,2))*c + b*(-Power(ax,2) + ax*bx + ay*(ay - by))*Power(c,2) + a*(-((Power(bx,2) - Power(by,2))*(Power(b,2) + 2*b*c + 2*Power(c,2))) - Power(ax,2)*(Power(b,2) + 3*b*c + 6*Power(c,2)) + Power(ay,2)*(Power(b,2) + 3*b*c + 6*Power(c,2)) + ax*bx*(2*Power(b,2) + 5*b*c + 8*Power(c,2)) - ay*by*(2*Power(b,2) + 5*b*c + 8*Power(c,2)))))/(a*r5) - (Power(ax,2) - Power(ay,2) - 2*ax*bx + Power(bx,2) + 2*ay*by - Power(by,2))*Log(a) + (Power(ax,2) - Power(ay,2) - 2*ax*bx + Power(bx,2) + 2*ay*by - Power(by,2))*Log(r6))/(2.*Power(c,2));
		B(5,0) = ((ax - bx)*(-2*(ay*(b + 2*c)*(Power(b,2) - 2*b*c - 2*c*(3*a + c)) + by*(-Power(b,3) + 8*a*Power(c,2) + 2*b*c*(3*a + c)))*r2*r4 + 2*(ay*(b + 2*c)*(Power(b,2) - 2*b*c - 2*c*(3*a + c)) + by*(-Power(b,3) + 8*a*Power(c,2) + 2*b*c*(3*a + c)))*r3*r4 - (-2*by*(Power(b,2) - 2*a*c + b*c) + 2*ay*(Power(b,2) + 2*b*c + 2*c*(-a + c)))/r5 + (2*(2*Power(a,2)*(-ay + by)*c + ay*b*Power(c,2) + a*(-(by*(Power(b,2) + 2*b*c + 2*Power(c,2))) + ay*(Power(b,2) + 3*b*c + 6*Power(c,2)))))/(a*r5) - (ay - by)*Log(a) + (ay - by)*Log(r6)))/Power(c,2);
		bary2[i*3 + 0] += k*B;
		
		B(0,0) = 0;
		B(1,0) = 4*(ay - by)*(r1 + (2*a + b)*r2 - (2*a + b)*r3)*r4;
		B(2,0) = -4*(ax - bx)*(r1 + (2*a + b)*r2 - (2*a + b)*r3)*r4;
		B(3,0) = (2*(ay - by)*r4*(-2*(bx*(-Power(b,3) + 6*a*b*c + 4*a*Power(c,2)) + ax*(Power(b,3) - 8*a*Power(c,2) - 2*b*c*(3*a + c)))*r2 + 2*(bx*(-Power(b,3) + 6*a*b*c + 4*a*Power(c,2)) + ax*(Power(b,3) - 8*a*Power(c,2) - 2*b*c*(3*a + c)))*r3 + r1*(2*c*(ax*b - b*bx + 2*ax*c) - (ax - bx)*r5*Log(a) + (ax - bx)*r5*Log(r6))))/Power(c,2);
		B(4,0) = (2*(-2*ax*bx*(b + c)*(Power(b,2) - 6*a*c - b*c) + 2*ay*by*(b + c)*(Power(b,2) - 6*a*c - b*c) + (Power(bx,2) - Power(by,2))*(Power(b,3) - 6*a*b*c - 4*a*Power(c,2)) + Power(ax,2)*(Power(b,3) - 8*a*Power(c,2) - 2*b*c*(3*a + c)) + Power(ay,2)*(-Power(b,3) + 8*a*Power(c,2) + 2*b*c*(3*a + c)))*r2*r4 - 2*(-2*ax*bx*(b + c)*(Power(b,2) - 6*a*c - b*c) + 2*ay*by*(b + c)*(Power(b,2) - 6*a*c - b*c) + (Power(bx,2) - Power(by,2))*(Power(b,3) - 6*a*b*c - 4*a*Power(c,2)) + Power(ax,2)*(Power(b,3) - 8*a*Power(c,2) - 2*b*c*(3*a + c)) + Power(ay,2)*(-Power(b,3) + 8*a*Power(c,2) + 2*b*c*(3*a + c)))*r3*r4 - (2*(-((Power(bx,2) - Power(by,2))*(Power(b,2) - 2*a*c)) + ax*bx*(2*Power(b,2) - 4*a*c + b*c) - ay*by*(2*Power(b,2) - 4*a*c + b*c) - Power(ax,2)*(Power(b,2) - 2*a*c + b*c) + Power(ay,2)*(Power(b,2) - 2*a*c + b*c)))/r5 - (2*((Power(bx,2) - Power(by,2))*(Power(b,2) - 2*a*c + b*c) - ax*bx*(2*Power(b,2) + 3*b*c + 2*c*(-2*a + c)) + ay*by*(2*Power(b,2) + 3*b*c + 2*c*(-2*a + c)) + Power(ax,2)*(Power(b,2) + 2*b*c + 2*c*(-a + c)) - Power(ay,2)*(Power(b,2) + 2*b*c + 2*c*(-a + c))))/r5 + (Power(ax,2) - Power(ay,2) - 2*ax*bx + Power(bx,2) + 2*ay*by - Power(by,2))*Log(a) + (-Power(ax,2) + Power(ay,2) + 2*ax*bx - Power(bx,2) - 2*ay*by + Power(by,2))*Log(r6))/Power(c,2);
		B(5,0) = (-2*(ax - bx)*r4*(-2*(by*(-Power(b,3) + 6*a*b*c + 4*a*Power(c,2)) + ay*(Power(b,3) - 8*a*Power(c,2) - 2*b*c*(3*a + c)))*r2 + 2*(by*(-Power(b,3) + 6*a*b*c + 4*a*Power(c,2)) + ay*(Power(b,3) - 8*a*Power(c,2) - 2*b*c*(3*a + c)))*r3 + r1*(2*c*(ay*b - b*by + 2*ay*c) - (ay - by)*r5*Log(a) + (ay - by)*r5*Log(r6))))/Power(c,2);
		bary2[i*3 + 1] += k*B;
		
		B(0,0) = 0;
		B(1,0) = -(((ay - by)*r4*((2*a + b)*r1 + 4*a*r2*r6 - 4*a*r3*r6))/r6);
		B(2,0) = ((ax - bx)*r4*((2*a + b)*r1 + 4*a*r2*r6 - 4*a*r3*r6))/r6;
		B(3,0) = ((ay - by)*(-2*(b*bx*(Power(b,2) - 6*a*c) + ax*(-Power(b,3) + 6*a*b*c + 4*a*Power(c,2)))*r2*r4 + 2*(b*bx*(Power(b,2) - 6*a*c) + ax*(-Power(b,3) + 6*a*b*c + 4*a*Power(c,2)))*r3*r4 - (2*(-(bx*(Power(b,2) - 2*a*c)) + ax*(Power(b,2) - 2*a*c + b*c)))/r5 + (2*(2*Power(a,2)*(-ax + bx)*c + Power(b,2)*(-(b*bx) + ax*(b + c)) + a*(-(b*bx*(b - 3*c)) + ax*(Power(b,2) - 2*b*c - 2*Power(c,2)))))/(r5*r6) + (ax - bx)*Log(a) + (-ax + bx)*Log(r6)))/Power(c,2);
		B(4,0) = ((2*(-2*Power(a,2)*(Power(ax,2) - Power(ay,2) - 2*ax*bx + Power(bx,2) + 2*ay*by - Power(by,2))*c + Power(b,2)*(b*(Power(bx,2) - Power(by,2)) + Power(ax,2)*(b + c) - Power(ay,2)*(b + c) - ax*bx*(2*b + c) + ay*by*(2*b + c)) + a*(b*(Power(bx,2) - Power(by,2))*(b - 3*c) + ay*by*(2*Power(b,2) - 5*b*c - 2*Power(c,2)) + Power(ax,2)*(Power(b,2) - 2*b*c - 2*Power(c,2)) + Power(ay,2)*(-Power(b,2) + 2*b*c + 2*Power(c,2)) + ax*bx*(-2*Power(b,2) + 5*b*c + 2*Power(c,2)))))/(-4*Power(a,2)*c + Power(b,2)*(b + c) + a*(Power(b,2) - 4*b*c - 4*Power(c,2))) - 2*(b*(Power(bx,2) - Power(by,2))*(Power(b,2) - 6*a*c) + Power(ax,2)*(Power(b,3) - 6*a*b*c - 4*a*Power(c,2)) - 2*ax*bx*(Power(b,3) - 6*a*b*c - 2*a*Power(c,2)) + 2*ay*by*(Power(b,3) - 6*a*b*c - 2*a*Power(c,2)) + Power(ay,2)*(-Power(b,3) + 6*a*b*c + 4*a*Power(c,2)))*r2*r4 + 2*(b*(Power(bx,2) - Power(by,2))*(Power(b,2) - 6*a*c) + Power(ax,2)*(Power(b,3) - 6*a*b*c - 4*a*Power(c,2)) - 2*ax*bx*(Power(b,3) - 6*a*b*c - 2*a*Power(c,2)) + 2*ay*by*(Power(b,3) - 6*a*b*c - 2*a*Power(c,2)) + Power(ay,2)*(-Power(b,3) + 6*a*b*c + 4*a*Power(c,2)))*r3*r4 - (2*(-((Power(bx,2) - Power(by,2))*(Power(b,2) - 2*a*c)) + ax*bx*(2*Power(b,2) - 4*a*c + b*c) - ay*by*(2*Power(b,2) - 4*a*c + b*c) - Power(ax,2)*(Power(b,2) - 2*a*c + b*c) + Power(ay,2)*(Power(b,2) - 2*a*c + b*c)))/r5 - (Power(ax,2) - Power(ay,2) - 2*ax*bx + Power(bx,2) + 2*ay*by - Power(by,2))*Log(a) + (Power(ax,2) - Power(ay,2) - 2*ax*bx + Power(bx,2) + 2*ay*by - Power(by,2))*Log(r6))/(2.*Power(c,2));
		B(5,0) = ((ax - bx)*((2*(2*Power(a,2)*(-ay + by)*c + Power(b,2)*(-(b*by) + ay*(b + c)) + a*(-(b*by*(b - 3*c)) + ay*(Power(b,2) - 2*b*c - 2*Power(c,2)))))/(-4*Power(a,2)*c + Power(b,2)*(b + c) + a*(Power(b,2) - 4*b*c - 4*Power(c,2))) + 2*(b*by*(Power(b,2) - 6*a*c) + ay*(-Power(b,3) + 6*a*b*c + 4*a*Power(c,2)))*r2*r4 - 2*(b*by*(Power(b,2) - 6*a*c) + ay*(-Power(b,3) + 6*a*b*c + 4*a*Power(c,2)))*r3*r4 + (2*(-(by*(Power(b,2) - 2*a*c)) + ay*(Power(b,2) - 2*a*c + b*c)))/r5 - (ay - by)*Log(a) + (ay - by)*Log(r6)))/Power(c,2);
		bary2[i*3 + 2] += k*B;
		}

		// calc second derivative coordinates
		if (g.deriv == 2)
		{
		B(0,0) = 0;
		B(1,0) = 0;
		B(2,0) = 0;
		B(3,0) = (-2*Power(ay - by,2)*r4*((2*a + b)*r1 + 4*a*r2*r6 - 4*a*r3*r6))/a;
		B(4,0) = (2*(ax - bx)*(ay - by)*r4*((2*a + b)*r1 + 4*a*r2*r6 - 4*a*r3*r6))/a;
		B(5,0) = (-2*Power(ax - bx,2)*r4*((2*a + b)*r1 + 4*a*r2*r6 - 4*a*r3*r6))/a;
		bary2[i*3 + 0] += (k*B) / sqrt_c;
		
		B(0,0) = 0;
		B(1,0) = 0;
		B(2,0) = 0;
		B(3,0) = 8*Power(ay - by,2)*(r1 + (2*a + b)*r2 - (2*a + b)*r3)*r4;
		B(4,0) = -8*(ax - bx)*(ay - by)*(r1 + (2*a + b)*r2 - (2*a + b)*r3)*r4;
		B(5,0) = 8*Power(ax - bx,2)*(r1 + (2*a + b)*r2 - (2*a + b)*r3)*r4;
		bary2[i*3 + 1] += (k*B) / sqrt_c;
		
		B(0,0) = 0;
		B(1,0) = 0;
		B(2,0) = 0;
		B(3,0) = (-2*Power(ay - by,2)*r4*((2*a + b)*r1 + 4*a*r2*r6 - 4*a*r3*r6))/r6;
		B(4,0) = (2*(ax - bx)*(ay - by)*r4*((2*a + b)*r1 + 4*a*r2*r6 - 4*a*r3*r6))/r6;
		B(5,0) = (-2*Power(ax - bx,2)*r4*((2*a + b)*r1 + 4*a*r2*r6 - 4*a*r3*r6))/r6;
		bary2[i*3 + 2] += (k*B) / sqrt_c;
		}
	}

	// bary coords must sum to one
#ifndef QUADRIC_BOUNDARY_DERIV_FIGURE
	double sum = 0;
	for (int i = 0; i < n; i++)
		sum += bary[i];
	for (int i = 0; i < n; i++)
		bary[i] /= sum;
#endif
}